@@ -190,7 +190,7 @@ public final class OperationContainerImpl<Support> {
         }
         clearCache ();
         // handle eager modules
-        affectedEagers = new HashSet<OperationInfo<Support>> ();
+        
         if (type == OperationType.INSTALL || type == OperationType.UPDATE) {
             Collection<UpdateElement> all = new HashSet<UpdateElement> (operations.size ());
             Collection<ModuleInfo> allModuleInfos = new HashSet<ModuleInfo> (operations.size ());
@@ -214,11 +214,12 @@ public final class OperationContainerImpl<Support> {
                         reqs.add (req);
                     }
                 }
-                if (! reqs.isEmpty() && all.containsAll (reqs) && ! all.contains (eagerEl)) {
+                if ((! reqs.isEmpty() && all.containsAll (reqs) && ! all.contains (eagerEl)) ||
+                        (reqs.isEmpty() && impl.getUpdateUnit().getInstalled()!=null && type == OperationType.UPDATE && operations.size() > 0)) {
                     // adds affectedEager into list of elements for the operation
                     OperationInfo<Support> i = add (eagerEl.getUpdateUnit (), eagerEl);
                     if (i != null) {
-                        affectedEagers.add (i);
+                        affectedEagers.add (i);                    
                     }
                 }
             }
@@ -297,6 +298,7 @@ public final class OperationContainerImpl<Support> {
         synchronized(this) {
             changeState (true);
             operations.clear ();
+            affectedEagers.clear ();
         }
     }
     
