@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -24,7 +24,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -50,17 +50,16 @@ import org.netbeans.api.autoupdate.DefaultTestCase;
 import org.netbeans.api.autoupdate.InstallSupport;
 import org.netbeans.api.autoupdate.OperationContainer;
 import org.netbeans.api.autoupdate.OperationContainer.OperationInfo;
+import org.netbeans.api.autoupdate.OperationException;
 import org.netbeans.api.autoupdate.OperationSupport;
 import org.netbeans.api.autoupdate.UpdateElement;
 import org.netbeans.api.autoupdate.UpdateManager;
 import org.netbeans.api.autoupdate.UpdateUnit;
-import org.netbeans.api.autoupdate.UpdateUnitProviderFactory;
 import org.openide.filesystems.FileChangeAdapter;
 import org.openide.filesystems.FileChangeListener;
 import org.openide.filesystems.FileEvent;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileUtil;
-import org.openide.filesystems.Repository;
 import org.openide.modules.ModuleInfo;
 
 /**
@@ -69,9 +68,9 @@ import org.openide.modules.ModuleInfo;
  */
 public abstract class OperationsTestImpl extends DefaultTestCase {
     //{fileDataCreated, fileDeleted}
-    private Boolean[] fileChanges = {false,false};
-    private Thread[] fileChangeThreads = {null,null};
-    private Exception[] exceptions = {null,null};
+    private Boolean[] fileChanges = {false, false, false};
+    private Thread[] fileChangeThreads = {null,null,null};
+    private Exception[] exceptions = {null,null,null};
     
     private FileChangeListener fca;
     private FileObject modulesRoot;
@@ -84,8 +83,8 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
     protected void setUp () throws Exception {
         super.setUp ();
         getModuleInfos ();
-        Repository.getDefault ().getDefaultFileSystem ().refresh (false);
-        modulesRoot = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules"); // NOI18N
+        FileUtil.getConfigRoot().getFileSystem().refresh (false);
+        modulesRoot = FileUtil.getConfigFile("Modules"); // NOI18N
         fca = new FileChangeAdapter (){
             @Override
             public void fileDataCreated (FileEvent fe) {
@@ -93,6 +92,14 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
                 fileChangeThreads[0] = Thread.currentThread ();
                 exceptions[0] = new Exception ();
             }
+
+            @Override
+            public void fileChanged(FileEvent fe) {
+                fileChanges[2] = true;
+                fileChangeThreads[2] = Thread.currentThread ();
+                exceptions[2] = new Exception ();
+            }
+
             
             @Override
             public void fileDeleted (FileEvent fe) {
@@ -102,6 +109,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
             }
         };
         modulesRoot.addFileChangeListener (fca);
+        modulesRoot.getChildren();
     }
     
     @Override
@@ -151,6 +159,10 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         
         return installElement;
     }
+
+    boolean incrementNumberOfModuleConfigFiles() {
+        return true;
+    }
     
     /*public void installModuleDirect(UpdateUnit toInstall) throws Exception {
         installModuleImpl(toInstall, false);
@@ -158,7 +170,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
     
     
     private UpdateElement installModuleImpl (UpdateUnit toInstall, UpdateElement installElement, final boolean installSupport) throws Exception {
-        fileChanges = new Boolean[]{false,false};
+        fileChanges = new Boolean[]{false, false, false};
         installElement = (installElement != null) ? installElement : toInstall.getAvailableUpdates ().get (0);
         File f = InstallManager.findTargetDirectory(installElement.getUpdateUnit ().getInstalled (), Trampoline.API.impl(installElement),false);
         File configModules = new File (f, "config/Modules");
@@ -166,7 +178,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         int configModulesSize = (configModules.listFiles () != null) ? configModules.listFiles ().length : 0;
         int modulesSize = (modules.listFiles () != null) ? modules.listFiles ().length : 0;
         assertFalse (fileChanges[0]);
-        FileObject foConfigModules = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject foConfigModules = FileUtil.getConfigFile("Modules");
         assertNotNull (foConfigModules);
         int foConfigModulesSize = foConfigModules.getChildren ().length;
         assertNull (getModuleInfos ().get (toInstall.getCodeName ()));
@@ -201,10 +213,28 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
             assertFalse (support.isTrusted (i, installElement));
             assertFalse (support.isSigned (i, installElement));
             if (hiddenUpdate) {
-                r = support.doInstall (i, null);
+                try {
+                    r = support.doInstall (i, null);
+                } catch (OperationException ex) {
+                    if (OperationException.ERROR_TYPE.INSTALL == ex.getErrorType ()) {
+                        // can ingore
+                        // module system cannot load the module either
+                    } else {
+                        fail (ex.toString ());
+                    }
+                }
                 assertNotNull ("If there is hiddenUpdate then returns Restarer.", r);
             } else {
-                assertNull ("No Restarer when no hidden update.", support.doInstall (i, null));
+                try {
+                    assertNull ("No Restarer when no hidden update.", support.doInstall (i, null));
+                } catch (OperationException ex) {
+                    if (OperationException.ERROR_TYPE.INSTALL == ex.getErrorType ()) {
+                        // can ingore
+                        // module system cannot load the module either
+                    } else {
+                        fail (ex.toString ());
+                    }
+                }
             }
         } else {
             OperationContainer<OperationSupport> container = OperationContainer.createForDirectInstall ();
@@ -225,10 +255,16 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         if (r == null) {
             assertTrue ("Config module files are more than before Install test, " + Arrays.asList (configModules.listFiles ()), configModules.listFiles ().length > configModulesSize);
             assertTrue ("Installed modules are more than before Install test, " + Arrays.asList (modules.listFiles ()), modules.listFiles ().length > modulesSize);
-            assertTrue (foConfigModules.getPath (), foConfigModules.getChildren ().length > foConfigModulesSize);
-            assertEquals (configModules.listFiles ()[0], FileUtil.toFile (foConfigModules.getChildren ()[0]));
-            
-            assertTrue (fileChanges[0]);
+            if (incrementNumberOfModuleConfigFiles()) {
+                assertTrue (foConfigModules.getPath (), foConfigModules.getChildren ().length > foConfigModulesSize);
+                assertEquals (configModules.listFiles ()[0], FileUtil.toFile (foConfigModules.getChildren ()[0]));
+            }
+
+            if (incrementNumberOfModuleConfigFiles()) {
+                assertTrue (fileChanges[0]);
+            } else {
+                assertTrue(fileChanges[2]);
+            }
             fileChanges[0]=false;
         }
         
@@ -241,6 +277,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         
         fileChangeThreads[0]=null;
         //if (! customInstaller) Thread.sleep(3000);
+        @SuppressWarnings("unchecked")
         List<OperationContainer.OperationInfo> all = container2.listAll ();
         for (OperationContainer.OperationInfo oi : all) {
             UpdateUnit toInstallUnit = oi.getUpdateUnit ();
@@ -258,9 +295,13 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         return installElement;
     }
     
-    private void assertInstalledModule (UpdateUnit toInstallUnit) {
+    private void assertInstalledModule (UpdateUnit toInstallUnit) throws InterruptedException {
         ModuleInfo info = getModuleInfos ().get (toInstallUnit.getCodeName ());
         assertNotNull (info);
+        int timeout = 250;
+        while (! info.isEnabled () && timeout-- > 0) {
+            Thread.sleep (10);
+        }
         assertTrue (info.getCodeNameBase (), info.isEnabled ());
         assertNotNull (Utilities.toModule (toInstallUnit.getCodeName (), null));
         assertTrue (Utilities.toModule (toInstallUnit.getCodeName (), null).isEnabled ());
@@ -278,7 +319,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         File configModules = new File (getWorkDir (), "config/Modules");
         File modules = new File (getWorkDir (), "modules");
         assertFalse (fileChanges[0]);
-        FileObject foConfigModules = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject foConfigModules = FileUtil.getConfigFile("Modules");
         assertNotNull (foConfigModules);
         assertTrue (configModules.listFiles () != null && configModules.listFiles ().length != 0);
         assertTrue (modules.listFiles () != null && modules.listFiles ().length != 0);
@@ -307,7 +348,17 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
             //assertNotNull(support.getCertificate(i, upEl));
             assertFalse (support.isTrusted (i, upEl));
             assertFalse (support.isSigned (i, upEl));
-            OperationSupport.Restarter r = support.doInstall (i, null);
+            OperationSupport.Restarter r = null;
+            try {
+                r = support.doInstall (i, null);
+            } catch (OperationException ex) {
+                if (OperationException.ERROR_TYPE.INSTALL == ex.getErrorType ()) {
+                    // can ingore
+                    // module system cannot load the module either
+                } else {
+                    fail (ex.toString ());
+                }
+            }
             if (r != null) {
                 support.doRestartLater (r);
             }
@@ -334,6 +385,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         }
         
         
+        @SuppressWarnings("unchecked")
         List<OperationContainer.OperationInfo> all = container2.listAll ();
         for (OperationContainer.OperationInfo oi : all) {
             UpdateUnit toUpdateUnit = oi.getUpdateUnit ();
@@ -349,7 +401,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
     }
     
     void disableModule (UpdateUnit toDisable) throws Exception {
-        FileObject fo = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject fo = FileUtil.getConfigFile("Modules");
         File f = new File (getWorkDir (), "config/Modules");
         File f2 = new File (getWorkDir (), "modules");
         assertTrue (f.listFiles () != null && f.listFiles ().length != 0);
@@ -382,7 +434,7 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
     }
     
     void enableModule (UpdateUnit toEnable) throws Exception {
-        FileObject fo = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject fo = FileUtil.getConfigFile("Modules");
         File f = new File (getWorkDir (), "config/Modules");
         File f2 = new File (getWorkDir (), "modules");
         assertTrue (f.listFiles () != null && f.listFiles ().length != 0);
@@ -421,12 +473,12 @@ public abstract class OperationsTestImpl extends DefaultTestCase {
         File modules = new File (getWorkDir (), "modules");
         int configModulesSize = (configModules.listFiles () != null) ? configModules.listFiles ().length : 0;
         int modulesSize = (modules.listFiles () != null) ? modules.listFiles ().length : 0;
-        FileObject foConfigModules = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject foConfigModules = FileUtil.getConfigFile("Modules");
         assertNotNull (foConfigModules);
         int foConfigModulesSize = foConfigModules.getChildren ().length;
         
         assertFalse (fileChanges[1]);
-        FileObject fo = Repository.getDefault ().getDefaultFileSystem ().findResource ("Modules");
+        FileObject fo = FileUtil.getConfigFile("Modules");
         assertNotNull (fo);
         assertTrue (fo.getChildren ().length > 0);
         assertNotNull (getModuleInfos ().get (toUnInstall.getCodeName ()));
