@@ -92,10 +92,35 @@ public class PluginManagerUI extends javax.swing.JPanel  {
     public static final int INDEX_OF_DOWNLOAD_TAB = 2;
     public static final int INDEX_OF_INSTALLED_TAB = 3;
     public static final int INDEX_OF_SETTINGS_TAB = 4;
+
+    public static final String[] TAB_NAMES = { "update", "available", "local", "installed" }; //NOI18N
+    private int initialTabToSelect;
+    private boolean detailView;
+    public static final String DETAIL_VIEW_SELECTED_PROP = "plugin.manager.detail.view.selected";//NOI18N
+    
+    public PluginManagerUI (JButton closeButton ) {
+        this(closeButton, null, true);
+    }
+
+    public PluginManagerUI (JButton closeButton, Object initialTab) {
+        this(closeButton, initialTab, Boolean.getBoolean(DETAIL_VIEW_SELECTED_PROP));
+    }
     
     /** Creates new form PluginManagerUI */
-    public PluginManagerUI (JButton closeButton) {
+    public PluginManagerUI (JButton closeButton, Object initialTab, boolean detailView) {
+        this.detailView = detailView;
         this.closeButton = closeButton;
+        int selIndex = -1;
+        for( int i=0; i<TAB_NAMES.length; i++ ) {
+            if( TAB_NAMES[i].equals(initialTab) ) {
+                selIndex = i;
+                break;
+            }
+        }
+        if( selIndex < 0 && null != initialTab ) {
+            throw new IllegalArgumentException("Invalid tab name: " + initialTab); //NOI18N
+        }
+        initialTabToSelect = selIndex;
         initComponents ();
         postInitComponents ();
         //start initialize method as soon as possible
@@ -105,7 +130,15 @@ public class PluginManagerUI extends javax.swing.JPanel  {
             }
         });
     }
-    
+
+    boolean isDetailView() {
+        return detailView;
+    }
+
+    void setDetailView(boolean detailView) {
+        this.detailView = detailView;
+    }
+
     private Window findWindowParent () {
         Component c = this;
         while(c != null) {
@@ -285,6 +318,19 @@ public class PluginManagerUI extends javax.swing.JPanel  {
     }
 
     private void setSelectedTab() {
+        if( initialTabToSelect >= 0 && initialTabToSelect != tpTabs.getSelectedIndex()
+                && initialTabToSelect < tpTabs.getComponentCount() ) {
+            Component c = tpTabs.getComponentAt(initialTabToSelect);
+            if (c instanceof UnitTab) {
+                UnitTab unitTab = (UnitTab) c;
+                if (unitTab.getModel().isTabEnabled() && unitTab.getModel().canBePrimaryTab() ) {
+                    tpTabs.setSelectedIndex(initialTabToSelect);
+                    initialTabToSelect = -1;
+                    return;
+                }
+            }
+        }
+        initialTabToSelect = -1;
         Component component = tpTabs.getSelectedComponent();
         if (component instanceof UnitTab) {
             UnitTab unitTab = (UnitTab) component;
@@ -299,7 +345,7 @@ public class PluginManagerUI extends javax.swing.JPanel  {
                         }
                     } else {
                         tpTabs.setSelectedIndex(i);
-                        break;                        
+                        break;
                     }
                 }
             }
@@ -328,7 +374,6 @@ public class PluginManagerUI extends javax.swing.JPanel  {
         bClose = closeButton;
         bHelp = new javax.swing.JButton();
 
-        tpTabs.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
         tpTabs.addChangeListener(new javax.swing.event.ChangeListener() {
             public void stateChanged(javax.swing.event.ChangeEvent evt) {
                 tpTabsStateChanged(evt);
@@ -355,11 +400,11 @@ public class PluginManagerUI extends javax.swing.JPanel  {
                 .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                     .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                         .add(pProgress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 562, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
-                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 123, Short.MAX_VALUE)
+                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 157, Short.MAX_VALUE)
                         .add(bClose)
                         .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                         .add(bHelp))
-                    .add(tpTabs, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 813, Short.MAX_VALUE))
+                    .add(tpTabs, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 841, Short.MAX_VALUE))
                 .addContainerGap())
         );
         layout.setVerticalGroup(
@@ -535,13 +580,18 @@ private void bHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:e
             //which is synchronized and may wait until cache is created
             //even more AutoUpdateCatalog.getUpdateItems () can at first start call refresh and thus writeToCache again
             units = UpdateManager.getDefault().getUpdateUnits(Utilities.getUnitTypes());
-            UnitCategoryTableModel installTableModel = ((UnitCategoryTableModel)installedTable.getModel());
+            InstalledTableModel installTableModel = (InstalledTableModel)installedTable.getModel();
             UnitCategoryTableModel updateTableModel = ((UnitCategoryTableModel)updateTable.getModel());
             UnitCategoryTableModel availableTableModel = ((UnitCategoryTableModel)availableTable.getModel());
             LocallyDownloadedTableModel localTableModel = ((LocallyDownloadedTableModel)localTable.getModel());
             
             updateTableModel.setUnits(units);
-            installTableModel.setUnits(units);
+            List<UpdateUnit> features = UpdateManager.getDefault().getUpdateUnits(UpdateManager.TYPE.FEATURE);
+            if (isDetailView() && !features.isEmpty()) {
+                installTableModel.setUnits(units);
+            } else {
+                installTableModel.setUnits(units, features);
+            }
             availableTableModel.setUnits(units);
             localTableModel.setUnits(units);
             selectFirstRow(installedTable);
@@ -568,7 +618,7 @@ private void bHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:e
     }
 
     public static void registerRunningTask (RequestProcessor.Task it) {
-        assert runningTask == null || runningTask.isFinished () : "Only once task can be running.";
+        assert runningTask == null || runningTask.isFinished () : "Only once task can be running. Already running : " + runningTask;
         runningTask = it;
     }
     
