@@ -61,7 +61,7 @@ import java.util.prefs.BackingStoreException;
 import java.util.prefs.Preferences;
 import org.netbeans.modules.autoupdate.ui.Utilities;
 import org.openide.filesystems.FileObject;
-import org.openide.filesystems.Repository;
+import org.openide.filesystems.FileUtil;
 import org.openide.util.NbPreferences;
 
 /**
@@ -89,7 +89,8 @@ public class AutoupdateSettings {
     public static final int EVERY_MONTH = 4;
     public static final int NEVER = 5;
     public static final int CUSTOM_CHECK_INTERVAL = 6;
-    private static final String EXPIRATION_RECORD = "expiration";
+    private static final String EXPIRATION_RECORD = "expiration"; // NOI18N
+    private static final String IMPORTED = "imported"; // NOI18N
     private static final String [] VERSIONS_FOR_IMPORT = new String[0];
     
     private static final String [][] KNOWN = {
@@ -195,7 +196,7 @@ public class AutoupdateSettings {
         err.log (Level.FINER, "Old IDE Identity ID: " + id); // NOI18N
         String newPrefix = "";
         try {
-            FileObject fo = Repository.getDefault().getDefaultFileSystem().findResource("/productid"); // NOI18N
+            FileObject fo = FileUtil.getConfigFile("productid"); // NOI18N
             if (fo != null) {
                 InputStream is = fo.getInputStream();
                 try {
@@ -258,13 +259,35 @@ public class AutoupdateSettings {
         Collection<String> forImport = new HashSet<String> (Arrays.asList (VERSIONS_FOR_IMPORT));
         String currentVersion = new File (System.getProperty ("netbeans.user")).getName ();
         forImport.add (currentVersion);
-        if (exp == null || ! forImport.contains (exp)) {
+        if (exp != null && ! forImport.contains (exp)) {
             try {
+                final int period = getPeriod();
+                boolean shared = Utilities.isGlobalInstallation();
                 p.removeNode ();
+                getPreferences ().put (IMPORTED, Boolean.toString (true));
+                getPreferences ().putInt(PROP_PERIOD, period);
+                if (shared) {
+                    // Check that we can use 'shared' flag imported from previous installation with a new one.
+                    // Obtained from SettingsTab.cbGlobalInstallActionPerformed
+                    Collection<File> dirs = Utilities.sharedDirs();
+                    if (!dirs.isEmpty()) {
+                        for (File f : dirs) {
+                            if (f.exists() && f.isDirectory() && !Utilities.canWriteInCluster(f)) {
+                                shared = false;
+                                break;
+                            }
+                        }
+                        if (shared) {
+                            Utilities.setGlobalInstallation(shared);
+                        }
+                    }
+                }
                 err.log (Level.FINE, "Don't read preferences from userdir " + exp);
             } catch (BackingStoreException ex) {
                 err.log (Level.INFO, ex.getLocalizedMessage (), ex);
             }
+        } else if (exp == null) {
+            err.log (Level.FINEST, "No preferences imported from previous versoin.");
         } else {
             err.log (Level.FINEST, "Read preferences from userdir " + exp);
         }
@@ -335,7 +358,9 @@ public class AutoupdateSettings {
                 err.log (Level.FINER, null, ex);
             } finally {
                 try {
-                    os.close ();
+                    if(os!=null) {
+                        os.close ();
+                    }
                 } catch (IOException ex) {
                     // let's ignore it
                     err.log (Level.FINER, null, ex);
