@@ -109,9 +109,11 @@ class ExtPlainView extends PlainView {
             if (!getText(p0, p1 - p0, s)) {
                 return x;
             }
-            g.setColor(getColorForLocation(p0, doc, true));
+            Lines lines = ((OutputDocument) doc).getLines();
+            int line = lines.getLineAt(p0);
+            g.setColor(lines.getColorForLine(line));
             int ret = Utilities.drawTabbedText(s, x, y, g, this, p0);
-            if (g.getColor() == WrappedTextView.selectedLinkFg || g.getColor() == WrappedTextView.selectedImportantLinkFg) {
+            if (lines.isHyperlink(line)) {
                 //#47263 - start hyperlink underline at first
                 //non-whitespace character
                 underline(g, s, x, p0, y);
@@ -132,9 +134,11 @@ class ExtPlainView extends PlainView {
             if (!getText(p0, p1 - p0, s)) {
                 return x;
             }
-            g.setColor(getColorForLocation(p0, doc, false));
+            Lines lines = ((OutputDocument) doc).getLines();
+            int line = lines.getLineAt(p0);
+            g.setColor(lines.getColorForLine(line));
             int ret = Utilities.drawTabbedText(s, x, y, g, this, p0);
-            if (g.getColor() == WrappedTextView.selectedLinkFg || g.getColor() == WrappedTextView.selectedImportantLinkFg) {
+            if (lines.isHyperlink(line)) {
                 //#47263 - start hyperlink underline at first
                 //non-whitespace character
                 underline(g, s, x, p0, y);
@@ -177,63 +181,23 @@ class ExtPlainView extends PlainView {
     }
 
     private void underline(Graphics g, Segment s, int x, int p0, int y) {
-        int wid = g.getFontMetrics().charWidth(' '); //NOI18N
         char[] txt = s.array;
-        int txtOffset = s.offset;
-        int txtCount = s.count;
         int n = s.offset + s.count;
-        int tabCount = 0;
         int wsCount = 0;
         for (int i = s.offset; i < n; i++) {
-            if (txt[i] == '\t') { //NOI18N
-                x = (int) nextTabStop((float) x,
-                p0 + i - txtOffset);
-                tabCount++;
-            } else if (Character.isWhitespace(txt[i])) {
-                x += wid;
+            if (Character.isWhitespace(txt[i])) {
                 wsCount++;
             } else {
                 break;
             }
         }
-        int end = x + (wid * (txtCount - (wsCount + tabCount + 1)));
-        if (end > x) {
-            g.drawLine (x, y+1, end, y+1);
-        }
-    }
 
-    private static Color getColorForLocation (int start, Document d, boolean selected) {
-        OutputDocument od = (OutputDocument) d;
-        int line = od.getElementIndex (start);
-        boolean hyperlink = od.getLines().isHyperlink(line);
-        boolean important = hyperlink ? od.getLines().isImportantHyperlink(line) : false;
-        boolean isErr = od.getLines().isErr(line);
-        
-        return hyperlink ? 
-            (important ? 
-                (selected ? 
-                    WrappedTextView.selectedImportantLinkFg : 
-                    WrappedTextView.unselectedImportantLinkFg) :
-                (selected ?
-                    WrappedTextView.selectedLinkFg : 
-                    WrappedTextView.unselectedLinkFg)) :
-            (selected ? 
-                (isErr ? 
-                    WrappedTextView.selectedErr : 
-                    WrappedTextView.selectedFg) : 
-                (isErr ? 
-                    WrappedTextView.unselectedErr : 
-                    WrappedTextView.unselectedFg));
-    }
-
-    @Override
-    protected void damageLineRange(int line0, int line1, Shape a, Component host) {
-        super.damageLineRange(line0, line1, a, host);
-    }
-
-    @Override
-    protected void drawLine(int lineIndex, Graphics g, int x, int y) {
-        super.drawLine(lineIndex, g, x, y);
+        int lineLen = Utilities.getTabbedTextWidth(s, metrics, tabBase, this, p0);
+        s.offset += wsCount;
+        s.count -= wsCount;
+        int textLen = Utilities.getTabbedTextWidth(s, metrics, tabBase, this, p0 + wsCount);
+        int underlineShift = (int) g.getFontMetrics().getDescent() - 1;
+        g.drawLine(x + lineLen - textLen, y + underlineShift, x + lineLen, y + underlineShift);
     }
 
     @Override
@@ -311,11 +275,6 @@ class ExtPlainView extends PlainView {
         }
     }
 
-    @Override
-    public int getEndOffset() {
-        return super.getEndOffset();
-    }
-
     @Override
     public float getPreferredSpan(int axis) {
         if (axis == Y_AXIS) {
@@ -349,13 +308,12 @@ class ExtPlainView extends PlainView {
 
     @Override
     protected void updateMetrics() {
-	Component host = getContainer();
-	Font f = host.getFont();        
-	if (font != f) {
-	    // The font changed, we need to recalculate the longest line.
-	    calcLongestLineLength();
-	    //tabSize = getTabSize() * metrics.charWidth('m');
-	}
+        Component host = getContainer();
+        Font f = host.getFont();
+        if (font != f) {
+            // The font changed, we need to recalculate the longest line.
+            calcLongestLineLength();
+        }
     }
     
     /**
