@@ -41,9 +41,7 @@
 
 package org.netbeans.core.output2.ui;
 
-import java.awt.Rectangle;
 import javax.swing.plaf.TextUI;
-
 import javax.swing.*;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
@@ -52,6 +50,7 @@ import javax.swing.event.DocumentListener;
 import javax.swing.text.*;
 import java.awt.*;
 import java.awt.event.*;
+import org.netbeans.core.output2.Controller;
 import org.netbeans.core.output2.OutputDocument;
 import org.openide.util.Exceptions;
 
@@ -104,18 +103,29 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     public boolean requestFocusInWindow() {
         return textView.requestFocusInWindow();
     }
-    
+
+    public Font getViewFont() {
+        return textView.getFont();
+    }
+
+    public void setViewFont (Font f) {
+        fontWidth = -1;
+        fontHeight = -1;
+        textView.setFont(f);
+    }
+
     protected abstract JEditorPane createTextView();
 
     protected void documentChanged() {
         lastLength = -1;
-        if (pendingCaretLine != -1) {
-            if (!sendCaretToLine (pendingCaretLine, pendingCaretSelect)) {
-                ensureCaretPosition();
+        if (lineToScroll != -1) {
+            if (scrollToLine(lineToScroll)) {
+                lineToScroll = -1;
             }
         } else {
             ensureCaretPosition();
         }
+
         if (recentlyReset && isShowing()) {
             recentlyReset = false;
         }
@@ -136,7 +146,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         return textView.getSelectionStart() != textView.getSelectionEnd();
     }
 
-    boolean isScrollLocked() {
+    public boolean isScrollLocked() {
         return locked;
     }
 
@@ -148,13 +158,11 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
      * OutputDocument only fires changes on the event queue.
      */
     public final void ensureCaretPosition() {
-        if (locked) {           
+        if (locked && !enqueued) {
             //Make sure the scrollbar is updated *after* the document change
             //has been processed and the scrollbar model's maximum updated
-            if (!enqueued) {
-                enqueued = true;
-                SwingUtilities.invokeLater(this);
-            }
+            enqueued = true;
+            SwingUtilities.invokeLater(this);
         }
     }
     
@@ -166,8 +174,10 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
      */
     public void run() {
         enqueued = false;
-        getVerticalScrollBar().setValue(getVerticalScrollBar().getModel().getMaximum());
-        getHorizontalScrollBar().setValue(getHorizontalScrollBar().getModel().getMinimum());
+        if (locked) {
+            getVerticalScrollBar().setValue(getVerticalScrollBar().getModel().getMaximum());
+            getHorizontalScrollBar().setValue(getHorizontalScrollBar().getModel().getMinimum());
+        }
     }
 
     public int getSelectionStart() {
@@ -178,6 +188,20 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         return textView.getSelectionEnd();
     }
 
+    public String getSelectedText() {
+        int start = getSelectionStart();
+        int end = getSelectionEnd();
+        String str = null;
+        if (start > 0 && end > start) {
+            try {
+                str = getDocument().getText(start, end - start);
+            } catch (BadLocationException ex) {
+                ex.printStackTrace();
+            }
+        }
+        return str;
+    }
+
     public void setSelection (int start, int end) {
         int rstart = Math.min (start, end);
         int rend = Math.max (start, end);
@@ -225,15 +249,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         addMouseListener(this);
 
         getCaret().addChangeListener(this);
-        Integer i = (Integer) UIManager.get("customFontSize"); //NOI18N
-        int size;
-        if (i != null) {
-            size = i.intValue();
-        } else {
-            Font f = (Font) UIManager.get("controlFont");
-            size = f != null ? f.getSize() : 11;
-        }
-        textView.setFont (new Font ("Monospaced", Font.PLAIN, size)); //NOI18N
+        textView.setFont(isWrapped() ? Controller.getDefault().getCurrentFontMS() : Controller.getDefault().getCurrentFont()); //NOI18N
         setBorder (BorderFactory.createEmptyBorder());
         setViewportBorder (BorderFactory.createEmptyBorder());
         
@@ -269,6 +285,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         hadSelection = false;
         lastCaretLine = 0;
         lastLength = -1;
+        lineToScroll = -1;
         Document old = textView.getDocument();
         old.removeDocumentListener(this);
         if (doc != null) {
@@ -276,7 +293,6 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
             doc.addDocumentListener(this);
             lockScroll();
             recentlyReset = true;
-            pendingCaretLine = -1;
         } else {
             textView.setDocument (new PlainDocument());
             textView.setEditorKit(new DefaultEditorKit());
@@ -315,53 +331,66 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         }
         return lastLength;
     }
-    
-    /**
-     * If we are sending the caret to a hyperlinked line, but it is < 3 lines
-     * from the bottom, we will hold the line number in this field until there
-     * are enough lines that it will be semi-centered.
-     */
-    private int pendingCaretLine = -1;
-    private boolean pendingCaretSelect = false;
-    private boolean inSendCaretToLine = false;
-    
-    public final boolean sendCaretToLine(int idx, boolean select) {
-        int count = getLineCount();
-        if (count - idx < 3) {
-            pendingCaretLine = idx;
-            pendingCaretSelect = select;
-            return false;
-        } else {
-            inSendCaretToLine = true;
-            pendingCaretLine = -1;
-            unlockScroll();
-            getCaret().setVisible(true);
-            getCaret().setSelectionVisible(true);
-            Element el = textView.getDocument().getDefaultRootElement().getElement(Math.min(idx, getLineCount() - 1));
-            int position = el.getStartOffset();
-            if (select) {
-                getCaret().setDot (el.getEndOffset()-1);
-                getCaret().moveDot (position);
-                getCaret().setSelectionVisible(true);
-                textView.repaint();
-            } else {
-                getCaret().setDot(position);
-            }
-            if (idx + 3 < getLineCount()) {
-                try {
-                    Rectangle r = textView.modelToView(textView.getDocument().getDefaultRootElement().getElement(idx + 3).getStartOffset());
-                    if (r != null) { //Will be null if maximized - no parent, no coordinate space
-                        textView.scrollRectToVisible(r);
-                    }
-                } catch (BadLocationException ble) {
-                    Exceptions.printStackTrace(ble);
-                }
-            }
-            inSendCaretToLine = false;
-            return true;
+
+    public void scrollTo(int pos) {
+        getCaret().setDot(pos);
+        try {
+            Rectangle rect = textView.modelToView(pos);
+            textView.scrollRectToVisible(rect);
+            locked = false;
+        } catch (BadLocationException ex) {
         }
     }
 
+    private boolean inSendCaretToLine = false;
+    private int lineToScroll = -1;
+    public final boolean sendCaretToLine(int idx, boolean select) {
+        int lastLine = getLineCount() - 1;
+        if (idx > lastLine) {
+            idx = lastLine;
+        }
+        inSendCaretToLine = true;
+        getCaret().setVisible(true);
+        getCaret().setSelectionVisible(true);
+        Element el = textView.getDocument().getDefaultRootElement().getElement(idx);
+        int position = el.getStartOffset();
+        if (select) {
+            getCaret().setDot(el.getEndOffset() - 1);
+            getCaret().moveDot(position);
+            getCaret().setSelectionVisible(true);
+            textView.repaint();
+        } else {
+            getCaret().setDot(position);
+        }
+
+        if (!scrollToLine(idx + 3) && isScrollLocked()) {
+            lineToScroll = idx + 3;
+        }
+        locked = false;
+        inSendCaretToLine = false;
+        return true;
+    }
+    
+    boolean scrollToLine(int line) {
+        int lineIdx = Math.min(getLineCount() - 1, line);
+        Rectangle rect = null;
+        try {
+            rect = textView.modelToView(textView.getDocument().getDefaultRootElement().getElement(lineIdx).getStartOffset());
+        } catch (BadLocationException ex) {
+            Exceptions.printStackTrace(ex);
+        }
+
+        if (rect == null) {
+            return false;
+        }
+
+        boolean oldLocked = locked;
+        textView.scrollRectToVisible(rect);
+        locked = oldLocked;
+
+        Rectangle visRect = textView.getVisibleRect();
+        return line == lineIdx && visRect.y + visRect.height == rect.y + rect.height;
+    }
 
     public final void lockScroll() {
         if (!locked) {
@@ -373,6 +402,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         if (locked) {
             locked = false;
         }
+        lineToScroll = -1;
     }
 
     protected abstract void caretEnteredLine (int line);
@@ -382,7 +412,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     protected abstract void postPopupMenu (Point p, Component src);
     
     public final int getCaretLine() {
-        int result = -1;
+        int result = 0;
         int charPos = getCaret().getDot();
         if (charPos > 0) {
             result = textView.getDocument().getDefaultRootElement().getElementIndex(charPos);
@@ -390,6 +420,11 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         return result;
     }
 
+    public final boolean isLineSelected(int idx) {
+        Element line = textView.getDocument().getDefaultRootElement().getElement(idx);
+        return line.getStartOffset() == getSelectionStart() && line.getEndOffset()-1 == getSelectionEnd();
+    }
+
     public final int getCaretPos() {
         return getCaret().getDot();
     }
@@ -399,6 +434,7 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         if (fontHeight == -1) {
             fontHeight = g.getFontMetrics(textView.getFont()).getHeight();
             fontWidth = g.getFontMetrics(textView.getFont()).charWidth('m'); //NOI18N
+            getVerticalScrollBar().setUnitIncrement(fontHeight);
         }
         super.paint(g);
     }
@@ -406,11 +442,6 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
 //***********************Listener implementations*****************************
 
     public void stateChanged(ChangeEvent e) {
-        /*if (e.getSource() instanceof JViewport) { // #78191
-            if (locked) {
-                ensureCaretPosition();
-            }
-        } else*/
         if (e.getSource() == getVerticalScrollBar().getModel()) {
             if (!locked) { //XXX check if doc is still being written?
                 BoundedRangeModel mdl = getVerticalScrollBar().getModel();
@@ -449,9 +480,8 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
         //it can cause the controller to send events to OutputListeners which
         //should only happen for user events
         if ((!locked && caretLineChanged()) && !inSendCaretToLine) {
-            int line = getCaretLine();
             boolean sel = textView.getSelectionStart() != textView.getSelectionEnd();
-            if (line != -1 && !sel) {
+            if (!sel) {
                 caretEnteredLine(getCaretLine());
             }
             if (sel != hadSelection) {
@@ -565,8 +595,14 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
             }
         }
     }
-    
+
+    /** last pressed position for hyperlink test */
+    private int lastPressedPos = -1;
+
     public void mousePressed(MouseEvent e) {
+        if (e.getSource() == textView && SwingUtilities.isLeftMouseButton(e)) {
+            lastPressedPos = textView.viewToModel(e.getPoint());
+        }
         if (locked && !e.isPopupTrigger()) {
             Element el = getDocument().getDefaultRootElement().getElement(getLineCount()-1);
             getCaret().setDot(el.getStartOffset());
@@ -591,13 +627,14 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     public final void mouseReleased(MouseEvent e) {
         if (e.getSource() == textView && SwingUtilities.isLeftMouseButton(e)) {
             int pos = textView.viewToModel(e.getPoint());
-            if (pos != -1) {
+            if (pos != -1 && pos == lastPressedPos) {
                 int line = textView.getDocument().getDefaultRootElement().getElementIndex(pos);
                 if (line >= 0) {
                     lineClicked(line, e.getPoint());
                     e.consume(); //do NOT allow this window's caret to steal the focus from editor window
                 }
             }
+            lastPressedPos = -1;
         }
         if (e.isPopupTrigger()) {
             Point p = SwingUtilities.convertPoint((Component) e.getSource(), 
@@ -613,7 +650,9 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     public void keyPressed(KeyEvent keyEvent) {
         switch (keyEvent.getKeyCode()) {
             case KeyEvent.VK_END:
-                lockScroll();
+                if (keyEvent.isControlDown()) {
+                    lockScroll();
+                }
                 break;
             case KeyEvent.VK_UP:
             case KeyEvent.VK_DOWN:
@@ -633,15 +672,22 @@ public abstract class AbstractOutputPane extends JScrollPane implements Document
     public void keyTyped(KeyEvent keyEvent) {
     }
 
+    protected abstract void changeFontSizeBy(int change);
+
     public final void mouseWheelMoved(MouseWheelEvent e) {
+        if (e.isControlDown()) {
+            int change = -e.getWheelRotation();
+            changeFontSizeBy(change);
+            e.consume();
+            return;
+        }
         BoundedRangeModel sbmodel = getVerticalScrollBar().getModel();
         int max = sbmodel.getMaximum();
         int range = sbmodel.getExtent();
 
         int currPosition = sbmodel.getValue();
         if (e.getSource() == textView) {
-            int newPosition = Math.max (0, Math.min (sbmodel.getMaximum(),
-                currPosition + (e.getUnitsToScroll() * textView.getFontMetrics(textView.getFont()).getHeight())));
+            int newPosition = Math.max(0, Math.min(sbmodel.getMaximum(), currPosition + (e.getUnitsToScroll() * fontHeight)));
             // height is a magic constant because of #57532
             sbmodel.setValue (newPosition);
             if (newPosition + range >= max) {
