@@ -44,6 +44,7 @@ package org.netbeans.core.startup;
 import java.beans.Introspector;
 import java.io.File;
 import java.io.IOException;
+import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.net.URL;
 import java.util.logging.Level;
@@ -55,19 +56,22 @@ import org.netbeans.Stamps;
 import org.netbeans.Util;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileStateInvalidException;
+import org.openide.filesystems.FileUtil;
 import org.openide.filesystems.Repository;
-import org.openide.modules.Dependency;
 import org.openide.modules.InstalledFileLocator;
-import org.openide.modules.SpecificationVersion;
 import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.NbBundle;
+import org.openide.util.UserCancelException;
 import org.openide.util.Utilities;
 
 /**
  * Main class for NetBeans when run in GUI mode.
  */
 public final class Main extends Object {
+
+    private static final Logger LOG = Logger.getLogger(Main.class.getName());
+
     /** module subsystem */
     private static ModuleSystem moduleSystem;
     /** module subsystem is fully ready */
@@ -78,7 +82,7 @@ public final class Main extends Object {
   public static void setStatusText (String msg) {
         Splash.getInstance().print (msg);
         if (moduleSystemInitialized) {
-            org.netbeans.core.startup.CoreBridge.conditionallyPrintStatus (msg);
+            CoreBridge.getDefault().setStatusText(msg);
         }
   }
   
@@ -114,8 +118,7 @@ public final class Main extends Object {
       try {
           if (wantTheme) {
               //Put a couple things into UIDefaults for the plaf library to process if it wants
-               FileObject fo =
-                    Repository.getDefault().getDefaultFileSystem().findResource("themes.xml"); //NOI18N
+               FileObject fo = FileUtil.getConfigFile("themes.xml"); //NOI18N
                if (fo == null) {            
                     // File on SFS failed --> try to load from a jar from path
                     // /org/netbeans/core/startup/resources/themes.xml
@@ -152,7 +155,7 @@ public final class Main extends Object {
 
             StartLog.logStart ("Modules initialization"); // NOI18N
             try {
-                moduleSystem = new ModuleSystem(Repository.getDefault().getDefaultFileSystem());
+                moduleSystem = new ModuleSystem(FileUtil.getConfigRoot().getFileSystem());
             } catch (IOException ioe) {
                 // System will be screwed up.
                 throw (IllegalStateException) new IllegalStateException("Module system cannot be created").initCause(ioe); // NOI18N
@@ -194,12 +197,6 @@ public final class Main extends Object {
     System.setProperty ("org.openide.version", "deprecated"); // NOI18N
     System.setProperty ("org.openide.major.version", "IDE/1"); // NOI18N
 
-    // Enforce JDK 1.5+ since we would not work without it.
-    if (Dependency.JAVA_SPEC.compareTo(new SpecificationVersion("1.5")) < 0) { // NOI18N
-        System.err.println("The IDE requires JDK 5 or higher to run."); // XXX I18N?
-        TopLogging.exit(1);
-    }
-
     // In the past we derived ${jdk.home} from ${java.home} by appending
     // "/.." to the end of ${java.home} assuming that JRE is under JDK
     // directory.  It does not always work.  On MacOS X JDK and JRE files
@@ -398,19 +395,19 @@ public final class Main extends Object {
                         new String[0]
                     });
                     executedOk = true;
-                } catch (java.lang.reflect.InvocationTargetException ex) {
+                } catch (InvocationTargetException ex) {
                     // canceled by user, all is fine
-                    if (ex.getTargetException () instanceof org.openide.util.UserCancelException) {
+                    if (ex.getTargetException() instanceof UserCancelException) {
                         executedOk = true;
                     } else {
-                        ex.printStackTrace();
+                        LOG.log(Level.WARNING, null, ex);
                     }
                 } catch (Exception e) {
                     // If exceptions are thrown, notify them - something is broken.
-                    e.printStackTrace();
+                    LOG.log(Level.WARNING, null, e);
                 } catch (LinkageError e) {
                     // These too...
-                    e.printStackTrace();
+                    LOG.log(Level.WARNING, null, e);
                 }
             }
             
@@ -430,12 +427,12 @@ public final class Main extends Object {
                         }
                     } catch (IOException ex) {
                         // file was not created a bit of problem but go on
-                        ex.printStackTrace();
+                        LOG.log(Level.WARNING, null, ex);
                         return true;
                     } catch (java.lang.reflect.InvocationTargetException ex) {
                         return false;
                     } catch (InterruptedException ex) {
-                        ex.printStackTrace();
+                        LOG.log(Level.WARNING, null, ex);
                         return false;
                     }
                 } else {
@@ -487,22 +484,22 @@ public final class Main extends Object {
                         try {
                             f.createNewFile();
                         } catch (IOException exc) {
-                            exc.printStackTrace();
+                            LOG.log(Level.WARNING, null, exc);
                         }
                     }
-                } catch (java.lang.reflect.InvocationTargetException ex) {
+                } catch (InvocationTargetException ex) {
                     // canceled by user, all is fine
-                    if (ex.getTargetException() instanceof org.openide.util.UserCancelException) {
+                    if (ex.getTargetException() instanceof UserCancelException) {
                         executedOk = false;
                     } else {
-                        ex.printStackTrace();
+                        LOG.log(Level.WARNING, null, ex);
                     }
                 } catch (Exception ex) {
                     // If exceptions are thrown, notify them - something is broken.
-                    ex.printStackTrace();
+                    LOG.log(Level.WARNING, null, ex);
                 } catch (LinkageError ex) {
                     // These too...
-                    ex.printStackTrace();
+                    LOG.log(Level.WARNING, null, ex);
                 }
             }
             
@@ -518,7 +515,7 @@ public final class Main extends Object {
                     } catch (java.lang.reflect.InvocationTargetException ex) {
                         return false;
                     } catch (InterruptedException ex) {
-                        ex.printStackTrace();
+                        LOG.log(Level.WARNING, null, ex);
                         return false;
                     }
                 } else {
