@@ -45,13 +45,17 @@ import java.io.ByteArrayInputStream;
 import java.io.DataOutputStream;
 import java.io.File;
 import java.io.IOException;
+import java.lang.reflect.Method;
 import java.net.URL;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
@@ -74,7 +78,7 @@ import org.netbeans.Util;
 import org.netbeans.core.startup.layers.ModuleLayeredFileSystem;
 import org.openide.filesystems.FileObject;
 import org.openide.filesystems.FileStateInvalidException;
-import org.openide.filesystems.Repository;
+import org.openide.filesystems.FileUtil;
 import org.openide.modules.Dependency;
 import org.openide.modules.ModuleInstall;
 import org.openide.modules.SpecificationVersion;
@@ -115,6 +119,8 @@ final class NbInstaller extends ModuleInstaller {
     private final Map<Module,Set<String>> kosherPackages = new HashMap<Module,Set<String>>(100);
     /** classpath ~ JRE packages to be hidden from a module */
     private final Map<Module,List<Module.PackageExport>> hiddenClasspathPackages = new  HashMap<Module,List<Module.PackageExport>>();
+    /** #164510: similar to {@link #hiddenClasspathPackages} but backwards for efficiency */
+    private final Map<Module.PackageExport,List<Module>> hiddenClasspathPackagesReverse = new HashMap<Module.PackageExport,List<Module>>();
         
     /** Create an NbInstaller.
      * You should also call {@link #registerManager} and if applicable
@@ -269,7 +275,16 @@ final class NbInstaller extends ModuleInstaller {
             }
         }
         if (!hiddenPackages.isEmpty()) {
-            hiddenClasspathPackages.put(m, hiddenPackages);
+            synchronized (hiddenClasspathPackages) {
+                hiddenClasspathPackages.put(m, hiddenPackages);
+                for (Module.PackageExport pkg : hiddenPackages) {
+                    List<Module> ms = hiddenClasspathPackagesReverse.get(pkg);
+                    if (ms == null) {
+                        hiddenClasspathPackagesReverse.put(pkg, ms = new LinkedList<Module>());
+                    }
+                    ms.add(m);
+                }
+            }
         }
     }
     
@@ -285,7 +300,13 @@ final class NbInstaller extends ModuleInstaller {
         installs.remove(m);
         layers.remove(m);
         kosherPackages.remove(m);
-        hiddenClasspathPackages.remove(m);
+        synchronized (hiddenClasspathPackages) {
+            hiddenClasspathPackages.remove(m);
+            for (List<Module> ms : hiddenClasspathPackagesReverse.values()) {
+                ms.remove(m);
+                // could also delete entry if ms.isEmpty()
+            }
+        }
     }
     
     public void load(List<Module> modules) {
@@ -307,7 +328,7 @@ final class NbInstaller extends ModuleInstaller {
 	
         ev.log(Events.PERF_START, "NbInstaller.load - sections"); // NOI18N
         ev.log(Events.LOAD_SECTION);
-        CoreBridge.conditionallyLoaderPoolTransaction(true);
+        CoreBridge.getDefault().loaderPoolTransaction(true);
         try {
             for (Module m: modules) {
                 try {
@@ -320,7 +341,7 @@ final class NbInstaller extends ModuleInstaller {
                 ev.log(Events.PERF_TICK, "sections for " + m.getCodeName() + " loaded"); // NOI18N
             }
         } finally {
-            CoreBridge.conditionallyLoaderPoolTransaction(false);
+            CoreBridge.getDefault().loaderPoolTransaction(false);
         }
         ev.log(Events.PERF_END, "NbInstaller.load - sections"); // NOI18N
 
@@ -367,7 +388,7 @@ final class NbInstaller extends ModuleInstaller {
                 Util.err.log(Level.SEVERE, null, le);
             }
         }
-        CoreBridge.conditionallyLoaderPoolTransaction(true);
+        CoreBridge.getDefault().loaderPoolTransaction(true);
         try {
             for (Module m: modules) {
                 try {
@@ -380,7 +401,7 @@ final class NbInstaller extends ModuleInstaller {
             }
         } finally {
             try {
-                CoreBridge.conditionallyLoaderPoolTransaction(false);
+                CoreBridge.getDefault().loaderPoolTransaction(false);
             } catch (RuntimeException e) {
                 Util.err.log(Level.SEVERE, null, e);
             }
@@ -541,13 +562,26 @@ final class NbInstaller extends ModuleInstaller {
         // #23609: dependent modules should be able to override:
         modules = new ArrayList<Module>(modules);
         Collections.reverse(modules);
-        Map<ModuleLayeredFileSystem,List<URL>> urls = new HashMap<ModuleLayeredFileSystem,List<URL>>(5);
-        urls.put(ModuleLayeredFileSystem.getUserModuleLayer(), new ArrayList<URL>(1000));
-        urls.put(ModuleLayeredFileSystem.getInstallationModuleLayer(), new ArrayList<URL>(1000));
+        Map<ModuleLayeredFileSystem,Collection<URL>> urls = new HashMap<ModuleLayeredFileSystem,Collection<URL>>(5);
+        ModuleLayeredFileSystem userModuleLayer = ModuleLayeredFileSystem.getUserModuleLayer();
+        ModuleLayeredFileSystem installationModuleLayer = ModuleLayeredFileSystem.getInstallationModuleLayer();
+        urls.put(userModuleLayer, new LinkedHashSet<URL>(1000));
+        urls.put(installationModuleLayer, new LinkedHashSet<URL>(1000));
         for (Module m: modules) {
+            // #19458: only put reloadables into the "session layer"
+            // (where they will not have their layers cached). All others
+            // should go into "installation layer" (so that they can mask
+            // layers according to cross-dependencies).
+            ModuleLayeredFileSystem host = m.isReloadable() ? userModuleLayer : installationModuleLayer;
+            Collection<URL> theseurls = urls.get(host);
+            if (theseurls == null) {
+                theseurls = new LinkedHashSet<URL>(1000);
+                urls.put(host, theseurls);
+            }
+            ClassLoader cl = m.getClassLoader();
             String s = layers.get(m);
             if (s != null) {
-                Util.err.fine("loadLayer: " + s + " load=" + load);
+                Util.err.log(Level.FINE, "loadLayer: {0} load={1}", new Object[] { s, load });
                 // Actually add a sequence of layers, in locale order.
                 String base, ext;
                 int idx = s.lastIndexOf('.'); // NOI18N
@@ -558,22 +592,6 @@ final class NbInstaller extends ModuleInstaller {
                     base = s.substring(0, idx);
                     ext = s.substring(idx);
                 }
-                ClassLoader cl = m.getClassLoader();
-                ModuleLayeredFileSystem host;
-                // #19458: only put reloadables into the "session layer"
-                // (where they will not have their layers cached). All others
-                // should go into "installation layer" (so that they can mask
-                // layers according to cross-dependencies).
-                if (m.isReloadable()) {
-                    host = ModuleLayeredFileSystem.getUserModuleLayer();
-                } else {
-                    host = ModuleLayeredFileSystem.getInstallationModuleLayer();
-                }
-                List<URL> theseurls = urls.get(host);
-                if (theseurls == null) {
-                    theseurls = new ArrayList<URL>(1000);
-                    urls.put(host, theseurls);
-                }
                 boolean foundSomething = false;
                 for (String suffix : NbCollections.iterable(NbBundle.getLocalizingSuffixes())) {
                     String resource = base + suffix + ext;
@@ -589,20 +607,33 @@ final class NbInstaller extends ModuleInstaller {
                     continue;
                 }
             }
+            try { // #149136
+                // Cannot use getResources because we do not wish to delegate to parents.
+                // In fact both URLClassLoader and ProxyClassLoader override this method to be public.
+                Method findResources = ClassLoader.class.getDeclaredMethod("findResources", String.class); // NOI18N
+                findResources.setAccessible(true);
+                Enumeration e = (Enumeration) findResources.invoke(cl, "META-INF/generated-layer.xml"); // NOI18N
+                while (e.hasMoreElements()) {
+                    URL u = (URL)e.nextElement();
+                    theseurls.add(u);
+                }
+            } catch (Exception x) {
+                Exceptions.printStackTrace(x);
+            }
         }
         // Now actually do it.
-        for (Map.Entry<ModuleLayeredFileSystem,List<URL>> entry: urls.entrySet()) {
+        for (Map.Entry<ModuleLayeredFileSystem,Collection<URL>> entry: urls.entrySet()) {
             ModuleLayeredFileSystem host = entry.getKey();
-            List<URL> theseurls = entry.getValue();
-            Util.err.fine("Adding/removing layer URLs: host=" + host + " urls=" + theseurls);
+            Collection<URL> theseurls = entry.getValue();
+            Util.err.log(Level.FINE, "Adding/removing layer URLs: host={0} urls={1}", new Object[] { host, theseurls });
             try {
                 if (load) {
                     host.addURLs(theseurls);
                 } else {
                     // #106737: we might have the wrong host, since it switches when reloadable flag is toggled.
                     // To be safe, remove from both.
-                    ModuleLayeredFileSystem.getUserModuleLayer().removeURLs(theseurls);
-                    ModuleLayeredFileSystem.getInstallationModuleLayer().removeURLs(theseurls);
+                    userModuleLayer.removeURLs(theseurls);
+                    installationModuleLayer.removeURLs(theseurls);
                 }
             } catch (Exception e) {
                 Util.err.log(Level.WARNING, null, e);
@@ -686,6 +717,19 @@ final class NbInstaller extends ModuleInstaller {
             }
         }
     }
+
+    private static String cacheCnb;
+    private static Set<Dependency> cacheDeps;
+    @Override
+    protected Set<Dependency> loadDependencies(String cnb) {
+        return cnb.equals(cacheCnb) ? cacheDeps : null;
+    }
+    @SuppressWarnings("unchecked")
+    static void register(String name, Object obj) {
+        cacheCnb = name;
+        cacheDeps = (Set<Dependency>)obj;
+    }
+
     
     private AutomaticDependencies autoDepsHandler = null;
     
@@ -709,7 +753,7 @@ final class NbInstaller extends ModuleInstaller {
             return;
         }
         if (autoDepsHandler == null) {
-            FileObject depsFolder = Repository.getDefault().getDefaultFileSystem().findResource("ModuleAutoDeps");
+            FileObject depsFolder = FileUtil.getConfigFile("ModuleAutoDeps");
             if (depsFolder != null) {
                 FileObject[] kids = depsFolder.getChildren();
                 List<URL> urls = new ArrayList<URL>(Math.max(kids.length, 1));
@@ -793,7 +837,10 @@ final class NbInstaller extends ModuleInstaller {
                     return false;
                 }
             }
-            List<Module.PackageExport> hiddenPackages = hiddenClasspathPackages.get(m);
+            List<Module.PackageExport> hiddenPackages;
+            synchronized (hiddenClasspathPackages) {
+                hiddenPackages = hiddenClasspathPackages.get(m);
+            }
             if (hiddenPackages != null) {
                 for (Module.PackageExport hidden : hiddenPackages) {
                     if (hidden.recursive ? pkg.startsWith(hidden.pkg) : pkg.equals(hidden.pkg)) {
@@ -805,11 +852,30 @@ final class NbInstaller extends ModuleInstaller {
                 }
             }
         }
-        if (LOG.isLoggable(Level.FINER)) {
+        if (LOG.isLoggable(Level.FINER) && /* otherwise ClassCircularityError on LogRecord*/!pkg.equals("java/util/logging/")) {
             LOG.finer("Delegating resource " + pkg + " from " + parent + " for " + m.getCodeNameBase());
         }
         return true;
     }
+
+    public @Override boolean shouldDelegateClasspathResource(String pkg) {
+        synchronized (hiddenClasspathPackages) {
+            for (Map.Entry<Module.PackageExport,List<Module>> entry : hiddenClasspathPackagesReverse.entrySet()) {
+                Module.PackageExport hidden = entry.getKey();
+                if (hidden.recursive ? pkg.startsWith(hidden.pkg) : pkg.equals(hidden.pkg)) {
+                    for (Module m : entry.getValue()) {
+                        if (m.isEnabled()) {
+                            if (LOG.isLoggable(Level.FINE)) {
+                                LOG.fine("Refusing to load classpath package " + pkg + " because of " + m.getCodeNameBase());
+                            }
+                            return false;
+                        }
+                    }
+                }
+            }
+        }
+        return true;
+    }
     
     private static final String[] CLASSPATH_PACKAGES = {
         // core.jar shall be inaccessible
@@ -1091,7 +1157,7 @@ final class NbInstaller extends ModuleInstaller {
      * or there is no available cache directory.
      */
     private boolean usingManifestCache;
-    private Object MANIFEST_CACHE = new Object();
+    private final Object MANIFEST_CACHE = new Object();
 
     {
         usingManifestCache = Boolean.valueOf(System.getProperty("netbeans.cache.manifests", "true")).booleanValue();
@@ -1130,13 +1196,9 @@ final class NbInstaller extends ModuleInstaller {
         }
         DateAndManifest entry = cache.get(jar);
         if (entry != null) {
-            if (entry.date == jar.lastModified()) {
-                // Cache hit.
-                MANIFEST_LOG.fine("Found manifest for " + jar + " in cache");
-                return entry.manifest;
-            } else {
-                MANIFEST_LOG.fine("Wrong timestamp for " + jar + " in manifest cache");
-            }
+            // Cache hit.
+            MANIFEST_LOG.fine("Found manifest for " + jar + " in cache");
+            return entry.manifest;
         } else {
             MANIFEST_LOG.fine("No entry for " + jar + " in manifest cache");
         }
