@@ -44,17 +44,20 @@ package org.netbeans.core.startup;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
+import java.net.URL;
+import java.net.URLClassLoader;
 import java.text.DateFormat;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.Locale;
 import java.util.Map;
 import java.util.MissingResourceException;
@@ -68,9 +71,10 @@ import java.util.logging.LogManager;
 import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import java.util.logging.StreamHandler;
+import java.util.regex.Pattern;
+import java.util.regex.PatternSyntaxException;
 import org.netbeans.TopSecurityManager;
 import org.openide.filesystems.FileUtil;
-import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.LookupEvent;
 import org.openide.util.LookupListener;
@@ -92,6 +96,32 @@ public final class TopLogging {
         System.setProperty("sun.awt.exception.handler", "org.netbeans.core.startup.TopLogging$AWTHandler"); // NOI18N
     }
 
+    private static final PrintStream DEBUG;
+    private static final Pattern unwantedMessages;
+    static {
+        PrintStream _D = null;
+        String uMS = System.getProperty("TopLogging.unwantedMessages"); // NOI18N
+        if (uMS != null || Boolean.getBoolean("TopLogging.DEBUG")) { // NOI18N
+            try {
+                File debugLog = new File(System.getProperty("java.io.tmpdir"), "TopLogging.log"); // NOI18N
+                System.err.println("Logging sent to: " + debugLog); // NOI18N
+                _D = new PrintStream(new FileOutputStream(debugLog), true);
+            } catch (FileNotFoundException x) {
+                x.printStackTrace();
+            }
+        }
+        DEBUG = _D;
+        Pattern uMP = null;
+        if (uMS != null) {
+            try {
+                uMP = Pattern.compile(uMS);
+                DEBUG.println("On the lookout for log messages matching: " + uMS); // NOI18N
+            } catch (PatternSyntaxException x) {
+                x.printStackTrace();
+            }
+        }
+        unwantedMessages = uMP;
+    }
 
     /** Initializes the logging configuration. Invoked by <code>LogManager.readConfiguration</code> method.
      */
@@ -135,11 +165,7 @@ public final class TopLogging {
         }
         logger.addHandler(new LookupDel());
 
-        /* TBD:
-        for (Handler h : Lookup.getDefault().lookupAll(Handler.class)) {
-            logger.addHandler(h);
-        }
-         */
+        StartLog.register();
     }
 
     /**
@@ -188,6 +214,11 @@ public final class TopLogging {
 
         if (!(System.err instanceof LgStream)) {
             System.setErr(new LgStream(Logger.getLogger("stderr"))); // NOI18N
+            if (DEBUG != null) DEBUG.println("initializing stderr"); // NOI18N
+        }
+        if (!(System.out instanceof LgStream)) {
+            System.setOut(new LgStream(Logger.getLogger("stderr"))); // NOI18N
+            if (DEBUG != null) DEBUG.println("initializing stdout"); // NOI18N
         }
     }
     
@@ -230,13 +261,35 @@ public final class TopLogging {
         if (nbdirs != null) { // noted in #67862: should show all clusters here.
             StringTokenizer tok = new StringTokenizer(nbdirs, File.pathSeparator);
             while (tok.hasMoreTokens()) {
-                ps.print(FileUtil.normalizeFile(new File(tok.nextToken())));
-                ps.print("\n                            "); //NOI18N
+                File dir = FileUtil.normalizeFile(new File(tok.nextToken()));
+                if (dir.isDirectory()) {
+                    ps.print(dir);
+                    ps.print("\n                            "); //NOI18N
+                }
             }
         }
         ps.println(CLIOptions.getHomeDir());
         ps.println("  Boot & Ext. Classpath   = " + createBootClassPath()); // NOI18N
-        ps.println("  Application Classpath   = " + System.getProperty("java.class.path", "unknown")); // NOI18N
+        String cp;
+        ClassLoader l = Lookup.class.getClassLoader();
+        if (l == ClassLoader.getSystemClassLoader()) {
+            cp = System.getProperty("java.class.path", "unknown"); // NOI18N
+        } else {
+            StringBuilder sb = new StringBuilder("loaded by "); // NOI18N
+            if (l instanceof URLClassLoader) {
+                sb.append("URLClassLoader "); // NOI18N
+                String pref = "";
+                for (URL u : ((URLClassLoader)l).getURLs()) {
+                    sb.append(pref);
+                    sb.append(u.toExternalForm());
+                    pref = File.pathSeparator;
+                }
+            } else {
+                sb.append(l);
+            }
+            cp = sb.toString();
+        }
+        ps.println("  Application Classpath   = " + cp); // NOI18N
         ps.println("  Startup Classpath       = " + System.getProperty("netbeans.dynamic.classpath", "unknown")); // NOI18N
         ps.println("-------------------------------------------------------------------------------"); // NOI18N
     }
@@ -309,6 +362,9 @@ public final class TopLogging {
                 File f1 = new File(dir, "messages.log.1");
                 File f2 = new File(dir, "messages.log.2");
 
+                if (f2.exists()) {
+                    f2.delete();
+                }
                 if (f1.exists()) {
                     f1.renameTo(f2);
                 }
@@ -376,6 +432,7 @@ public final class TopLogging {
     private static final class NonClose extends Handler
     implements Runnable {
         private static RequestProcessor RP = new RequestProcessor("Logging Flush"); // NOI18N
+        private static ThreadLocal<Boolean> FLUSHING = new ThreadLocal<Boolean>();
 
         private final Handler delegate;
         private RequestProcessor.Task flush;
@@ -390,7 +447,14 @@ public final class TopLogging {
 
         public void publish(LogRecord record) {
             delegate.publish(record);
-            flush.schedule(delay);
+            if (!Boolean.TRUE.equals(FLUSHING.get())) {
+                try {
+                    FLUSHING.set(true);
+                    flush.schedule(delay);
+                } finally {
+                    FLUSHING.set(false);
+                }
+            }
         }
 
         public void flush() {
@@ -409,7 +473,7 @@ public final class TopLogging {
             delegate.close();
         }
 
-        public Formatter getFormatter() {
+        public @Override Formatter getFormatter() {
             return delegate.getFormatter();
         }
 
@@ -503,11 +567,15 @@ public final class TopLogging {
         private static String lineSeparator = System.getProperty ("line.separator"); // NOI18N
         static java.util.logging.Formatter FORMATTER = new NbFormatter ();
 
-        
         public String format(java.util.logging.LogRecord record) {
             StringBuilder sb = new StringBuilder();
             print(sb, record, new HashSet<Throwable>());
-            return sb.toString();
+            String r = sb.toString();
+            if (DEBUG != null) DEBUG.print("received: " + r); // NOI18N
+            if (unwantedMessages != null && unwantedMessages.matcher(r).find()) {
+                new Exception().printStackTrace(DEBUG);
+            }
+            return r;
         }
 
 
@@ -638,7 +706,7 @@ public final class TopLogging {
      */
     private static final class LgStream extends PrintStream implements Runnable {
         private Logger log;
-        private StringBuffer sb = new StringBuffer();
+        private final StringBuffer sb = new StringBuffer();
         private static RequestProcessor RP = new RequestProcessor("StdErr Flush");
         private RequestProcessor.Task flush = RP.create(this, true);
 
@@ -647,19 +715,23 @@ public final class TopLogging {
             this.log = log;
         }
 
-        public void write(byte[] buf, int off, int len) {
+        public @Override void write(byte[] buf, int off, int len) {
             if (RP.isRequestProcessorThread()) {
                 return;
             }
-            sb.append(new String(buf, off, len));
+            String s = new String(buf, off, len);
+            if (unwantedMessages != null && unwantedMessages.matcher(s).find()) {
+                new Exception().printStackTrace(DEBUG);
+            }
+            sb.append(s);
             checkFlush();
         }
 
-        public void write(byte[] b) throws IOException {
+        public @Override void write(byte[] b) throws IOException {
             write(b, 0, b.length);
         }
 
-        public void write(int b) {
+        public @Override void write(int b) {
             if (RP.isRequestProcessorThread()) {
                 return;
             }
@@ -681,6 +753,7 @@ public final class TopLogging {
         
         
         private void checkFlush() {
+            //if (DEBUG != null) DEBUG.println("checking flush; buffer: " + sb); // NOI18N
             try {
                 flush.schedule(100);
             } catch (IllegalStateException ex) {
@@ -711,7 +784,6 @@ public final class TopLogging {
         }
         
         public void run() {
-            int lines = 0;
             for (;;) {
                 String toLog;
                 synchronized (sb) {
@@ -727,6 +799,7 @@ public final class TopLogging {
                         sb.delete(0, first + 1);
                     }
                 }
+                //if (DEBUG != null) DEBUG.println("delegating: " + toLog); // NOI18N
                 log.log(Level.INFO, toLog);
             }
         }
@@ -778,7 +851,8 @@ public final class TopLogging {
             if (t.getClass().getName().endsWith(".ExitSecurityException")) { // NOI18N
                 return;
             }
-            Logger.global.log(Level.SEVERE, null, t);
+            Logger g = Logger.getLogger("global");
+            g.log(Level.SEVERE, null, t);
         }
     } // end of AWTHandler
 
