@@ -41,10 +41,15 @@
 
 package org.netbeans.core.startup;
 
-import java.io.*;
+import org.netbeans.SetupHid;
+import org.netbeans.MockEvents;
 import java.io.File;
+import java.io.FileOutputStream;
+import java.lang.reflect.Field;
 import java.util.Collections;
-import java.util.jar.*;
+import java.util.jar.JarEntry;
+import java.util.jar.JarOutputStream;
+import java.util.jar.Manifest;
 import org.netbeans.Module;
 import org.netbeans.ModuleManager;
 import org.openide.util.Utilities;
@@ -69,15 +74,14 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
         moduleJarFile = new File(getWorkDir(), "PlatformDependencySatisfiedModule.jar");
 
         // clean the operatingSystem field
-        java.lang.reflect.Field f;
-        f = org.openide.util.Utilities.class.getDeclaredField("operatingSystem");
+        Field f = Utilities.class.getDeclaredField("operatingSystem");
         f.setAccessible(true);
-        f.set(null, new Integer(-1));
+        f.set(null, -1);
     }
     
     public void testWindows2000() throws Exception {
         System.setProperty("os.name", "Windows 2000");
-        assertTrue("We are on windows", org.openide.util.Utilities.isWindows());
+        assertTrue("We are on windows", Utilities.isWindows());
         
         assertEnableModule("org.openide.modules.os.Windows", true);
         assertEnableModule("org.openide.modules.os.MacOSX", false);
@@ -89,7 +93,7 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
     
     public void testMacOSX() throws Exception {
         System.setProperty("os.name", "Mac OS X");
-        assertTrue("We are on mac", (org.openide.util.Utilities.getOperatingSystem() & org.openide.util.Utilities.OS_MAC) != 0);
+        assertTrue("We are on mac", Utilities.isMac());
         
         assertEnableModule("org.openide.modules.os.Windows", false);
         assertEnableModule("org.openide.modules.os.MacOSX", true);
@@ -101,7 +105,7 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
 
     public void testDarwin() throws Exception {
         System.setProperty("os.name", "Darwin");
-        assertTrue("We are on mac", (org.openide.util.Utilities.getOperatingSystem() & org.openide.util.Utilities.OS_MAC) != 0);
+        assertTrue("We are on mac", Utilities.isMac());
         
         assertEnableModule("org.openide.modules.os.Windows", false);
         assertEnableModule("org.openide.modules.os.MacOSX", true);
@@ -113,7 +117,7 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
     
     public void testLinux() throws Exception {
         System.setProperty("os.name", "Fedora Linux");
-        assertTrue("We are on linux", (org.openide.util.Utilities.getOperatingSystem() & org.openide.util.Utilities.OS_LINUX) != 0);
+        assertTrue("We are on linux", (Utilities.getOperatingSystem() & Utilities.OS_LINUX) != 0);
         
         assertEnableModule("org.openide.modules.os.Windows", false);
         assertEnableModule("org.openide.modules.os.MacOSX", false);
@@ -141,7 +145,7 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
 
     public void testBSD() throws Exception {
         System.setProperty("os.name", "FreeBSD X1.4");
-        assertTrue("We are on unix", org.openide.util.Utilities.isUnix());
+        assertTrue("We are on unix", Utilities.isUnix());
         
         assertEnableModule("org.openide.modules.os.Windows", false);
         assertEnableModule("org.openide.modules.os.MacOSX", false);
@@ -153,7 +157,7 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
 
     public void testOS2() throws Exception {
         System.setProperty("os.name", "OS/2");
-        assertEquals ("We are on os/2", org.openide.util.Utilities.OS_OS2, org.openide.util.Utilities.getOperatingSystem());
+        assertEquals ("We are on os/2", Utilities.OS_OS2, Utilities.getOperatingSystem());
         
         assertEnableModule("org.openide.modules.os.Windows", false);
         assertEnableModule("org.openide.modules.os.MacOSX", false);
@@ -163,7 +167,6 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
         assertEnableModule("org.openide.modules.os.OS2", true);
     }
     
-    /**  */
     private void assertEnableModule(String req, boolean enable) throws Exception {
         Manifest man = new Manifest ();
         man.getMainAttributes ().putValue ("Manifest-Version", "1.0");
@@ -176,17 +179,14 @@ public class PlatformDependencySatisfiedTest extends SetupHid {
         os.putNextEntry (new JarEntry ("empty/test.txt"));
         os.close ();
         
-        
-        final FakeEvents ev = new FakeEvents();
-        org.netbeans.core.startup.NbInstaller installer = new org.netbeans.core.startup.NbInstaller(ev);
+        final MockEvents ev = new MockEvents();
+        NbInstaller installer = new NbInstaller(ev);
         ModuleManager mgr = new ModuleManager(installer, ev);
         ModuleFormatSatisfiedTest.addOpenideModules(mgr);
         installer.registerManager(mgr);
         mgr.mutexPrivileged().enterWriteAccess();
         try {
             Module m1 = mgr.create(moduleJarFile, null, false, false, false);
-            
-            
             if (enable) {
                 assertEquals(Collections.EMPTY_SET, m1.getProblems());
                 mgr.enable(m1);
