@@ -55,7 +55,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.Stack;
 import java.util.WeakHashMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
@@ -172,7 +171,6 @@ public final class FileObjectFactory {
     
     public BaseFileObj getFileObject(FileInfo fInfo, Caller caller) {
         File file = fInfo.getFile();
-        FileObject retVal = null;
         FolderObj parent = BaseFileObj.getExistingParentFor(file, this);
         FileNaming child = null;
         boolean isInitializedCache = true;
@@ -241,9 +239,11 @@ public final class FileObjectFactory {
         sb.append(file.exists() ? "created " : "deleted "); //NOI18N
         sb.append(file.isDirectory() ? "folder: " : "file: "); //NOI18N
         sb.append(file.getAbsolutePath());
-        sb.append("  (For additional information see: http://wiki.netbeans.org/wiki/view/FileSystems)");//NOI18N        
+        sb.append(" (Possibly not refreshed FileObjects when external command finished.");//NOI18N
+        sb.append(" For additional information see: http://wiki.netbeans.org/wiki/view/FileSystems)");//NOI18N
         IllegalStateException ise = new IllegalStateException(sb.toString());
-        Logger.getLogger(getClass().getName()).log(Level.INFO, ise.getMessage(), ise);
+        // set to INFO level to be notified about possible problems.
+        Logger.getLogger(getClass().getName()).log(Level.FINE, ise.getMessage(), ise);
     }
 
     private BaseFileObj issueIfExist(File file, Caller caller, final FileObject parent, FileNaming child, int initTouch,boolean asyncFire) {
@@ -399,10 +399,6 @@ public final class FileObjectFactory {
     }
 
     private BaseFileObj create(final FileInfo fInfo) {
-        if (fInfo.isWindowsFloppy()) {
-            return null;
-        }
-
         if (!fInfo.isConvertibleToFileObject()) {
             return null;
         }
@@ -480,10 +476,8 @@ public final class FileObjectFactory {
     }
 
     public static boolean isParentOf(final File dir, final File file) {
-        Stack stack = new Stack();
         File tempFile = file;
         while (tempFile != null && !tempFile.equals(dir)) {
-            stack.push(tempFile.getName());
             tempFile = tempFile.getParentFile();
         }
         return tempFile != null;
