@@ -49,14 +49,17 @@ import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.Writer;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.StringTokenizer;
 import java.util.TreeMap;
 import java.util.TreeSet;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
@@ -80,6 +83,11 @@ public final class CheckModuleConfigs extends Task {
     public void setNbroot(File f) {
         nbroot = f;
     }
+
+    private File masterProjectXml;
+    public void setMasterProjectXml(File masterProjectXml) {
+        this.masterProjectXml = masterProjectXml;
+    }
     
     public @Override void execute() throws BuildException {
         if (nbroot == null) {
@@ -88,7 +96,6 @@ public final class CheckModuleConfigs extends Task {
         File buildPropertiesFile = new File(nbroot, "nbbuild" + File.separatorChar + "build.properties");
         File clusterPropertiesFile = new File(nbroot, "nbbuild" + File.separatorChar + "cluster.properties");
         File goldenFile = new File(nbroot, "nbbuild" + File.separatorChar + "build" + File.separatorChar + "generated" + File.separatorChar + "moduleconfigs.txt");
-        File masterProjectXml = new File(nbroot, "ide" + File.separatorChar + "allmodules" + File.separatorChar + "nbproject" + File.separatorChar + "project.xml");
         @SuppressWarnings("unchecked")
         Map<String,String> properties = getProject().getProperties();
         Map<String,Set<String>> configs = loadModuleConfigs(properties, buildPropertiesFile);
@@ -110,28 +117,12 @@ public final class CheckModuleConfigs extends Task {
             throw new BuildException("Could not write to " + masterProjectXml, e, getLocation());
         }
         Set<String> s;
-        /* Apparently wanted now?
-        // Check that stable != daily-alpha-nbms:
-        s = new TreeSet((Set) configs.get("stable"));
-        s.retainAll((Set) configs.get("daily-alpha-nbms"));
-        if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: stable and daily-alpha-nbms configs overlap: " + s);
-        }
-        */
         /* This is not actually desired; just includes everything:
         // Check that sigtest <= javadoc:
         s = new TreeSet((Set) configs.get("sigtest"));
         s.removeAll((Set) configs.get("javadoc"));
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: sigtest config contains entries not in javadoc config: " + s);
-        }
-        */
-        /* This config is no more?
-        // Check that platform-javadoc <= javadoc:
-        s = new TreeSet((Set) configs.get("platform-javadoc"));
-        s.removeAll((Set) configs.get("javadoc"));
-        if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: platform-javadoc config contains entries not in javadoc config: " + s);
+            throw new BuildException(buildPropertiesFile + ": sigtest config contains entries not in javadoc config: " + s);
         }
         */
         // Check that javadoc <= stable + daily-alpha-nbms:
@@ -139,35 +130,19 @@ public final class CheckModuleConfigs extends Task {
         s.removeAll(configs.get("stable"));
         s.removeAll(configs.get("daily-alpha-nbms"));
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: javadoc config contains entries not in stable and daily-alpha-nbms configs: " + s);
+            throw new BuildException(buildPropertiesFile + ": javadoc config contains entries not in stable and daily-alpha-nbms configs: " + s);
         }
-        /*
-        // Check that platform-javadoc = javadoc where module in platform cluster:
-        Set/ *<String>* / platformJavadoc = (Set) configs.get("platform-javadoc");
-        Set/ *<String>* / platformClusterJavadoc = (Set) configs.get("javadoc");
-        platformClusterJavadoc.retainAll((Set) clusters.get("nb.cluster.platform"));
-        s = new TreeSet(platformJavadoc);
-        s.removeAll(platformClusterJavadoc);
-        if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: platform-javadoc config not equal to javadoc config for platform cluster modules: " + s);
-        }
-        s = new TreeSet(platformClusterJavadoc);
-        s.removeAll(platformJavadoc);
-        if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: platform-javadoc config not equal to javadoc config restricted to platform cluster modules: " + s);
-        }
-        */
         // Check that stable = modules in enumerated clusters:
         Set<String> stable = configs.get("all");
         s = new TreeSet<String>(stable);
         s.removeAll(allClusterModules);
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: 'all' config not equal to listed cluster modules: " + s);
+            throw new BuildException(buildPropertiesFile + ": 'all' config not equal to listed cluster modules: " + s);
         }
         s = new TreeSet<String>(allClusterModules);
         s.removeAll(stable);
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: 'all' config not equal to listed cluster modules: " + s);
+            throw new BuildException(buildPropertiesFile + ": 'all' config not equal to listed cluster modules: " + s);
         }
         // Check that platform = modules in platform cluster:
         Set<String> platform = configs.get("platform");
@@ -175,31 +150,69 @@ public final class CheckModuleConfigs extends Task {
         s = new TreeSet<String>(platform);
         s.removeAll(platformCluster);
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: platform config not equal to platform cluster modules: " + s);
+            throw new BuildException(buildPropertiesFile + ": platform config not equal to platform cluster modules: " + s);
         }
         s = new TreeSet<String>(platformCluster);
         s.removeAll(platform);
         if (!s.isEmpty()) {
-            log(buildPropertiesFile + ": warning: platform config not equal to platform cluster modules: " + s);
+            throw new BuildException(buildPropertiesFile + ": platform config not equal to platform cluster modules: " + s);
+        }
+        // Verify sorting and overlaps:
+        Pattern clusterNamePat = Pattern.compile("nb\\.cluster\\.([^.]+)");
+        Map<String,List<String>> allClusters = new HashMap<String,List<String>>();
+        for (Map.Entry<String,String> clusterDef : properties.entrySet()) {
+            Matcher m = clusterNamePat.matcher(clusterDef.getKey());
+            if (!m.matches()) {
+                continue;
+            }
+            allClusters.put(m.group(1), splitToList(clusterDef.getValue(), clusterDef.getKey()));
+        }
+        allClusters.get("experimental").removeAll(allClusters.get("stableuc")); // intentionally a superset
+        for (Map.Entry<String,List<String>> entry : allClusters.entrySet()) {
+            String name = entry.getKey();
+            List<String> modules = entry.getValue();
+            Set<String> modulesS = new HashSet<String>(modules);
+            if (modulesS.size() < modules.size()) {
+                throw new BuildException("duplicates found in " + name + ": " + modules);
+            }
+            for (Map.Entry<String,List<String>> entry2 : allClusters.entrySet()) {
+                String other = entry.getKey();
+                if (other.equals(name)) {
+                    continue;
+                }
+                if (modulesS.removeAll(entry2.getValue())) {
+                    throw new BuildException("some entries in " + name + " also found in " + other);
+                }
+            }
+            List<String> sorted = new ArrayList<String>(modules);
+            Collections.sort(sorted);
+            if (!sorted.equals(modules)) {
+                throw new BuildException("unsorted list for " + name + ": " + modules);
+            }
         }
     }
     
-    @SuppressWarnings("unchecked")
-    private Set<String> split(String list, boolean warnIfUnsorted, String what) {
-        List elements = Collections.list(new StringTokenizer(list, ", "));
-        if (warnIfUnsorted) {
-            List sorted = new ArrayList(elements);
-            Collections.sort(sorted);
-            if (!sorted.equals(elements)) {
-                log("warning: unsorted list: " + elements);
-            }
+    private List<String> splitToList(String list, String what) {
+        if (list.length() == 0) {
+            return Collections.emptyList();
         }
-        HashSet set = new HashSet(elements);
-        for (Object o : set) {
-            elements.remove(o);
+        // !list.matches("[^\\s,]+(,[^\\s,]+)*") exhausts stack for long module lists
+        if (list.matches(".*\\s.*|^,.*|.*,$|.*,,.*")) {
+            throw new BuildException("remove whitespaces or fix leading/trailing commas in " + what + ": " + list);
+        }
+        List<String> r = new ArrayList<String>(Arrays.asList(list.split(",")));
+        assert !r.contains(null) : r;
+        assert !r.contains("") : r;
+        return r;
+    }
+    private Set<String> splitToSet(String list, String what) {
+        List<String> elements = splitToList(list, what);
+        Set<String> set = new HashSet<String>(elements);
+        for (String s : set) {
+            elements.remove(s);
         }
         if (!elements.isEmpty()) { // #147690
-            log("warning: duplicates found in " + what + ": " + elements);
+            throw new BuildException("duplicates found in " + what + ": " + elements);
         }
         return set;
     }
@@ -212,13 +225,13 @@ public final class CheckModuleConfigs extends Task {
                 continue;
             }
             String config = k.substring(prefix.length());
-            Set<String> modules = new TreeSet<String>(split(buildProperties.get(k), false, k));
+            Set<String> modules = new TreeSet<String>(splitToSet(buildProperties.get(k), k));
             String fixedK = "config.fixedmodules." + config;
             String fixed = buildProperties.get(fixedK);
             if (fixed != null) {
-                modules.addAll(split(fixed, false, fixedK));
+                modules.addAll(splitToSet(fixed, fixedK));
             } else {
-                log(buildPropertiesFile + ": warning: have " + k + " but no " + fixedK, Project.MSG_WARN);
+                throw new BuildException(buildPropertiesFile + ": have " + k + " but no " + fixedK);
             }
             configs.put(config, modules);
         }
@@ -226,12 +239,11 @@ public final class CheckModuleConfigs extends Task {
     }
 
     private void writeModuleConfigs(File goldenFile, Map<String,Set<String>> configs, File buildPropertiesFile) throws IOException {
-        log("Writing moduleconfigs " + configs.keySet() + " from " + buildPropertiesFile + " to " + goldenFile);
+        log("Writing moduleconfigs " + configs.keySet() + " from " + buildPropertiesFile + " to " + goldenFile, Project.MSG_VERBOSE);
         goldenFile.getParentFile().mkdirs();
         Writer w = new FileWriter(goldenFile); // default encoding OK
         try {
             PrintWriter pw = new PrintWriter(w);
-            pw.println("# To update, run: ant -f nbbuild/build.xml check-module-configs");
             for (Map.Entry<String,Set<String>> entry : configs.entrySet()) {
                 String config = entry.getKey();
                 for (String module : entry.getValue()) {
@@ -248,23 +260,24 @@ public final class CheckModuleConfigs extends Task {
         String fullConfig = "clusters.config.full.list";
         String l = clusterProperties.get(fullConfig);
         if (l == null) {
-            log(clusterPropertiesFile + ": warning: no definition for clusters.config.full.list", Project.MSG_WARN);
-            return Collections.emptyMap();
+            throw new BuildException(clusterPropertiesFile + ": no definition for clusters.config.full.list");
         }
         Map<String,Set<String>> clusters = new TreeMap<String,Set<String>>();
-        for (String cluster : split(l, false,fullConfig)) {
+        for (String cluster : splitToSet(l, fullConfig)) {
             l = clusterProperties.get(cluster);
             if (l == null) {
-                log(clusterPropertiesFile + ": warning: no definition for " + cluster, Project.MSG_WARN);
-                continue;
+                throw new BuildException(clusterPropertiesFile + ": no definition for " + cluster);
             }
-            clusters.put(cluster, new TreeSet<String>(split(l, true,fullConfig)));
+            clusters.put(cluster, new TreeSet<String>(splitToSet(l, fullConfig)));
         }
         return clusters;
     }
 
     private void writeMasterProjectXml(File masterProjectXml, Set<String> allClusterModules) throws IOException, SAXException {
-        log("Writing module list  to " + masterProjectXml);
+        if (masterProjectXml == null) {
+            return;
+        }
+        log("Writing module list to " + masterProjectXml);
         Document doc = XMLUtil.parse(new InputSource(masterProjectXml.toURI().toString()), false, true, null, null);
         NodeList nl = doc.getElementsByTagName("subprojects");
         if (nl.getLength() != 1) {
@@ -275,10 +288,10 @@ public final class CheckModuleConfigs extends Task {
         while (nl.getLength() > 0) {
             sp.removeChild(nl.item(0));
         }
-        sp.appendChild(doc.createComment(" To update, run: ant -f nbbuild/build.xml check-module-configs "));
+        sp.appendChild(doc.createComment(" To update, run target listed above "));
         for (String module : allClusterModules) {
             if (new File(nbroot, (module + "/nbproject/project.xml").replace('/', File.separatorChar)).isFile()) {
-                Element e = doc.createElementNS("http://www.netbeans.org/ns/freeform-project/1", "project");
+                Element e = doc.createElementNS("http://www.netbeans.org/ns/freeform-project/2", "project");
                 String path = "../../" + module;
                 e.appendChild(doc.createTextNode(path.replaceFirst("^\\.\\./\\.\\./ide/", "../")));
                 sp.appendChild(e);
@@ -293,5 +306,5 @@ public final class CheckModuleConfigs extends Task {
             os.close();
         }
     }
-    
+
 }
