@@ -50,7 +50,9 @@ import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.taskdefs.MatchingTask;
 import org.apache.tools.ant.DirectoryScanner;
 import java.io.File;
+import java.io.FileFilter;
 import java.io.FileOutputStream;
+import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -163,6 +165,13 @@ public class MakeUpdateDesc extends MatchingTask {
         dist_base = dbase;
     }
 
+    private boolean useLicenseUrl;
+
+
+    public void setUseLicenseUrl(boolean useLicenseUrl) {
+        this.useLicenseUrl = useLicenseUrl;
+    }
+    
     private Path updaterJar;
     /** Fileset for platform/modules/ext/updater.jar, to be used in DTD validation. */
     public Path createUpdaterJar() {
@@ -269,7 +278,9 @@ public class MakeUpdateDesc extends MatchingTask {
                     }
                     File desc_ent = new File(ent_name);
                     desc_ent.delete();
-                    if (use25DTD) {
+                    if(useLicenseUrl) {
+                        pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.6//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_6.dtd\" [");
+                    } else if (use25DTD) {
                         pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.5//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_5.dtd\" [");
                     } else if (targetClustersDefined) {
                         pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.4//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_4.dtd\" [");
@@ -303,7 +314,9 @@ public class MakeUpdateDesc extends MatchingTask {
                     pw.println ();
                     
                 } else {
-                    if (use25DTD) {
+                    if(useLicenseUrl) {
+                        pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.6//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_6.dtd\">");
+                    } else if (use25DTD) {
                         pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.5//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_5.dtd\">");
                     } else if (targetClustersDefined) {
                         pw.println("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.4//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_4.dtd\">");
@@ -316,8 +329,31 @@ public class MakeUpdateDesc extends MatchingTask {
 
                 pw.println ();
 		Map<String,Element> licenses = new HashMap<String,Element>();
-                Set<String> licenseNames = new HashSet<String>();
-                
+                String prefix = null;
+                if (dist_base != null) {
+                    // fix/enforce distribution URL base
+                    if (dist_base.equals(".")) {
+                        prefix = "";
+                    } else {
+                        prefix = dist_base + "/";
+                    }
+                }
+                final File licensesDir = new File(desc.getParentFile(), "licenses");
+                if (useLicenseUrl) {
+                     if (licensesDir.exists()) {
+                         File [] licenseFiles = licensesDir.listFiles(new FileFilter() {
+                            public boolean accept(File pathname) {
+                                return pathname.getName().endsWith(".license");
+                            }
+                        });
+                         for (File f : licenseFiles) {
+                            f.delete();
+                        }
+                        } else {
+                        licensesDir.mkdir();
+                     }
+                }
+
                 for (Map.Entry<String,Collection<Module>> entry : modulesByGroup.entrySet()) {
                     String groupName = entry.getKey();
                     // Don't indent; embedded descriptions would get indented otherwise.
@@ -335,20 +371,26 @@ public class MakeUpdateDesc extends MatchingTask {
                         String name = manifest.getAttribute("OpenIDE-Module-Name");
                         if (name.length() > 0) {
                             log(" Adding module " + name + " (" + m.nbm.getAbsolutePath() + ")");
-                        }
-                        if (dist_base != null) {
-                            // fix/enforce distribution URL base
-                            String prefix;
-                            if (dist_base.equals(".")) {
-                                prefix = "";
-                            } else {
-                                prefix = dist_base + "/";
-                            }
+                        }                        
+                        if(prefix!=null) {
                             module.setAttribute("distribution", prefix + m.relativePath);
                         }
                         NodeList licenseList = module.getElementsByTagName("license");
                         if (licenseList.getLength() > 0) {
                             Element license = (Element) licenseList.item(0);
+                            if (useLicenseUrl) {
+                                String relativePath = "licenses/" + license.getAttribute("name") + ".license";
+                                String path = relativePath;
+                                if (prefix != null) {
+                                    path = prefix + relativePath;
+                                }
+                                license.setAttribute("url", path);
+                                String licenseText = license.getTextContent();
+                                license.setTextContent("");
+                                FileOutputStream fos = new FileOutputStream(new File(desc.getParentFile(), relativePath));
+                                fos.write(licenseText.getBytes("UTF-8"));
+                                fos.close();
+                            }
                             // XXX ideally would compare the license texts to make sure they actually match up
                             licenses.put(license.getAttribute("name"), license);
                             module.removeChild(license);
