@@ -75,19 +75,20 @@ public class ModuleListParserTest extends TestCase {
         String prop = System.getProperty("nb_all");
         assertNotNull("${nb_all} defined", prop);
         nball = new File(prop);
-        new File(nball, "nbproject/private/scan-cache-full.ser").delete();
-        new File(nball, "nbproject/private/scan-cache-standard.ser").delete();
+        new File(nball, "nbbuild/nbproject/private/scan-cache-full.ser").delete();
+        new File(nball, "nbbuild/nbproject/private/scan-cache-standard.ser").delete();
     }
     
     public void testScanSourcesInNetBeansOrg() throws Exception {
         Hashtable<String,String> properties = new Hashtable<String,String>();
         properties.put("nb_all", nball.getAbsolutePath());
-        File build = file(nball, "build");
+        File build = file(nball, "nbbuild/netbeans");
         properties.put("netbeans.dest.dir", build.getAbsolutePath());
         properties.put("nb.cluster.foo", "beans,clazz");
         properties.put("nb.cluster.foo.dir", "foodir");
         properties.put("nb.cluster.bar", "core.startup");
         properties.put("nb.cluster.bar.dir", "bardir");
+        properties.put("basedir", new File(nball, "nbbuild").getAbsolutePath());
         long start = System.currentTimeMillis();
         ModuleListParser p = new ModuleListParser(properties, ParseProjectXml.TYPE_NB_ORG, null);
         System.err.println("Scanned " + nball + " sources in " + (System.currentTimeMillis() - start) + "msec");
@@ -141,7 +142,8 @@ public class ModuleListParserTest extends TestCase {
             public void buildFinished(BuildEvent buildEvent) {}
         });
         Hashtable<String,String> properties = new Hashtable<String,String>();
-        properties.put("netbeans.dest.dir", filePath(nball, "nbbuild/netbeans"));
+        properties.put("cluster.path.final", filePath(nball, "nbbuild/netbeans/platform10")
+                + File.pathSeparator + filePath(nball, "nbbuild/netbeans/ide11"));
         properties.put("basedir", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite1/action-project"));
         properties.put("suite.dir", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite1"));
         long start = System.currentTimeMillis();
@@ -156,39 +158,40 @@ public class ModuleListParserTest extends TestCase {
         assertNotNull("found sister project in suite", e);
         assertEquals("org.netbeans.examples.modules.lib", e.getCnb());
         assertEquals(file(nball, "apisupport.project/test/unit/data/example-external-projects/suite1/build/cluster/modules/org-netbeans-examples-modules-lib.jar"), e.getJar());
-        File jar = file(nball, "nbbuild/netbeans/ide10/modules/org-netbeans-libs-xerces.jar");
+        File jar = file(nball, "nbbuild/netbeans/ide11/modules/org-netbeans-libs-xerces.jar");
         assertTrue("Build all-libs/xerces first!", jar.isFile());
         e = p.findByCodeNameBase("org.netbeans.libs.xerces");
         assertNotNull("found netbeans.org module by its binary", e);
         assertEquals("org.netbeans.libs.xerces", e.getCnb());
         assertEquals(jar, e.getJar());
         assertEquals("correct CP extensions (using Class-Path header in manifest)",
-                Collections.singletonList(file(nball, "nbbuild/netbeans/ide10/modules/ext/xerces-2.8.0.jar")),
+                Collections.singletonList(file(nball, "nbbuild/netbeans/ide11/modules/ext/xerces-2.8.0.jar")),
                 Arrays.asList(e.getClassPathExtensions()));
         e = p.findByCodeNameBase("org.openide.loaders");
         assertNotNull(e);
         assertEquals("org.openide.loaders", e.getCnb());
-        assertEquals(file(nball, "nbbuild/netbeans/platform9/modules/org-openide-loaders.jar"), e.getJar());
+        assertEquals(file(nball, "nbbuild/netbeans/platform10/modules/org-openide-loaders.jar"), e.getJar());
         assertEquals(Collections.EMPTY_LIST, Arrays.asList(e.getClassPathExtensions()));
         e = p.findByCodeNameBase("org.netbeans.bootstrap");
         assertNotNull(e);
         assertEquals("org.netbeans.bootstrap", e.getCnb());
-        assertEquals(file(nball, "nbbuild/netbeans/platform9/lib/boot.jar"), e.getJar());
+        assertEquals(file(nball, "nbbuild/netbeans/platform10/lib/boot.jar"), e.getJar());
         assertEquals(Collections.EMPTY_LIST, Arrays.asList(e.getClassPathExtensions()));
-        jar = file(nball, "nbbuild/netbeans/ide10/modules/org-netbeans-modules-xml-tax.jar");
+        jar = file(nball, "nbbuild/netbeans/ide11/modules/org-netbeans-modules-xml-tax.jar");
         assertTrue("Build all-xml/tax first!", jar.isFile());
         e = p.findByCodeNameBase("org.netbeans.modules.xml.tax");
         assertNotNull(e);
         assertEquals("org.netbeans.modules.xml.tax", e.getCnb());
         assertEquals(jar, e.getJar());
         assertEquals(Arrays.asList(new File[] {
-            file(nball, "nbbuild/netbeans/ide10/modules/ext/org-netbeans-tax.jar"),
+            file(nball, "nbbuild/netbeans/ide11/modules/ext/org-netbeans-tax.jar"),
         }), Arrays.asList(e.getClassPathExtensions()));
     }
     
     public void testScanSourcesAndBinariesForExternalStandaloneModule() throws Exception {
         Hashtable<String,String> properties = new Hashtable<String,String>();
-        properties.put("netbeans.dest.dir", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite3/nbplatform"));
+        properties.put("cluster.path.final", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite3/nbplatform/platform5") +
+                File.pathSeparator + filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite3/nbplatform/random"));
         properties.put("basedir", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite3/dummy-project"));
         properties.put("project", filePath(nball, "apisupport.project/test/unit/data/example-external-projects/suite3/dummy-project"));
         ModuleListParser p = new ModuleListParser(properties, ParseProjectXml.TYPE_STANDALONE, null);
