@@ -39,6 +39,7 @@
 
 package org.netbeans.junit;
 
+import org.netbeans.junit.internal.NbModuleLogHandler;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -53,11 +54,16 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Enumeration;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
+import java.util.StringTokenizer;
 import java.util.TreeSet;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.swing.SwingUtilities;
@@ -89,6 +95,10 @@ import junit.framework.TestResult;
  */
 public class NbModuleSuite {
 
+    static {
+        System.setProperty("org.netbeans.MainImpl.154417", "true");
+    }
+
     private NbModuleSuite() {}
     
     
@@ -101,20 +111,29 @@ public class NbModuleSuite {
     public static final class Configuration extends Object {
         final List<Item> tests;
         final Class<? extends TestCase> latestTestCaseClass;
-        final String clusterRegExp;
-        final String moduleRegExp;
+        final List<String> clusterRegExp;
+        /** each odd is cluster reg exp, each even is module reg exp */
+        final List<String> moduleRegExp;
         final ClassLoader parentClassLoader;
         final boolean reuseUserDir;
         final boolean gui;
+        final boolean enableClasspathModules;
+        final boolean honorAutoEager;
+        final Level failOnMessage;
+        final Level failOnException;
 
         private Configuration(
-            String clusterRegExp, 
-            String moduleRegExp,
-            ClassLoader parent, 
+            List<String> clusterRegExp,
+            List<String> moduleRegExp,
+            ClassLoader parent,
             List<Item> testItems,
-            Class<? extends TestCase> latestTestCase, 
+            Class<? extends TestCase> latestTestCase,
             boolean reuseUserDir,
-            boolean gui
+            boolean gui,
+            boolean enableCPModules,
+            boolean honorAutoEager,
+            Level failOnMessage,
+            Level failOnException
         ) {
             this.clusterRegExp = clusterRegExp;
             this.moduleRegExp = moduleRegExp;
@@ -123,10 +142,17 @@ public class NbModuleSuite {
             this.reuseUserDir = reuseUserDir;
             this.latestTestCaseClass = latestTestCase;
             this.gui = gui;
+            this.enableClasspathModules = enableCPModules;
+            this.honorAutoEager = honorAutoEager;
+            this.failOnException = failOnException;
+            this.failOnMessage = failOnMessage;
         }
 
         static Configuration create(Class<? extends TestCase> clazz) {            
-            return new Configuration(null, null, ClassLoader.getSystemClassLoader().getParent(), Collections.<Item>emptyList(), clazz, false, true);
+            return new Configuration(
+                null, null, ClassLoader.getSystemClassLoader().getParent(),
+                Collections.<Item>emptyList(), clazz, false, true, true, false
+                , null, null);
         }
         
         /** Regular expression to match clusters that shall be enabled.
@@ -134,27 +160,76 @@ public class NbModuleSuite {
          * ide and java clusters, it is handy to pass in <code>"ide.*|java.*</code>.
          * There is no need to requrest presence of <code>platform.*</code> cluster,
          * as that is available all the time by default.
+         * <p>
+         * Since version 1.55 this method can be called multiple times.
          * 
          * @param regExp regular expression to match cluster names
          * @return clone of this configuration with cluster set to regExp value
          */
         public Configuration clusters(String regExp) {
-            return new Configuration(regExp, moduleRegExp, parentClassLoader, tests, latestTestCaseClass, reuseUserDir, gui);
+            ArrayList<String> list = new ArrayList<String>();
+            if (clusterRegExp != null) {
+                list.addAll(clusterRegExp);
+            }
+            if (regExp != null) {
+                list.add(regExp);
+            }
+            if (list.isEmpty()) {
+                list = null;
+            }
+            return new Configuration(
+                list, moduleRegExp, parentClassLoader, tests,
+                latestTestCaseClass, reuseUserDir, gui, enableClasspathModules,
+                honorAutoEager
+            , failOnMessage, failOnException);
         }
 
         /** By default only modules on classpath of the test are enabled, 
          * the rest are just autoloads. If you need to enable more, you can
          * specify that with this method. To enable all available modules
-         * in all clusters pass in <code>".*"</code>.
+         * in all clusters pass in <code>".*"</code>. Since 1.55 this method
+         * is cummulative.
+         * 
          * @param regExp regular expression to match code name base of modules
          * @return clone of this configuration with enable modules set to regExp value
          */
         public Configuration enableModules(String regExp) {
-            return new Configuration(clusterRegExp, regExp, parentClassLoader, tests, latestTestCaseClass, reuseUserDir, gui);
+            if (regExp == null) {
+                return this;
+            }
+            return enableModules(".*", regExp);
+        }
+
+        /** By default only modules on classpath of the test are enabled,
+         * the rest are just autoloads. If you need to enable more, you can
+         * specify that with this method. To enable all available modules in
+         * one cluster, use this method and pass <code>".*"</code> as list of
+         * modules. This method is cumulative.
+         *
+         * @param clusterRegExp regular expression to match clusters
+         * @param moduleRegExp regular expression to match code name base of modules
+         * @return clone of this configuration with enable modules set to regExp value
+         * @since 1.55
+         */
+        public Configuration enableModules(String clusterRegExp, String moduleRegExp) {
+            List<String> arr = new ArrayList<String>();
+            if (this.moduleRegExp != null) {
+                arr.addAll(this.moduleRegExp);
+            }
+            arr.add(clusterRegExp);
+            arr.add(moduleRegExp);
+            return new Configuration(
+                this.clusterRegExp, arr, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir, gui,
+                enableClasspathModules, honorAutoEager, failOnMessage, failOnException);
         }
 
         Configuration classLoader(ClassLoader parent) {
-            return new Configuration(clusterRegExp, moduleRegExp, parent, tests, latestTestCaseClass, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parent, tests,
+                latestTestCaseClass, reuseUserDir, gui, enableClasspathModules,
+                honorAutoEager
+            , failOnMessage, failOnException);
         }
 
         /** Adds new test name, or array of names into the configuration. By 
@@ -174,7 +249,10 @@ public class NbModuleSuite {
             }
             List<Item> newTests = new ArrayList<Item>(tests);
             newTests.add(new Item(true, latestTestCaseClass, testNames));
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, newTests, latestTestCaseClass, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                newTests, latestTestCaseClass, reuseUserDir, gui,
+                enableClasspathModules, honorAutoEager, failOnMessage, failOnException);
         }
         
         /** Adds new test class to run, together with a list of its methods
@@ -197,7 +275,11 @@ public class NbModuleSuite {
             if ((testNames != null) && (testNames.length != 0)){
                 newTests.add(new Item(true, test, testNames));
             }
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, newTests, test, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                newTests, test, reuseUserDir, gui, enableClasspathModules,
+                honorAutoEager
+            , failOnMessage, failOnException);
         }
         
         /**
@@ -218,7 +300,76 @@ public class NbModuleSuite {
             }
             List<Item> newTests = new ArrayList<Item>(tests);
             newTests.add(new Item(false, test, null));
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, newTests, latestTestCaseClass, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                newTests, latestTestCaseClass, reuseUserDir,
+                gui, enableClasspathModules, honorAutoEager
+            , failOnMessage, failOnException);
+        }
+
+        /** By default all modules on classpath are enabled (so you can link
+         * with modules that you compile against), this method allows you to
+         * disable this feature, which is useful if the test is known to not
+         * link against any of classpath classes.
+         *
+         * @param enable pass false to ignore modules on classpath
+         * @return new configuration clone
+         * @since 1.56
+         */
+        public Configuration enableClasspathModules(boolean enable) {
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir,
+                gui, enable, honorAutoEager, failOnMessage, failOnException);
+        }
+
+        /** By default the {@link #enableModules(java.lang.String)} method
+         * converts all autoloads into regular modules and enables them. This
+         * is maybe useful in certain situations, but does not really mimic the
+         * real behaviour of the system when it is executed. Those who need
+         * to as closely as possible simulate the real run, can use
+         * <code>honorAutoloadEager(true)</code>.
+         *
+         * @param honor true in case autoloads shall remain autoloads and eager modules eager
+         * @return new configuration filled with this data
+         * @since 1.57
+         */
+        public Configuration honorAutoloadEager(boolean honor) {
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir,
+                gui, enableClasspathModules, honor
+            , failOnMessage, failOnException);
+        }
+
+        /** Fails if there is a message sent to {@link Logger} with appropriate
+         * level or higher during the test run execution.
+         *
+         * @param level the minimal level of the message
+         * @return new configuration filled with this data
+         * @since 1.58
+         */
+        public Configuration failOnMessage(Level level) {
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir,
+                gui, enableClasspathModules, honorAutoEager
+                , level, failOnException);
+        }
+
+        /** Fails if there is an exception reported to {@link Logger} with appropriate
+         * level or higher during the test run execution.
+         *
+         * @param level the minimal level of the message
+         * @return new configuration filled with this data
+         * @since 1.58
+         */
+        public Configuration failOnException(Level level) {
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir,
+                gui, enableClasspathModules, honorAutoEager
+                , failOnMessage, level);
         }
 
         private void addLatest(List<Item> newTests) {
@@ -236,7 +387,11 @@ public class NbModuleSuite {
         private Configuration getReady() {
             List<Item> newTests = new ArrayList<Item>(tests);
             addLatest(newTests);
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, newTests, latestTestCaseClass, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                newTests, latestTestCaseClass, reuseUserDir, gui,
+                enableClasspathModules
+            ,honorAutoEager, failOnMessage, failOnException);
         }
         
         /** Should the system run with GUI or without? The default behaviour
@@ -249,7 +404,11 @@ public class NbModuleSuite {
          * @return clone of this configuration with gui mode associated
          */
         public Configuration gui(boolean gui) {
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, tests, latestTestCaseClass, reuseUserDir, gui);
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader,
+                tests, latestTestCaseClass, reuseUserDir, gui,
+                enableClasspathModules
+            ,honorAutoEager, failOnMessage, failOnException);
         }
 
         /**
@@ -259,9 +418,12 @@ public class NbModuleSuite {
          * @since 1.52
          */
         public Configuration reuseUserDir(boolean reuse) {
-            return new Configuration(clusterRegExp, moduleRegExp, parentClassLoader, tests, latestTestCaseClass, reuse, gui);
-        }
+            return new Configuration(
+                clusterRegExp, moduleRegExp, parentClassLoader, tests,
+                latestTestCaseClass, reuse, gui, enableClasspathModules
+            ,honorAutoEager, failOnMessage, failOnException);
         }
+    }
 
     /** Factory method to create wrapper test that knows how to setup proper
      * NetBeans Runtime Container environment. 
@@ -325,7 +487,8 @@ public class NbModuleSuite {
      * in the tested applicationwill be included in the test. 
      * 
      * @param clazz the class with bunch of testXYZ methods
-     * @param tests names of test methods to execute from the <code>clazz</code>
+     * @param tests names of test methods to execute from the <code>clazz</code>, if
+     *    no test methods are specified, all tests in the class are executed
      * @return runtime container ready test
      * @since 1.49
      */
@@ -409,13 +572,99 @@ public class NbModuleSuite {
                 result.addError(this, ex);
             }
         }
+        
+        private static String[] tokenizePath(String path) {
+            List<String> l = new ArrayList<String>();
+            StringTokenizer tok = new StringTokenizer(path, ":;", true); // NOI18N
+            char dosHack = '\0';
+            char lastDelim = '\0';
+            int delimCount = 0;
+            while (tok.hasMoreTokens()) {
+                String s = tok.nextToken();
+                if (s.length() == 0) {
+                    // Strip empty components.
+                    continue;
+                }
+                if (s.length() == 1) {
+                    char c = s.charAt(0);
+                    if (c == ':' || c == ';') {
+                        // Just a delimiter.
+                        lastDelim = c;
+                        delimCount++;
+                        continue;
+                    }
+                }
+                if (dosHack != '\0') {
+                    // #50679 - "C:/something" is also accepted as DOS path
+                    if (lastDelim == ':' && delimCount == 1 && (s.charAt(0) == '\\' || s.charAt(0) == '/')) {
+                        // We had a single letter followed by ':' now followed by \something or /something
+                        s = "" + dosHack + ':' + s;
+                        // and use the new token with the drive prefix...
+                    } else {
+                        // Something else, leave alone.
+                        l.add(Character.toString(dosHack));
+                        // and continue with this token too...
+                    }
+                    dosHack = '\0';
+                }
+                // Reset count of # of delimiters in a row.
+                delimCount = 0;
+                if (s.length() == 1) {
+                    char c = s.charAt(0);
+                    if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
+                        // Probably a DOS drive letter. Leave it with the next component.
+                        dosHack = c;
+                        continue;
+                    }
+                }
+                l.add(s);
+            }
+            if (dosHack != '\0') {
+                //the dosHack was the last letter in the input string (not followed by the ':')
+                //so obviously not a drive letter.
+                //Fix for issue #57304
+                l.add(Character.toString(dosHack));
+            }
+            return l.toArray(new String[l.size()]);
+        }
+
+        static void findClusters(Collection<File> clusters, List<String> regExps) throws IOException {
+            File plat = findPlatform().getCanonicalFile();
+            String selectiveClusters = System.getProperty("cluster.path.final"); // NOI18N
+            Set<File> path = null;
+            if (selectiveClusters != null) {
+                path = new HashSet<File>();
+                for (String p : tokenizePath(selectiveClusters)) {
+                    File f = new File(p);
+                    path.add(f.getCanonicalFile());
+                }
+            }
+            for (String c : regExps) {
+                for (File f : plat.getParentFile().listFiles()) {
+                    if (f.equals(plat)) {
+                        continue;
+                    }
+                    if (!f.getName().matches(c)) {
+                        continue;
+                    }
+                    if (path != null && !path.contains(f)) {
+                        continue;
+                    }
+                    File m = new File(new File(f, "config"), "Modules");
+                    if (m.exists()) {
+                        clusters.add(f);
+                    }
+                }
+            }
+        }
 
         private void runInRuntimeContainer(TestResult result) throws Exception {
+            System.getProperties().remove("netbeans.dirs");
             File platform = findPlatform();
             File[] boot = new File(platform, "lib").listFiles();
             List<URL> bootCP = new ArrayList<URL>();
             for (int i = 0; i < boot.length; i++) {
-                URL u = boot[i].toURL();
+                URL u = boot[i].toURI().toURL();
                 if (u.toExternalForm().endsWith(".jar")) {
                     bootCP.add(u);
                 }
@@ -458,20 +707,24 @@ public class NbModuleSuite {
             System.setProperty("netbeans.user", ud.getPath());
 
             TreeSet<String> modules = new TreeSet<String>();
-            modules.addAll(findEnabledModules(NbTestSuite.class.getClassLoader()));
+            if (config.enableClasspathModules) {
+                modules.addAll(findEnabledModules(NbTestSuite.class.getClassLoader()));
+            }
             modules.add("org.openide.filesystems");
             modules.add("org.openide.modules");
             modules.add("org.openide.util");
             modules.remove("org.netbeans.insane");
             modules.add("org.netbeans.core.startup");
             modules.add("org.netbeans.bootstrap");
-            turnModules(ud, modules, config.moduleRegExp, platform);
-            turnClassPathModules(ud, NbTestSuite.class.getClassLoader());
+            turnModules(ud, !config.honorAutoEager, modules, config.moduleRegExp, platform);
+            if (config.enableClasspathModules) {
+                turnClassPathModules(ud, NbTestSuite.class.getClassLoader());
+            }
 
             StringBuilder sb = new StringBuilder();
             String sep = "";
             for (File f : findClusters()) {
-                turnModules(ud, modules, config.moduleRegExp, f);
+                turnModules(ud, !config.honorAutoEager, modules, config.moduleRegExp, f);
                 sb.append(sep);
                 sb.append(f.getPath());
                 sep = File.pathSeparator;
@@ -491,6 +744,8 @@ public class NbModuleSuite {
             if (!config.gui) {
                 args.add("--nogui");
             }
+
+            Test handler = NbModuleLogHandler.registerBuffer(config.failOnMessage, config.failOnException);
             m.invoke(null, (Object)args.toArray(new String[0]));
 
             ClassLoader global = Thread.currentThread().getContextClassLoader();
@@ -518,6 +773,11 @@ public class NbModuleSuite {
                         toRun.addTest(sndClazz.newInstance());
                     }
                 }
+
+                if (handler != null) {
+                    toRun.addTest(handler);
+                }
+
                 testCount = toRun.countTestCases();
                 toRun.run(result);
             } catch (ClassNotFoundException ex) {
@@ -525,6 +785,9 @@ public class NbModuleSuite {
             } catch (NoClassDefFoundError ex) {
                 result.addError(this, ex);
             }
+            if (handler != null) {
+                NbModuleLogHandler.finish();
+            }
             
             Class<?> lifeClazz = global.loadClass("org.openide.LifecycleManager"); // NOI18N
             Method getDefault = lifeClazz.getMethod("getDefault"); // NOI18N
@@ -540,7 +803,7 @@ public class NbModuleSuite {
             }
         }
 
-        private File findPlatform() {
+        static File findPlatform() {
             try {
                 Class<?> lookup = Class.forName("org.openide.util.Lookup"); // NOI18N
                 File util = new File(lookup.getProtectionDomain().getCodeSource().getLocation().toURI());
@@ -567,34 +830,23 @@ public class NbModuleSuite {
             }
         }
 
-        private File[] findClusters() {
+        private File[] findClusters() throws IOException {
             Collection<File> clusters = new LinkedHashSet<File>();
             if (config.clusterRegExp != null) {
-                File plat = findPlatform();
-
-                for (File f : plat.getParentFile().listFiles()) {
-                    if (f.equals(plat)) {
-                        continue;
-                    }
-                    if (!f.getName().matches(config.clusterRegExp)) {
-                        continue;
-                    }
-                    File m = new File(new File(f, "config"), "Modules");
-                    if (m.exists()) {
-                        clusters.add(f);
-                    }
-                }
+                findClusters(clusters, config.clusterRegExp);
             }
-            
-            // find "cluster" from
-            // k/o.n.m.a.p.N/csam/testModule/build/cluster/modules/org-example-testModule.jar
-            // tested in apisupport.project
-            for (String s : System.getProperty("java.class.path").split(File.pathSeparator)) {
-                File module = new File(s);
-                File cluster = module.getParentFile().getParentFile();
-                File m = new File(new File(cluster, "config"), "Modules");
-                if (m.exists() || cluster.getName().equals("cluster")) {
-                    clusters.add(cluster);
+
+            if (config.enableClasspathModules) {
+                // find "cluster" from
+                // k/o.n.m.a.p.N/csam/testModule/build/cluster/modules/org-example-testModule.jar
+                // tested in apisupport.project
+                for (String s : tokenizePath(System.getProperty("java.class.path"))) {
+                    File module = new File(s);
+                    File cluster = module.getParentFile().getParentFile();
+                    File m = new File(new File(cluster, "config"), "Modules");
+                    if (m.exists() || cluster.getName().equals("cluster")) {
+                        clusters.add(cluster);
+                    }
                 }
             }
             return clusters.toArray(new File[0]);
@@ -633,6 +885,9 @@ public class NbModuleSuite {
                         throw new IllegalStateException("Cannot find version:\n" + manifest);
                     }
                     File jar = jarFromURL(url);
+                    if (jar == null) {
+                        continue;
+                    }
                     if (jar.getParentFile().getName().equals("lib")) {
                         // Otherwise will get DuplicateException.
                         continue;
@@ -668,7 +923,11 @@ public class NbModuleSuite {
             if (m.matches()) {
                 return new File(URI.create(m.group(1)));
             } else {
-                throw new IllegalStateException(u.toExternalForm());
+                if (!u.getProtocol().equals("file")) {
+                    throw new IllegalStateException(u.toExternalForm());
+                } else {
+                    return null;
+                }
             }
         }
 
@@ -677,7 +936,7 @@ public class NbModuleSuite {
             Pattern tests = Pattern.compile(".*\\" + File.separator + "([^\\" + File.separator + "]+)\\" + File.separator + "tests\\.jar");
             StringBuilder sb = new StringBuilder();
             String sep = "";
-            for (String jar : path.split(File.pathSeparator)) {
+            for (String jar : tokenizePath(path)) {
                 Matcher m = tests.matcher(jar);
                 if (m.matches()) {
                     // in case we need it one day, let's add a switch to Configuration
@@ -734,6 +993,9 @@ public class NbModuleSuite {
                 if (isUnit(name)) {
                     return junit.getResource(name);
                 }
+                if (name.equals("META-INF/services/java.util.logging.Handler")) { // NOI18N
+                    return junit.getResource("org/netbeans/junit/internal/FakeMetaInf.txt"); // NOI18N
+                }
                 return super.findResource(name);
             }
 
@@ -742,6 +1004,9 @@ public class NbModuleSuite {
                 if (isUnit(name)) {
                     return junit.getResources(name);
                 }
+                if (name.equals("META-INF/services/java.util.logging.Handler")) { // NOI18N
+                    return junit.getResources("org/netbeans/junit/internal/FakeMetaInf.txt"); // NOI18N
+                }
                 return super.findResources(name);
             }
 
@@ -764,46 +1029,76 @@ public class NbModuleSuite {
 
         private static Pattern ENABLED = Pattern.compile("<param name=[\"']enabled[\"']>([^<]*)</param>", Pattern.MULTILINE);
         private static Pattern AUTO = Pattern.compile("<param name=[\"']autoload[\"']>([^<]*)</param>", Pattern.MULTILINE);
+        private static Pattern EAGER = Pattern.compile("<param name=[\"']eager[\"']>([^<]*)</param>", Pattern.MULTILINE);
         
-        private static void turnModules(File ud, TreeSet<String> modules, String regExp, File... clusterDirs) throws IOException {
+        private static void turnModules(File ud, boolean autoloads, TreeSet<String> modules, List<String> regExp, File... clusterDirs) throws IOException {
+            if (regExp == null) {
+                return;
+            }
             File config = new File(new File(ud, "config"), "Modules");
             config.mkdirs();
 
-            Pattern modPattern = regExp == null ? null : Pattern.compile(regExp);
-            for (File c : clusterDirs) {
-                File modulesDir = new File(new File(c, "config"), "Modules");
-                File[] allModules = modulesDir.listFiles();
-                if (allModules == null) {
-                    continue;
+            Iterator<String> it = regExp.iterator();
+            for (;;) {
+                if (!it.hasNext()) {
+                    break;
                 }
-                for (File m : allModules) {
-                    String n = m.getName();
-                    if (n.endsWith(".xml")) {
-                        n = n.substring(0, n.length() - 4);
-                    }
-                    n = n.replace('-', '.');
-
-                    String xml = asString(new FileInputStream(m), true);
-                    
-                    boolean contains = modules.contains(n);
-                    if (!contains && modPattern != null) {
-                        contains = modPattern.matcher(n).matches();
-                    }
-                    if (!contains) {
+                String clusterReg = it.next();
+                String moduleReg = it.next();
+                Pattern modPattern = Pattern.compile(moduleReg);
+                for (File c : clusterDirs) {
+                    if (!c.getName().matches(clusterReg)) {
                         continue;
                     }
-                    enableModule(xml, contains, new File(config, m.getName()));
+
+                    File modulesDir = new File(new File(c, "config"), "Modules");
+                    File[] allModules = modulesDir.listFiles();
+                    if (allModules == null) {
+                        continue;
+                    }
+                    for (File m : allModules) {
+                        String n = m.getName();
+                        if (n.endsWith(".xml")) {
+                            n = n.substring(0, n.length() - 4);
+                        }
+                        n = n.replace('-', '.');
+
+                        String xml = asString(new FileInputStream(m), true);
+
+                        boolean contains = modules.contains(n);
+                        if (!contains && modPattern != null) {
+                            contains = modPattern.matcher(n).matches();
+                        }
+                        if (!contains) {
+                            continue;
+                        }
+                        enableModule(xml, autoloads, contains, new File(config, m.getName()));
+                    }
                 }
             }
         }
         
-        private static void enableModule(String xml, boolean enable, File target) throws IOException {
+        private static void enableModule(String xml, boolean autoloads, boolean enable, File target) throws IOException {
             boolean toEnable = false;
             {
-                Matcher matcherEnabled = ENABLED.matcher(xml);
+                  Matcher matcherEnabled = ENABLED.matcher(xml);
                 if (matcherEnabled.find()) {
                     toEnable = "false".equals(matcherEnabled.group(1));
                 }
+                Matcher matcherEager = EAGER.matcher(xml);
+                if (matcherEager.find()) {
+                    if ("true".equals(matcherEager.group(1))) {
+                        return;
+                    }
+                }
+                if (!autoloads) {
+                    Matcher matcherAuto = AUTO.matcher(xml);
+                    if (matcherAuto.find()) {
+                        if ("true".equals(matcherAuto.group(1))) {
+                            return;
+                        }
+                    }
+                }
                 if (toEnable) {
                     assert matcherEnabled.groupCount() == 1 : "Groups: " + matcherEnabled.groupCount() + " for:\n" + xml;
                     try {
@@ -831,6 +1126,13 @@ public class NbModuleSuite {
         }
 
         private static void writeModule(File file, String xml) throws IOException {
+            if (file.exists()) {
+                String previous = asString(new FileInputStream(file), true);
+                if (previous.equals(xml)) {
+                    return;
+                }
+            }
+
             FileOutputStream os = new FileOutputStream(file);
             os.write(xml.getBytes("UTF-8"));
             os.close();
