@@ -43,14 +43,16 @@ package org.netbeans;
 
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.IOException;
+import java.io.InputStream;
 import java.security.AllPermission;
 import java.security.CodeSource;
 import java.security.PermissionCollection;
 import java.security.Permissions;
 import java.util.ArrayList;
-import java.util.Collection;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -334,7 +336,7 @@ public final class ModuleManager {
 
     /** Get a classloader capable of loading from any
      * of the enabled modules or their declared extensions.
-     * Should be used as the result of TopManager.systemClassLoader.
+     * Normally used as {@link Thread#getContextClassLoader}.
      * Thread-safe.
      * @see #PROP_CLASS_LOADER
      */
@@ -419,47 +421,17 @@ public final class ModuleManager {
     }
 
     /** A classloader giving access to all the module classloaders at once. */
-    private static final class SystemClassLoader extends JarClassLoader {
+    private final class SystemClassLoader extends JarClassLoader {
 
         private final PermissionCollection allPermissions;
-        private final StringBuffer debugme;
-        private boolean empty = true;
+        int size;
 
         public SystemClassLoader(List<File> files, ClassLoader[] parents, Set<Module> modules) throws IllegalArgumentException {
             super(files, parents, false);
             allPermissions = new Permissions();
             allPermissions.add(new AllPermission());
             allPermissions.setReadOnly();
-            debugme = new StringBuffer(100 + 50 * modules.size());
-            debugme.append("SystemClassLoader["); // NOI18N
-            for (File file : files) {
-                if (empty) {
-                    empty = false;
-                } else {
-                    debugme.append(','); // NOI18N
-                }
-                debugme.append(file.getAbsolutePath());
-            }
-            record(modules);
-            debugme.append(']'); // NOI18N
-        }
-
-        private void record(Collection<Module> modules) {
-	    for (Module m: modules) {
-                if (empty) {
-                    empty = false;
-                } else {
-                    debugme.append(','); // NOI18N
-                }
-                debugme.append(m.getCodeNameBase());
-            }
-        }
-
-        public void append(ClassLoader[] ls, List<Module> modules) throws IllegalArgumentException {
-            super.append(ls);
-            debugme.deleteCharAt(debugme.length() - 1);
-            record(modules);
-            debugme.append(']'); // NOI18N
+            size = modules.size();
         }
 
         protected @Override void finalize() throws Throwable {
@@ -468,10 +440,7 @@ public final class ModuleManager {
         }
 
         public @Override String toString() {
-            if (debugme == null) {
-                return "SystemClassLoader";
-            }
-            return debugme.toString();
+            return "SystemClassLoader[" + size + " modules]";
         }
 
         /** Provide all permissions for any code loaded from the files list
@@ -481,6 +450,29 @@ public final class ModuleManager {
             return allPermissions;
         }
 
+        private final Set<String> JRE_PROVIDED_FACTORIES = new HashSet<String>(Arrays.asList(
+                "META-INF/services/javax.xml.parsers.SAXParserFactory", // NOI18N
+                "META-INF/services/javax.xml.parsers.DocumentBuilderFactory", // NOI18N
+                "META-INF/services/javax.xml.transform.TransformerFactory", // NOI18N
+                "META-INF/services/javax.xml.validation.SchemaFactory")); // NOI18N
+        @Override
+        public InputStream getResourceAsStream(String name) {
+            if (JRE_PROVIDED_FACTORIES.contains(name)) {
+                // #146082: prefer JRE versions of JAXP factories when available.
+                // #147082: use empty file rather than null (~ delegation to ClassLoader.systemClassLoader) to work around JAXP #6723276
+                return new ByteArrayInputStream(new byte[0]);
+            } else {
+                return super.getResourceAsStream(name);
+            }
+        }
+
+        protected @Override boolean shouldDelegateResource(String pkg, ClassLoader parent) {
+            if ((parent == null || parent instanceof MainImpl.BootClassLoader) && !installer.shouldDelegateClasspathResource(pkg)) {
+                return false;
+            }
+            return super.shouldDelegateResource(pkg, parent);
+        }
+
     }
 
     /** @see #create(File,Object,boolean,boolean,boolean)
@@ -492,17 +484,18 @@ public final class ModuleManager {
     }
 
     /** Create a module from a JAR and add it to the managed set.
-     * Will initially be disabled, unless it is eager and can
-     * be enabled immediately.
-     * May throw an IOException if the JAR file cannot be opened
+     * <p>Will initially be disabled.
+     * To make sure all available eager modules get enabled, just call:
+     * {@link #enable}({@link Collections#emptySet})
+     * <p>May throw an IOException if the JAR file cannot be opened
      * for some reason, or is malformed.
-     * If there is already a module of the same name managed,
+     * <p>If there is already a module of the same name managed,
      * throws a duplicate exception. In this case you may wish
      * to delete the original and try again.
-     * You must give it some history object which can be used
+     * <p>You must give it some history object which can be used
      * to provide context for where the module came from and
      * whether it has been here before.
-     * You cannot request that a module be both autoload and eager.
+     * <p>You cannot request that a module be both autoload and eager.
      */
     public Module create(File jar, Object history, boolean reloadable, boolean autoload, boolean eager) throws IOException, DuplicateException {
         assertWritable();
@@ -511,26 +504,6 @@ public final class ModuleManager {
                         history, reloadable, autoload, eager, this, ev);
         ev.log(Events.FINISH_CREATE_REGULAR_MODULE, jar);
         subCreate(m);
-        if (m.isEager()) {
-            List<Module> immediate = simulateEnable(Collections.<Module>emptySet());
-            if (!immediate.isEmpty()) {
-                if (!immediate.contains(m)) {
-                    throw new IllegalStateException("Can immediately enable modules " + immediate + ", but not including " + m + "; its problems: " + m.getProblems()); // NOI18N
-                }
-                boolean ok = true;
-		for (Module other: immediate) {
-                    if (!other.isAutoload() && !other.isEager()) {
-                        // Nope, would require a real module to be turned on first.
-                        ok = false;
-                        break;
-                    }
-                }
-                if (ok) {
-                    Util.err.fine("Enabling " + m + " immediately");
-                    enable(Collections.<Module>emptySet());
-                }
-            }
-        }
         return m;
     }
 
@@ -560,6 +533,9 @@ public final class ModuleManager {
     void refineDependencies(Module m, Set<Dependency> dependencies) {
         installer.refineDependencies(m, dependencies);
     }
+    Set<Dependency> loadDependencies(String cnb) {
+        return installer.loadDependencies(cnb);
+    }
     /** Allows the installer to add provides (used to provide name of platform we run on)
      */
     String[] refineProvides (Module m) {
@@ -631,7 +607,7 @@ public final class ModuleManager {
     }
 
     private void subCreate(Module m) throws DuplicateException {
-        Util.err.fine("created: " + m);
+        Util.err.log(Level.FINE, "created: {0}", m);
         Module old = get(m.getCodeNameBase());
         if (old != null) {
             throw new DuplicateException(old, m);
@@ -741,6 +717,7 @@ public final class ModuleManager {
     /** Enable a single module.
      * Must have satisfied its dependencies.
      * Must not be an autoload module, when supported.
+     * @see #enable(Set)
      */
     public final void enable(Module m) throws IllegalArgumentException, InvalidException {
         enable(Collections.singleton(m));
@@ -749,25 +726,27 @@ public final class ModuleManager {
     /** Disable a single module.
      * Must not be required by any enabled modules.
      * Must not be an autoload module, when supported.
+     * @see #disable(Set)
      */
     public final void disable(Module m) throws IllegalArgumentException {
         disable(Collections.singleton(m));
     }
 
     /** Enable a set of modules together.
-     * Must have satisfied their dependencies
+     * <p>Must have satisfied their dependencies
      * (possibly with one another).
-     * Must not contain autoload nor eager modules.
+     * <p>Must not contain autoload nor eager modules.
      * Might contain fixed modules (they can only be installed once of course).
-     * It is permissible to pass in modules which in fact at runtime cannot
+     * Other modules may become enabled automatically according to {@link #simulateEnable}.
+     * <p>It is permissible to pass in modules which in fact at runtime cannot
      * satisfy their package dependencies, or which {@link ModuleInstaller#prepare}
-     * rejects on the basis of missing contents. In such a case InvalidException
-     * will be thrown and nothing will be installed. The InvalidException in such
+     * rejects on the basis of missing contents. In such a case {@link InvalidException}
+     * will be thrown and nothing will be installed. The {@link InvalidException} in such
      * a case should contain a reference to the offending module.
      */
     public void enable(Set<Module> modules) throws IllegalArgumentException, InvalidException {
         assertWritable();
-        Util.err.fine("enable: " + modules);
+        Util.err.log(Level.FINE, "enable: {0}", modules);
         /* Consider eager modules:
         if (modules.isEmpty()) {
             return;
@@ -851,7 +830,8 @@ public final class ModuleManager {
                         }
                         if (! Util.checkPackageDependency(dep, m.getClassLoader())) {
                             failedPackageDep = dep;
-                            throw new InvalidException(m, "Dependency failed on " + dep); // NOI18N
+                            String polite = (String)m.getLocalizedAttribute("OpenIDE-Module-Package-Dependency-Message"); // NOI18N
+                            throw new InvalidException(m, "Dependency failed on " + dep, polite); // NOI18N
                         }
                         Util.err.fine("Successful check for: " + dep);
                     }
@@ -928,7 +908,8 @@ public final class ModuleManager {
                         }
                     }
                 }
-                classLoader.append((nueclassloaders.toArray(new ClassLoader[nueclassloaders.size()])), toEnable);
+                classLoader.append((nueclassloaders.toArray(new ClassLoader[nueclassloaders.size()])));
+                classLoader.size += toEnable.size();
             } else {
                 Util.err.fine("enable: no class loader yet, not appending");
             }
@@ -952,10 +933,11 @@ public final class ModuleManager {
     }
 
     /** Disable a set of modules together.
-     * Must not be required by any enabled
+     * <p>Must not be required by any enabled
      * modules (except one another).
-     * Must not contain autoload nor eager modules.
+     * <p>Must not contain autoload nor eager modules.
      * Must not contain fixed modules.
+     * Other modules may become disabled automatically according to {@link #simulateDisable}.
      */
     public void disable(Set<Module> modules) throws IllegalArgumentException {
         assertWritable();
