@@ -63,6 +63,7 @@ import org.netbeans.junit.MockServices;
 import org.netbeans.junit.NbTestCase;
 import org.openide.util.HelpCtx;
 import org.openide.util.Lookup;
+import org.openide.util.Utilities;
 import org.openide.util.actions.SystemAction;
 
 /**
@@ -345,12 +346,13 @@ public class ActionsTest extends NbTestCase {
         assertTrue(button.getToolTipText().equals(TestActionWithTooltip.TOOLTIP));
         
         action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_DOWN_MASK));
-        
-        assertTrue(button.getToolTipText().indexOf("Ctrl+C") != (-1));
+
+        String ctrlMod = Utilities.isMac() ? "\u2303" : "Ctrl";
+        assertTrue(button.getToolTipText().indexOf(ctrlMod + "+C") != (-1));
         
         action.putValue(Action.SHORT_DESCRIPTION, null);
         
-        assertTrue(button.getToolTipText().indexOf("Ctrl+C") != (-1));
+        assertTrue(button.getToolTipText().indexOf(ctrlMod + "+C") != (-1));
         
         f.setVisible(false);
     }
@@ -394,7 +396,7 @@ public class ActionsTest extends NbTestCase {
         
         Actions.connect(item, action, true);
         
-        assertEquals('A', item.getMnemonic());
+        assertEquals(Utilities.isMac() ? 0 : 'A', item.getMnemonic());
         assertEquals("Ahoj", item.getText());
     }
 
@@ -413,7 +415,7 @@ public class ActionsTest extends NbTestCase {
         
         Actions.connect(item, action, true);
         
-        assertEquals('B', item.getMnemonic());
+        assertEquals(Utilities.isMac() ? 0 : 'B', item.getMnemonic());
         assertEquals("Ble", item.getText());
     }
     
@@ -432,7 +434,7 @@ public class ActionsTest extends NbTestCase {
         
         Actions.connect(item, action, true);
         
-        assertEquals('M', item.getMnemonic());
+        assertEquals(Utilities.isMac() ? 0 : 'M', item.getMnemonic());
         assertEquals("Mle", item.getText());
     }
     
