@@ -51,6 +51,8 @@ import java.beans.*;
 import java.io.*;
 
 import java.util.*;
+import java.util.ArrayList;
+import javax.swing.SwingUtilities;
 
 
 /**
@@ -146,6 +148,9 @@ public final class ExplorerManager extends Object implements Serializable, Clone
     /** Actions factory provided for this explorer manager */
     private ExplorerActionsImpl actions;
 
+    /** private lock */
+    private transient Object LOCK;
+
     /** Construct a new manager. */
     public ExplorerManager() {
         init();
@@ -158,6 +163,7 @@ public final class ExplorerManager extends Object implements Serializable, Clone
         selectedNodes = new Node[0];
         listener = new Listener();
         weakListener = NodeOp.weakNodeListener(listener, null);
+        LOCK = new Object();
     }
 
     /** Clones the manager.
@@ -204,8 +210,7 @@ public final class ExplorerManager extends Object implements Serializable, Clone
     /** Set the set of selected nodes.
     * @param value the nodes to select; empty (not <code>null</code>) if none are to be selected
     * @exception PropertyVetoException when the given nodes cannot be selected
-    * @throws IllegalArgumentException if <code>null</code> is given, or if any elements
-    *                                  of the selection are not within the current root context
+    * @throws IllegalArgumentException if <code>null</code> is given (array or any element in array)
     */
     public final void setSelectedNodes(final Node[] value)
     throws PropertyVetoException {
@@ -213,43 +218,60 @@ public final class ExplorerManager extends Object implements Serializable, Clone
             public PropertyVetoException veto;
             private boolean doFire;
             private Node[] oldValue;
+            private Node[] newValue;
 
-            /** @return false if no further processing is needed */
-            private boolean checkArgumentIsValid() {
+            /** selects only nodes under root context */
+            private void checkAndSet() {
                 if (value == null) {
                     throw new IllegalArgumentException(getString("EXC_NodeCannotBeNull"));
                 }
 
                 if (equalNodes(value, selectedNodes)) {
-                    return false;
+                    return;
                 }
 
+                List<Node> validNodes = null;
                 for (int i = 0; i < value.length; i++) {
                     if (value[i] == null) {
                         throw new IllegalArgumentException(getString("EXC_NoElementOfNodeSelectionMayBeNull"));
                     }
-
-                    checkUnderRoot(value[i], "EXC_NodeSelectionCannotContainNodes");
-                }
-
-                if ((value.length != 0) && (vetoableSupport != null)) {
-                    try {
-                        // we send the vetoable change event only for non-empty selections
-                        vetoableSupport.fireVetoableChange(PROP_SELECTED_NODES, selectedNodes, value);
-                    } catch (PropertyVetoException ex) {
-                        veto = ex;
-
-                        return false;
+                    
+                    if (!isUnderRoot(value[i])) {
+                        if (validNodes == null) {
+                            validNodes = new ArrayList<Node>(value.length);
+                            for (int j = 0; j < i; j++) {
+                                validNodes.add(value[i]);
+                            }
+                        }
+                    } else if (validNodes != null) {
+                        validNodes.add(value[i]);
                     }
                 }
+                if (validNodes != null) {
+                    newValue = validNodes.toArray(new Node[validNodes.size()]);
+                    if (equalNodes(newValue, selectedNodes)) {
+                        return;
+                    }                    
+                } else {
+                    newValue = value;
+                }
 
-                return true;
+                if ((newValue.length != 0) && (vetoableSupport != null)) {
+                    try {
+                        // we send the vetoable change event only for non-empty selections
+                        vetoableSupport.fireVetoableChange(PROP_SELECTED_NODES, selectedNodes, newValue);
+                    } catch (PropertyVetoException ex) {
+                        veto = ex;
+                        return;
+                    }
+                }
+                updateSelection();
             }
 
             private void updateSelection() {
                 oldValue = selectedNodes;
                 addRemoveListeners(false);
-                selectedNodes = value;
+                selectedNodes = newValue;
                 addRemoveListeners(true);
 
                 doFire = true;
@@ -262,9 +284,7 @@ public final class ExplorerManager extends Object implements Serializable, Clone
             }
 
             public void run() {
-                if (checkArgumentIsValid()) {
-                    updateSelection();
-                }
+                checkAndSet();
             }
         }
 
@@ -450,32 +470,34 @@ public final class ExplorerManager extends Object implements Serializable, Clone
             throw new IllegalArgumentException(getString("EXC_CannotHaveNullRootContext"));
         }
 
-        if (rootContext.equals(value)) {
-            return;
-        }
-
-        class SetRootContext implements Runnable {
-            public void run() {
-                addRemoveListeners(false);
-                Node oldValue = rootContext;
-                rootContext = value;
-
-                oldValue.removeNodeListener(weakListener);
-                rootContext.addNodeListener(weakListener);
-
-                fireInAWT(PROP_ROOT_CONTEXT, oldValue, rootContext);
-
-                Node[] newselection = getSelectedNodes();
-
-                if (!areUnderTarget(newselection, rootContext)) {
-                    newselection = new Node[0];
-                }
-                setExploredContext(rootContext, newselection);
+        synchronized (LOCK) {
+            if (rootContext.equals(value)) {
+                return;
             }
+
+            class SetRootContext implements Runnable {
+                public void run() {
+                    addRemoveListeners(false);
+                    Node oldValue = rootContext;
+                    rootContext = value;
+
+                    oldValue.removeNodeListener(weakListener);
+                    rootContext.addNodeListener(weakListener);
+
+                    fireInAWT(PROP_ROOT_CONTEXT, oldValue, rootContext);
+
+                    Node[] newselection = getSelectedNodes();
+
+                    if (!areUnderTarget(newselection, rootContext)) {
+                        newselection = new Node[0];
+                    }
+                    setExploredContext(rootContext, newselection);
+                }
+            }
+
+            SetRootContext run = new SetRootContext();
+            Children.MUTEX.readAccess(run);
         }
-        
-        SetRootContext run = new SetRootContext();
-        Children.MUTEX.readAccess(run);
     }
 
     /** @return true iff all nodes are under the target node */
@@ -826,6 +848,7 @@ bigloop:
      * @return ExplorerActionsImpl
      */
     static synchronized ExplorerActionsImpl findExplorerActionsImpl(ExplorerManager em) {
+        assert em != null;
         if (em.actions == null) {
             em.actions = new ExplorerActionsImpl();
             em.actions.attach(em);
@@ -865,6 +888,7 @@ bigloop:
         public ExplorerManager getExplorerManager();
     }
 
+    static boolean SCHEDULE_REMOVE_ASYNCH = true;
     /** Listener to be notified when root node has been destroyed.
     * Then the root node is changed to Node.EMPTY
     */
@@ -886,7 +910,17 @@ bigloop:
                 setRootContext(Node.EMPTY);
             } else {
                 // assume that the node is among currently selected nodes
-                scheduleRemove(ev.getNode());
+                final Node n = ev.getNode();
+                Runnable r = new Runnable() {
+                    public void run() {
+                        scheduleRemove(n);
+                    }
+                };
+                if (SCHEDULE_REMOVE_ASYNCH) {
+                    SwingUtilities.invokeLater(r);
+                } else {
+                    r.run();
+                }
             }
         }
 
