@@ -73,6 +73,8 @@ import javax.swing.plaf.basic.BasicSplitPaneUI;
 import javax.swing.plaf.metal.*;
 
 import org.netbeans.modules.openide.explorer.UIException;
+import org.openide.DialogDisplayer;
+import org.openide.NotifyDescriptor;
 
 
 /** A few utility methods useful to implementors of Inplace Editors.
@@ -255,6 +257,7 @@ final class PropUtils {
                 return s1.compareToIgnoreCase(s2);
             }
 
+        @Override
             public String toString() {
                 return "Type comparator"; //NOI18N
             }
@@ -269,6 +272,7 @@ final class PropUtils {
                 return String.CASE_INSENSITIVE_ORDER.compare(s1, s2);
             }
 
+        @Override
             public String toString() {
                 return "Name comparator"; //NOI18N
             }
@@ -524,7 +528,7 @@ final class PropUtils {
         if (o instanceof Exception) {
             if( o instanceof InvocationTargetException )
                 o = ((InvocationTargetException)o).getTargetException();
-            processThrowable((Exception) o, title, newValue);
+            processThrowable((Throwable) o, title, newValue);
         }
 
         boolean result = (o instanceof Boolean) ? ((Boolean) o).booleanValue() : false;
@@ -586,7 +590,15 @@ final class PropUtils {
 
         try {
             if (value instanceof String) {
-                ed.setAsText((String) value);
+                try {
+                    ed.setAsText((String) value);
+                } catch( IllegalArgumentException iaE ) {
+                    //#137706 - always treat iae from setAsText as a an invalid
+                    //user input instead of broken code and display nice error message to the user
+                    if( null == Exceptions.findLocalizedMessage(iaE) )
+                        Exceptions.attachLocalizedMessage(iaE, NbBundle.getMessage(PropUtils.class, "MSG_SetAsText_InvalidValue", value));
+                    result = iaE;
+                }
             } else {
                 ed.setValue(value);
             }
@@ -668,7 +680,9 @@ final class PropUtils {
                                      null, null);
         }
 
-        Exceptions.printStackTrace(throwable);
+        String msg = Exceptions.findLocalizedMessage(throwable);
+        NotifyDescriptor d = new NotifyDescriptor.Message(msg, NotifyDescriptor.INFORMATION_MESSAGE);
+        DialogDisplayer.getDefault().notifyLater(d);
     }
 
     /** Fetches a localized message for an exception that may be displayed to
@@ -1009,26 +1023,25 @@ final class PropUtils {
                 UIManager.getLookAndFeel().getClass().getName()
             );
 
-        boolean aqua = "Aqua".equals(UIManager.getLookAndFeel().getID());
+        boolean nimbus = "Nimbus".equals(UIManager.getLookAndFeel().getID());
+
+        boolean gtk = "GTK".equals(UIManager.getLookAndFeel().getID());
 
         setRendererColor = UIManager.getColor(KEY_SETBG); //NOI18N
         selectedSetRendererColor = UIManager.getColor(KEY_SELSETBG); //NOI18N
 
-        if (setRendererColor == null) {
-            if (aqua) {
-                setRendererColor = new Color(225, 235, 240);
-            } else {
-                if (setRendererColor == null) {
-                    red = adjustColorComponent(controlColor.getRed(), -25, -25);
-                    green = adjustColorComponent(controlColor.getGreen(), -25, -25);
-                    blue = adjustColorComponent(controlColor.getBlue(), -25, -25);
-                    setRendererColor = new Color(red, green, blue);
-                }
-            }
+        if( nimbus || gtk ) {
+            setRendererColor = UIManager.getColor( "Menu.background" );//NOI18N
+            selectedSetRendererColor = UIManager.getColor("Tree.selectionBackground"); //NOI18N
         }
 
-        if (aqua) {
-            selectedSetRendererColor = UIManager.getColor("Table.selectionBackground");
+        if (setRendererColor == null) {
+            if (setRendererColor == null) {
+                red = adjustColorComponent(controlColor.getRed(), -25, -25);
+                green = adjustColorComponent(controlColor.getGreen(), -25, -25);
+                blue = adjustColorComponent(controlColor.getBlue(), -25, -25);
+                setRendererColor = new Color(red, green, blue);
+            }
         }
 
         if (selectedSetRendererColor == null) {
@@ -1053,6 +1066,9 @@ final class PropUtils {
 
         setForegroundColor = UIManager.getColor(KEY_SETFG);
 
+        if( nimbus || gtk )
+            setForegroundColor = new Color( UIManager.getColor( "Menu.foreground" ).getRGB() ); //NOI18N
+
         if (setForegroundColor == null) {
             setForegroundColor = UIManager.getColor("Table.foreground"); //NOI18N
 
@@ -1702,11 +1718,13 @@ final class PropUtils {
     }
 
     private static class CleanSplitPaneUI extends BasicSplitPaneUI {
+        @Override
         protected void installDefaults() {
             super.installDefaults();
             divider.setBorder(new SplitBorder());
         }
         
+        @Override
         public BasicSplitPaneDivider createDefaultDivider() {
             return new CleanSplitPaneDivider(this);
         }
@@ -1718,9 +1736,11 @@ final class PropUtils {
         public CleanSplitPaneDivider( BasicSplitPaneUI ui ) {
             super( ui );
         }
+        @Override
         public AccessibleContext getAccessibleContext() {
             if( null == accessibleContext ) {
                 accessibleContext = new AccessibleAWTComponent() {
+                    @Override
                             public AccessibleRole getAccessibleRole() {
                                 return AccessibleRole.SPLIT_PANE;
                             }
