@@ -314,9 +314,6 @@ public class PropertySheet extends JPanel {
         table.getReusablePropertyEnv().setBeans(null);
         table.getReusablePropertyEnv().setNode(null);
         table.getReusablePropertyModel().setProperty(null);
-        
-        //don't hold anything when not in component hierarchy
-        helperNodes = null;
     }
 
     /** Prepare the initial state of the property sheet */
@@ -437,6 +434,7 @@ public class PropertySheet extends JPanel {
         if ((nodes == null) || (nodes.length == 0)) {
             table.getPropertySetModel().setPropertySets(null);
             table.getReusablePropertyEnv().clear();
+            psheet.setTabbedContainerItems(new Object[0], new String[0]);
             return;
         }
 
@@ -451,7 +449,7 @@ public class PropertySheet extends JPanel {
         final boolean loggable = PropUtils.isLoggable(PropertySheet.class);
 
         if (loggable) {
-            PropUtils.log(PropertySheet.class, "SetNodes " + Arrays.asList(nodes));
+            PropUtils.log(PropertySheet.class, "SetNodes " + (null == nodes ? "<null>" : Arrays.asList(nodes)));
         }
 
         //Performance - check equality and avoid some extra repaints - repainting
@@ -524,6 +522,7 @@ public class PropertySheet extends JPanel {
                 table.getPropertySetModel().setPropertySets(null);
                 table.getReusablePropertyEnv().clear();
                 helperNodes = null;
+                psheet.setTabbedContainerItems(new Object[0], new String[0]);
             } else {
                 SwingUtilities.invokeLater(
                     new Runnable() {
@@ -538,6 +537,7 @@ public class PropertySheet extends JPanel {
                             table.getPropertySetModel().setPropertySets(null);
                             table.getReusablePropertyEnv().clear();
                             helperNodes = null;
+                            psheet.setTabbedContainerItems(new Object[0], new String[0]);
                         }
                     }
                 );
@@ -585,16 +585,16 @@ public class PropertySheet extends JPanel {
                                 new Runnable() {
                                     public void run() {
                                         final boolean loggable = PropUtils.isLoggable(PropertySheet.class);
-
+                                        Node[] nodesToSet = helperNodes;
                                         if (loggable) {
                                             PropUtils.log(
                                                 PropertySheet.class,
                                                 "Delayed " + "updater setting nodes to " + //NOI18N
-                                                    (null == helperNodes ? "null" : Arrays.asList(helperNodes)) //NOI18N
+                                                    (null == nodesToSet ? "null" : Arrays.asList(nodesToSet)) //NOI18N
                                             );
                                         }
 
-                                        doSetNodes(helperNodes);
+                                        doSetNodes(nodesToSet);
                                     }
                                 }
                             );
@@ -664,7 +664,8 @@ public class PropertySheet extends JPanel {
 
             psheet.setTabbedContainerItems(info.sets, info.titles);
             psheet.manager().setCurrentNodeName(node.getName());
-            psm.setPropertySets(info.getSets(0));
+            if( info.sets.length > 0 )
+                psm.setPropertySets(info.getSets(0));
         } else {
             psm.setPropertySets(ps);
             psheet.setState(
