@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -24,7 +24,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -47,6 +47,8 @@ import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.MouseEvent;
 import java.awt.dnd.DnDConstants;
+import java.awt.event.ActionListener;
+import java.awt.event.KeyEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyEditor;
 import java.beans.PropertyVetoException;
@@ -54,8 +56,12 @@ import java.beans.VetoableChangeListener;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.EventObject;
+import java.util.List;
 import java.util.Properties;
+import java.util.logging.Logger;
+import javax.swing.AbstractButton;
 import javax.swing.Action;
 import javax.swing.BorderFactory;
 import javax.swing.JComponent;
@@ -63,6 +69,7 @@ import javax.swing.JPopupMenu;
 import javax.swing.JScrollPane;
 import javax.swing.JTable;
 import javax.swing.JViewport;
+import javax.swing.KeyStroke;
 import javax.swing.ListSelectionModel;
 import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
@@ -82,6 +89,7 @@ import org.netbeans.swing.outline.Outline;
 import org.netbeans.swing.outline.OutlineModel;
 import org.netbeans.swing.outline.RowModel;
 import org.netbeans.swing.outline.TreePathSupport;
+import org.openide.awt.Mnemonics;
 import org.openide.awt.MouseUtils;
 import org.openide.explorer.ExplorerManager;
 import org.openide.explorer.propertysheet.PropertyPanel;
@@ -143,6 +151,10 @@ public class OutlineView extends JScrollPane {
     transient private int allowedDragActions = DnDConstants.ACTION_COPY_OR_MOVE | DnDConstants.ACTION_REFERENCE;
     transient private int allowedDropActions = DnDConstants.ACTION_COPY_OR_MOVE | DnDConstants.ACTION_REFERENCE;
 
+    /** Listener on keystroke to invoke default action */
+    private ActionListener defaultTreeActionListener;
+    private final String nodesColumnLabel;
+
     /** Creates a new instance of TableView */
     public OutlineView() {
         this(null);
@@ -153,7 +165,8 @@ public class OutlineView extends JScrollPane {
         treeModel = new NodeTreeModel();
         rowModel = new PropertiesRowModel();
         model = createOutlineModel(treeModel, rowModel, nodesColumnLabel);
-        outline = new OutlineViewOutline(model);
+        this.nodesColumnLabel = nodesColumnLabel;
+        outline = new OutlineViewOutline(model, rowModel);
         rowModel.setOutline(outline);
         outline.setRenderDataProvider(new NodeRenderDataProvider(outline));
         SheetCell tableCell = new SheetCell.OutlineSheetCell(outline);
@@ -175,6 +188,10 @@ public class OutlineView extends JScrollPane {
         popupFactory = new OutlinePopupFactory();
         // activation of drop target
         setDropTarget( DragDropUtilities.dragAndDropEnabled );
+        defaultTreeActionListener = new DefaultTreeAction (outline);
+        outline.registerKeyboardAction(
+            defaultTreeActionListener, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), JComponent.WHEN_FOCUSED
+        );
 
         final Color focusSelectionBackground = outline.getSelectionBackground();
         final Color focusSelectionForeground = outline.getSelectionForeground();
@@ -190,7 +207,7 @@ public class OutlineView extends JScrollPane {
             }
 
         });
-        TableColumnSelector tcs = (TableColumnSelector) Lookup.getDefault().lookup(TableColumnSelector.class);
+        TableColumnSelector tcs = Lookup.getDefault ().lookup (TableColumnSelector.class);
         if (tcs != null) {
             outline.setColumnSelector(tcs);
         }
@@ -211,6 +228,9 @@ public class OutlineView extends JScrollPane {
      * You can override it and create different model in the subclass.
      */
     protected OutlineModel createOutlineModel(NodeTreeModel treeModel, RowModel rowModel, String label) {
+        if (label == null) {
+            label = NbBundle.getMessage(OutlineView.class, "NodeOutlineModel_NodesColumnLabel"); // NOI18N
+        }
         return new NodeOutlineModel(treeModel, rowModel, false, label);
     }
     
@@ -272,6 +292,8 @@ public class OutlineView extends JScrollPane {
     public void addNotify () {
         super.addNotify ();
         lookupExplorerManager ();
+        ViewUtil.adjustBackground(outline);
+        ViewUtil.adjustBackground(getViewport());
     }
     
     /**
@@ -313,7 +335,7 @@ public class OutlineView extends JScrollPane {
     /** Registers in the tree of components.
      */
     private void lookupExplorerManager () {
-        // Enter key in the tree
+    // Enter key in the tree
 
         if (managerListener == null) {
             managerListener = new TableSelectionListener();
@@ -345,13 +367,17 @@ public class OutlineView extends JScrollPane {
     /** Synchronize the root context from the manager of this Explorer.
     */
     final void synchronizeRootContext() {
-        if( null != treeModel )
+        if( null != treeModel ) {
             treeModel.setNode(manager.getRootContext());
+        }
     }
 
     /** Synchronize the selected nodes from the manager of this Explorer.
      */
-    final void synchronizeSelectedNodes(boolean scroll) {
+    final void synchronizeSelectedNodes(boolean scroll, Node... nodes) {
+        if (! needToSynchronize ()) {
+            return ;
+        }
         expandSelection();
         outline.invalidate();
         invalidate();
@@ -394,6 +420,24 @@ public class OutlineView extends JScrollPane {
         }
     }
 
+    private boolean needToSynchronize () {
+        boolean doSync = false;
+        Node [] arr = manager.getSelectedNodes ();
+        if (outline.getSelectedRows ().length != arr.length) {
+            doSync = true;
+        } else if (arr.length > 0) {
+            List<Node> nodes = Arrays.asList (arr);
+            for (int idx : outline.getSelectedRows ()) {
+                Node n = getNodeFromRow (idx);
+                if (n == null || ! nodes.contains (n)) {
+                    doSync = true;
+                    break;
+                }
+            }
+        }
+        return doSync;
+    }
+
     /**
      * Tries to expand nodes selected in the explorer manager.
      */
@@ -406,7 +450,7 @@ public class OutlineView extends JScrollPane {
             }
             TreeNode tn = Visualizer.findVisualizer(arr[i]);
             if (tn != null) {
-                ArrayList al = new ArrayList();
+                ArrayList<TreeNode> al = new ArrayList<TreeNode> ();
                 while (tn != null) {
                     al.add(tn);
                     tn = tn.getParent();
@@ -416,7 +460,6 @@ public class OutlineView extends JScrollPane {
                 while ((tp != null) && (tp.getPathCount() > 0)) {
                     tp = tp.getParentPath();
                     if (tp != null) {
-                        System.out.println( "expanding: " + tp );
                         outline.expandPath(tp);
                     }
                 }
@@ -433,6 +476,7 @@ public class OutlineView extends JScrollPane {
         outline.getSelectionModel().removeListSelectionListener(managerListener);
         manager.removePropertyChangeListener (wlpc);
         manager.removeVetoableChangeListener (wlvc);
+        manager = null;
     }
 
     /**
@@ -462,14 +506,14 @@ public class OutlineView extends JScrollPane {
      */
     private JPopupMenu createPopup(Point p) {
         int[] selRows = outline.getSelectedRows();
-        ArrayList al = new ArrayList(selRows.length);
+        ArrayList<Node> al = new ArrayList<Node> (selRows.length);
         for (int i = 0; i < selRows.length; i++) {
             Node n = getNodeFromRow(selRows[i]);
             if (n != null) {
                 al.add(n);
             }
         }
-        Node[] arr = (Node[])al.toArray(new Node[al.size()]);
+        Node[] arr = al.toArray (new Node[al.size ()]);
         if (arr.length == 0) {
             // hack to show something even when no rows are selected
             arr = new Node[] { manager.getRootContext() };
@@ -559,6 +603,34 @@ public class OutlineView extends JScrollPane {
                 e.consume();
             }
         }
+
+        @Override
+        public void mouseClicked (MouseEvent e) {
+            int selRow = outline.rowAtPoint (e.getPoint ());
+
+            if ((selRow != -1) && SwingUtilities.isLeftMouseButton (e) && MouseUtils.isDoubleClick (e)) {
+                // Default action.
+                if (outline.getSelectedColumn () == 0) {
+                    TreePath selPath = outline.getClosestPathForLocation (e.getX (), e.getY ());
+                    Node node = Visualizer.findNode (selPath.getLastPathComponent ());
+                    if (node.isLeaf ()) {
+                        Action a = TreeView.takeAction (node.getPreferredAction (), node);
+
+                        if (a != null) {
+                            if (a.isEnabled ()) {
+                                a.actionPerformed (new ActionEvent (node, ActionEvent.ACTION_PERFORMED, "")); // NOI18N
+                            } else {
+                                Logger.getLogger (OutlineView.class.getName ()).info ("Action " + a + " on node " + node + " is disabled");
+                            }
+
+                            e.consume ();
+                            return;
+                        }
+                    }
+                }
+            }
+            super.mouseClicked (e);
+        }
     }
 
     /**
@@ -756,6 +828,13 @@ public class OutlineView extends JScrollPane {
             if (manager == null) return; // the tree view has been removed before the event got delivered
             if (evt.getPropertyName().equals(ExplorerManager.PROP_ROOT_CONTEXT)) {
                 synchronizeRootContext();
+                if (nodesColumnLabel == null && model instanceof NodeOutlineModel) {
+                    String oldLabel = ((NodeOutlineModel) model).getColumnName(0);
+                    String newLabel = manager.getRootContext().getDisplayName();
+                    if (oldLabel != null && ! oldLabel.equals(newLabel)) {
+                        ((NodeOutlineModel) model).setNodesColumnLabel(newLabel);
+                    }
+                }
             }
             if (evt.getPropertyName().equals(ExplorerManager.PROP_SELECTED_NODES)) {
                 synchronizeSelectedNodes(true);
@@ -764,14 +843,14 @@ public class OutlineView extends JScrollPane {
 
         public void valueChanged(javax.swing.event.ListSelectionEvent listSelectionEvent) {
             int selectedRows[] = outline.getSelectedRows();
-            ArrayList selectedNodes = new ArrayList(selectedRows.length);
+            ArrayList<Node> selectedNodes = new ArrayList<Node> (selectedRows.length);
             for (int i = 0; i < selectedRows.length;i++) {
                 Node n = getNodeFromRow(selectedRows[i]);
                 if (n != null) {
                     selectedNodes.add(n);
                 }
             }
-            callSelectionChanged((Node[])selectedNodes.toArray(new Node[selectedNodes.size()]));
+            callSelectionChanged(selectedNodes.toArray (new Node[selectedNodes.size ()]));
         }
 
         public void vetoableChange(java.beans.PropertyChangeEvent evt) throws java.beans.PropertyVetoException {
@@ -787,22 +866,81 @@ public class OutlineView extends JScrollPane {
         }
     }
 
+
+    /** Invokes default action.
+     */
+    private class DefaultTreeAction implements ActionListener {
+
+        private Outline outline;
+
+        DefaultTreeAction (Outline outline) {
+            this.outline = outline;
+        }
+
+        /**
+         * Invoked when an action occurs.
+         */
+        public void actionPerformed (ActionEvent e) {
+            if (outline.getSelectedColumn () != 0) {
+                return;
+            }
+
+            Node[] nodes = manager.getSelectedNodes ();
+
+            if (nodes.length == 1) {
+                Action a = nodes[0].getPreferredAction ();
+
+                if (a != null) {
+                    if (a.isEnabled ()) {
+                        a.actionPerformed (new ActionEvent (nodes[0], ActionEvent.ACTION_PERFORMED, "")); // NOI18N
+                    } else {
+                        Logger.getLogger (OutlineView.class.getName ()).info ("Action " + a + " on node " + nodes [0] + " is disabled");
+                    }
+                }
+            }
+        }
+    }
+
     /**
      * Extension of the ETable that allows adding a special comparator
      * for sorting the rows.
      */
     private static class OutlineViewOutline extends Outline {
-        public OutlineViewOutline(OutlineModel mdl) {
+        private final PropertiesRowModel rowModel;
+        private static final String COLUMNS_SELECTOR_HINT = "ColumnsSelectorHint"; // NOI18N
+        public OutlineViewOutline(OutlineModel mdl, PropertiesRowModel rowModel) {
             super(mdl);
-            setSelectVisibleColumnsLabel(NbBundle.getMessage(OutlineView.class, "ACN_ColumnsSelector")); //NOI18N
+            this.rowModel = rowModel;
+            setSelectVisibleColumnsLabel(NbBundle.getMessage(OutlineView.class, "CTL_ColumnsSelector")); //NOI18N
         }
         
         @Override
         public Object transformValue(Object value) {
+            if (value instanceof OutlineViewOutlineColumn) {
+                OutlineViewOutlineColumn c = (OutlineViewOutlineColumn) value;
+                String dn = c.getRawColumnName ();
+                if (dn == null) {
+                    dn = c.getHeaderValue ().toString ();
+                }
+                String desc = c.getShortDescription (null);
+                if (desc == null) {
+                    return dn;
+                }
+                return NbBundle.getMessage (OutlineView.class, "OutlineViewOutline_NameAndDesc", dn, desc); // NOI18N
+            } else if (COLUMNS_SELECTOR_HINT.equals (value)) {
+                return NbBundle.getMessage (OutlineView.class, COLUMNS_SELECTOR_HINT);
+            } else if (value instanceof AbstractButton) {
+                AbstractButton b = (AbstractButton) value;
+                Mnemonics.setLocalizedText (b, b.getText ());
+                return b;
+            } else if (value instanceof VisualizerNode) {
+                return Visualizer.findNode (value);
+            }
             return PropertiesRowModel.getValueFromProperty(value);
         }
-        
+
         @Override
+        @SuppressWarnings("unchecked")
         public boolean editCellAt(int row, int column, EventObject e) {
             Object o = getValueAt(row, column);
             if (o instanceof Node.Property) { // && (e == null || e instanceof KeyEvent)) {
@@ -828,6 +966,7 @@ public class OutlineView extends JScrollPane {
                     Node.Property p = (Node.Property)o;
                     if( !Boolean.TRUE.equals(p.getValue("suppressCustomEditor") ) ) { //NOI18N
                         PropertyPanel panel = new PropertyPanel(p);
+                        @SuppressWarnings("deprecation")
                         PropertyEditor ed = panel.getPropertyEditor();
 
                         if ((ed != null) && ed.supportsCustomEditor()) {
@@ -858,12 +997,15 @@ public class OutlineView extends JScrollPane {
         protected TableColumn createColumn(int modelIndex) {
             return new OutlineViewOutlineColumn(modelIndex);
         }
+
         /**
          * Extension of ETableColumn using TableViewRowComparator as
          * comparator.
          */
         private class OutlineViewOutlineColumn extends OutlineColumn {
             private String tooltip;
+            private final Comparator originalNodeComparator = new NodeNestedComparator ();
+
             public OutlineViewOutlineColumn(int index) {
                 super(index);
             }
@@ -884,12 +1026,46 @@ public class OutlineView extends JScrollPane {
                 }
                 return res;
             }
+
+            @Override
+            public Comparator getNestedComparator () {
+                // it it's the tree column
+                if (getModelIndex () == 0 && super.getNestedComparator () == null) {
+                    return originalNodeComparator;
+                }
+                return super.getNestedComparator ();
+            }
+
             @Override
             protected TableCellRenderer createDefaultHeaderRenderer() {
                 TableCellRenderer orig = super.createDefaultHeaderRenderer();
                 OutlineViewOutlineHeaderRenderer ovohr = new OutlineViewOutlineHeaderRenderer(orig);
                 return ovohr;
             }
+
+            public String getShortDescription (String defaultValue) {
+                TableModel model = getModel();
+                if (model.getRowCount() <= 0) {
+                    return null;
+                }
+                if (getModelIndex () == 0) {
+                    // 1st column
+                    return defaultValue;
+                }
+                return rowModel.getShortDescription (getModelIndex () - 1);
+            }
+
+            public String getRawColumnName () {
+                TableModel model = getModel();
+                if (model.getRowCount() <= 0) {
+                    return null;
+                }
+                if (getModelIndex () == 0) {
+                    return null;
+                }
+                return rowModel.getRawColumnName (getModelIndex () - 1);
+            }
+
             /** This is here to compute and set the header tooltip. */
             class OutlineViewOutlineHeaderRenderer implements TableCellRenderer {
                 private TableCellRenderer orig;
@@ -899,16 +1075,7 @@ public class OutlineView extends JScrollPane {
                 public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                     Component oc = orig.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                     if (tooltip == null) {
-                        TableModel model = getModel();
-                        if (model.getRowCount() <= 0) {
-                            // now rows --> we cannot compute the header tooltip
-                            return oc;
-                        }
-                        Object sampleValue = model.getValueAt(0, getModelIndex());
-                        if (sampleValue instanceof Node.Property) {
-                            Node.Property prop = (Node.Property)sampleValue;
-                            tooltip = prop.getShortDescription();
-                        }
+                        tooltip = getShortDescription (value.toString ());
                     }
                     if ((tooltip != null) && (oc instanceof JComponent)) {
                         JComponent jc = (JComponent)oc;
@@ -917,6 +1084,16 @@ public class OutlineView extends JScrollPane {
                     return oc;
                 }
             }
+
+            private class NodeNestedComparator implements
+                    Comparator {
+                public int compare (Object o1, Object o2) {
+                    assert o1 instanceof Node : o1 + " is instanceof Node";
+                    assert o2 instanceof Node : o2 + " is instanceof Node";
+                    return ((Node) o1).getDisplayName ().compareTo (((Node) o2).getDisplayName ());
+                }
+            }
+
         }
     }
     
@@ -967,4 +1144,5 @@ public class OutlineView extends JScrollPane {
             return result;
         }
     }
+
 }
