@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -24,7 +24,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -40,8 +40,8 @@
  */
 package org.openide.explorer.view;
 
-import java.awt.AWTEvent;
 import java.awt.Component;
+import java.awt.Container;
 import org.openide.awt.MouseUtils;
 import org.openide.explorer.ExplorerManager;
 import org.openide.nodes.Children;
@@ -60,7 +60,9 @@ import org.openide.util.lookup.ProxyLookup;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.Font;
+import java.awt.GradientPaint;
 import java.awt.Graphics;
+import java.awt.Graphics2D;
 import java.awt.Insets;
 import java.awt.Point;
 import java.awt.Rectangle;
@@ -108,6 +110,7 @@ import javax.swing.JTextField;
 import javax.swing.JTree;
 import javax.swing.JViewport;
 import javax.swing.KeyStroke;
+import javax.swing.ScrollPaneLayout;
 import javax.swing.SwingUtilities;
 import javax.swing.ToolTipManager;
 import javax.swing.TransferHandler;
@@ -125,6 +128,7 @@ import javax.swing.plaf.UIResource;
 import javax.swing.text.Position;
 import javax.swing.tree.ExpandVetoException;
 import javax.swing.tree.RowMapper;
+import javax.swing.tree.TreeCellEditor;
 import javax.swing.tree.TreeModel;
 import javax.swing.tree.TreeNode;
 import javax.swing.tree.TreePath;
@@ -236,6 +240,8 @@ public abstract class TreeView extends JScrollPane {
     * @param popupAllowed should right-click open popup?
     */
     public TreeView(boolean defaultAction, boolean popupAllowed) {
+        setLayout(new ExplorerScrollPaneLayout());
+
         initializeTree();
 
 //        // activation of drop target
@@ -335,7 +341,7 @@ public abstract class TreeView extends JScrollPane {
      * @return <code>true</code> if so
      */
     public boolean isPopupAllowed() {
-        return popupListener != null;
+        return popupListener != null && isShowing();
     }
 
     /** Enable/disable displaying popup menus on tree view items.
@@ -763,7 +769,7 @@ public abstract class TreeView extends JScrollPane {
         return false;
     }
     
-    private TreePath getTreePath(Node node) {
+    TreePath getTreePath(Node node) {
         return new TreePath(treeModel.getPathToRoot(VisualizerNode.getVisualizer(null, node)));
     }
 
@@ -790,6 +796,18 @@ public abstract class TreeView extends JScrollPane {
     /** Synchronize the root context from the manager of this Explorer.
     */
     final void synchronizeRootContext() {
+        // #151850 cancel editing before changing root node
+        Mutex.EVENT.readAccess(
+                new Runnable() {
+
+                    public void run() {
+                        TreeCellEditor cellEditor = tree.getCellEditor();
+                        if (cellEditor instanceof TreeViewCellEditor) {
+                            ((TreeViewCellEditor) cellEditor).abortTimer();
+                        }
+                        tree.cancelEditing();
+                    }
+                });
         treeModel.setNode(manager.getRootContext(), visHolder);
     }
 
@@ -1363,6 +1381,7 @@ public abstract class TreeView extends JScrollPane {
 
             for (int i = 0; i < paths.length; i++) {
                 Node n = Visualizer.findNode(paths[i].getLastPathComponent());
+                n = getOriginalNode (n);
 
                 if( isUnderRoot( manager.getRootContext(), n ) ) {
                     ll.add(n);
@@ -1370,7 +1389,7 @@ public abstract class TreeView extends JScrollPane {
             }
             callSelectionChanged(ll.toArray(new Node[ll.size()]));
         }
-        
+
         /** Checks whether given Node is a subnode of rootContext.
         * @return true if specified Node is under current rootContext
         */
@@ -1399,6 +1418,10 @@ public abstract class TreeView extends JScrollPane {
     }
      // end of TreePropertyListener
 
+    Node getOriginalNode (Node n) {
+        return n;
+    }
+
     /** Popup adapter.
     */
     class PopupAdapter extends MouseUtils.PopupMouseAdapter {
@@ -1537,50 +1560,157 @@ public abstract class TreeView extends JScrollPane {
         }
     }
 
+    @Override
+    public Insets getInsets() {
+        Insets res = getInnerInsets();
+        res = new Insets(res.top, res.left, res.bottom, res.right);
+        if( null != searchpanel && searchpanel.isVisible() ) {
+            res.bottom += searchpanel.getPreferredSize().height;
+        }
+        return res;
+    }
+
+    private Insets getInnerInsets() {
+        Insets res = super.getInsets();
+        if( null == res ) {
+            res = new Insets(0,0,0,0);
+        }
+        return res;
+    }
+
+    private JPanel searchpanel = null;
+    // searchTextField manages focus because it handles VK_TAB key
+    private JTextField searchTextField = new JTextField() {
+        @Override
+        public boolean isManagingFocus() {
+            return true;
+        }
+
+        @Override
+        public void processKeyEvent(KeyEvent ke) {
+            //override the default handling so that
+            //the parent will never receive the escape key and
+            //close a modal dialog
+            if (ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
+                removeSearchField();
+                ke.consume();
+
+                // bugfix #32909, reqest focus when search field is removed
+                SwingUtilities.invokeLater(
+                    new Runnable() {
+                        //additional bugfix - do focus change later or removing
+                        //the component while it's focused will cause focus to
+                        //get transferred to the next component in the
+                        //parent focusTraversalPolicy *after* our request
+                        //focus completes, so focus goes into a black hole - Tim
+                        public void run() {
+                            if( null != tree )
+                                tree.requestFocus();
+                        }
+                    }
+                );
+            } else {
+                super.processKeyEvent(ke);
+            }
+        }
+    };
+    private int originalScrollMode;
+
+    private static class SearchPanel extends JPanel {
+        public SearchPanel() {
+            if( ViewUtil.isAquaLaF )
+                setBorder(BorderFactory.createEmptyBorder(9,6,8,2));
+            else
+                setBorder(BorderFactory.createEmptyBorder(2,6,2,2));
+            setOpaque( true );
+        }
+
+        @Override
+        protected void paintComponent(Graphics g) {
+            if( ViewUtil.isAquaLaF && g instanceof Graphics2D ) {
+                Graphics2D g2d = (Graphics2D) g;
+                g2d.setPaint( new GradientPaint(0, 0, UIManager.getColor("NbExplorerView.quicksearch.background.top"),
+                        0, getHeight(), UIManager.getColor("NbExplorerView.quicksearch.background.bottom")));//NOI18N
+                g2d.fillRect(0, 0, getWidth(), getHeight());
+                g2d.setColor( UIManager.getColor("NbExplorerView.quicksearch.border") ); //NOI18N
+                g2d.drawLine(0, 0, getWidth(), 0);
+            } else {
+                super.paintComponent(g);
+            }
+        }
+    }
+
+
+    private void prepareSearchPanel() {
+        if( searchpanel == null ) {
+            searchpanel = new SearchPanel();
+
+            JLabel lbl = new JLabel(NbBundle.getMessage(TreeView.class, "LBL_QUICKSEARCH")); //NOI18N
+            searchpanel.setLayout(new BoxLayout(searchpanel, BoxLayout.X_AXIS));
+            searchpanel.add(lbl);
+            searchpanel.add(searchTextField);
+            lbl.setLabelFor(searchTextField);
+            searchTextField.setColumns(10);
+            searchTextField.setMaximumSize(searchTextField.getPreferredSize());
+            searchTextField.putClientProperty("JTextField.variant", "search"); //NOI18N
+            lbl.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
+        }
+    }
+
+    /**
+     * Adds the search field to the tree.
+     */
+    private void displaySearchField() {
+        if( null != searchpanel )
+            return;
+
+        JViewport vp = getViewport();
+        originalScrollMode = vp.getScrollMode();
+        vp.setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
+        searchTextField.setFont(tree.getFont());
+        prepareSearchPanel();
+        add(searchpanel);
+        invalidate();
+        revalidate();
+        repaint();
+        searchTextField.requestFocus();
+    }
+
+    /**
+     * Removes the search field from the tree.
+     */
+    private void removeSearchField() {
+        if( null == searchpanel )
+            return;
+
+        remove(searchpanel);
+        searchpanel = null;
+        getViewport().setScrollMode(originalScrollMode);
+        invalidate();
+        revalidate();
+        repaint();
+    }
+
+    private class ExplorerScrollPaneLayout extends ScrollPaneLayout {
+
+        @Override
+        public void layoutContainer( Container parent ) {
+            super.layoutContainer(parent);
+            if( null != searchpanel && searchpanel.isVisible() ) {
+                Insets innerInsets = getInnerInsets();
+                Dimension prefSize = searchpanel.getPreferredSize();
+                searchpanel.setBounds(innerInsets.left, parent.getHeight()-innerInsets.bottom-prefSize.height,
+                        parent.getWidth()-innerInsets.left-innerInsets.right, prefSize.height);
+            }
+        }
+    }
+
     private final class ExplorerTree extends JTree implements Autoscroll {
         AutoscrollSupport support;
         private String maxPrefix;
         int SEARCH_FIELD_SPACE = 3;
         private boolean firstPaint = true;
 
-        // searchTextField manages focus because it handles VK_TAB key
-        private JTextField searchTextField = new JTextField() {
-            @Override
-                public boolean isManagingFocus() {
-                    return true;
-                }
-
-            @Override
-                public void processKeyEvent(KeyEvent ke) {
-                    //override the default handling so that
-                    //the parent will never receive the escape key and
-                    //close a modal dialog
-                    if (ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
-                        removeSearchField();
-                        ke.consume();
-
-                        // bugfix #32909, reqest focus when search field is removed
-                        SwingUtilities.invokeLater(
-                            new Runnable() {
-                                //additional bugfix - do focus change later or removing
-                                //the component while it's focused will cause focus to
-                                //get transferred to the next component in the 
-                                //parent focusTraversalPolicy *after* our request
-                                //focus completes, so focus goes into a black hole - Tim
-                                public void run() {
-                                    ExplorerTree.this.requestFocus();
-                                }
-                            }
-                        );
-                    } else {
-                        super.processKeyEvent(ke);
-                    }
-                }
-            };
-
-        private JPanel searchpanel = null;
-        final private int heightOfTextField = searchTextField.getPreferredSize().height;
-        private int originalScrollMode;
 
         ExplorerTree(TreeModel model) {
             super(model);
@@ -1613,6 +1743,7 @@ public abstract class TreeView extends JScrollPane {
         public void addNotify() {
             super.addNotify();
             ViewTooltips.register(this);
+            ViewUtil.adjustBackground( this );
         }
         
         @Override
@@ -1673,10 +1804,6 @@ public abstract class TreeView extends JScrollPane {
             new GuardedActions(2, null);
         }
 
-        private void doProcessEvent(AWTEvent e) {
-            super.processEvent(e);
-        }
-
         private void guardedPaint(Graphics g) {
             if (firstPaint) {
                 firstPaint = false;
@@ -1705,18 +1832,6 @@ public abstract class TreeView extends JScrollPane {
 
         private void guardedDoLayout() {
             super.doLayout();
-
-            Rectangle visibleRect = getVisibleRect();
-
-            if ((searchpanel != null) && searchpanel.isDisplayable()) {
-                int width = searchpanel.getPreferredSize().width;
-
-                searchpanel.setBounds(
-                    Math.max(SEARCH_FIELD_SPACE, (visibleRect.x + visibleRect.width) - width),
-                    visibleRect.y + SEARCH_FIELD_SPACE, Math.min(visibleRect.width, width) - SEARCH_FIELD_SPACE,
-                    heightOfTextField
-                );
-            }
         }
 
         @Override
@@ -1761,21 +1876,6 @@ public abstract class TreeView extends JScrollPane {
             }
         }
 
-        private void prepareSearchPanel() {
-            if (searchpanel == null) {
-                searchpanel = new JPanel();
-
-                JLabel lbl = new JLabel(NbBundle.getMessage(TreeView.class, "LBL_QUICKSEARCH")); //NOI18N
-                searchpanel.setLayout(new BoxLayout(searchpanel, BoxLayout.X_AXIS));
-                searchpanel.add(lbl);
-                searchpanel.add(searchTextField);
-                lbl.setLabelFor(searchTextField);
-                searchTextField.setColumns(10);
-                searchpanel.setBorder(BorderFactory.createRaisedBevelBorder());
-                lbl.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
-            }
-        }
-
         private void setupSearch() {
             // Remove the default key listeners
             KeyListener[] keyListeners = getListeners(KeyListener.class);
@@ -1915,36 +2015,6 @@ public abstract class TreeView extends JScrollPane {
             return null;
         }
 
-        /**
-         * Adds the search field to the tree.
-         */
-        private void displaySearchField() {
-            if (!searchTextField.isDisplayable()) {
-                JViewport viewport = TreeView.this.getViewport();
-                originalScrollMode = viewport.getScrollMode();
-                viewport.setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
-                searchTextField.setFont(ExplorerTree.this.getFont());
-                prepareSearchPanel();
-                add(searchpanel);
-                revalidate();
-                repaint();
-                searchTextField.requestFocus();
-            }
-        }
-
-        /**
-         * Removes the search field from the tree.
-         */
-        private void removeSearchField() {
-            if (searchpanel.isDisplayable()) {
-                remove(searchpanel);
-                TreeView.this.getViewport().setScrollMode(originalScrollMode);
-
-                Rectangle r = searchpanel.getBounds();
-                this.repaint(r);
-            }
-        }
-
         /** notify the Component to autoscroll */
         public void autoscroll(Point cursorLoc) {
             getSupport().autoscroll(cursorLoc);
@@ -2148,7 +2218,8 @@ public abstract class TreeView extends JScrollPane {
             }
 
             public void focusGained(FocusEvent e) {
-                // Do nothing
+                // make sure nothing is selected
+                searchTextField.select(1, 1);
             }
 
             public void focusLost(FocusEvent e) {
