@@ -207,13 +207,11 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
         if (desc == UNKNOWN) {
             shortDescription = desc = node.getShortDescription();
         }
-        if (icon instanceof Image) {
-            String toolTip = ImageUtilities.getImageToolTip((Image) icon);
-            if (toolTip.length() > 0) {
-                StringBuilder str = new StringBuilder(128);
-                str.append("<html>").append(desc).append("<br>").append(toolTip).append("</html>");
-                desc = str.toString();
-            }
+        String toolTip = ImageUtilities.getImageToolTip(ImageUtilities.icon2Image(icon != null ? icon : getIcon(false, false)));
+        if (toolTip.length() > 0) {
+            StringBuilder str = new StringBuilder(128);
+            str.append("<html>").append(desc).append("<br>").append(toolTip).append("</html>");
+            desc = str.toString();
         }
         return desc;
     }
@@ -348,7 +346,8 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
     public void childrenAdded(NodeMemberEvent ev) {
         VisualizerChildren ch = children.get();
 
-        LOG.log(Level.FINER, "childrenAdded {0}", ev); // NOI18N
+        LOG.log(Level.FINER, "childrenAdded event: {0}\n  ch: {1}", new Object[]{ev, ch}); // NOI18N
+
         if (ch == null) {
             LOG.log(Level.FINER, "childrenAdded - exit"); // NOI18N
             return;
@@ -364,7 +363,7 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
     public void childrenRemoved(NodeMemberEvent ev) {
         VisualizerChildren ch = children.get();
 
-        LOG.log(Level.FINER, "childrenRemoved {0}", ev); // NOI18N
+        LOG.log(Level.FINER, "childrenRemoved event: {0}\n  ch: {1}", new Object[]{ev, ch}); // NOI18N
         if (ch == null) {
             LOG.log(Level.FINER, "childrenRemoved - exit"); // NOI18N
             return;
@@ -386,7 +385,7 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
         VisualizerChildren ch = children.get();
 
         int[] perm = ev.getPermutation();
-        LOG.log(Level.FINER, "childrenReordered {0}", perm); // NOI18N
+        LOG.log(Level.FINER, "childrenReordered {0}\n  ch: {1}", new Object[]{perm, ch}); // NOI18N
         if (ch == null) {
             LOG.log(Level.FINER, "childrenReordered - exit"); // NOI18N
             return;
@@ -620,7 +619,7 @@ final class VisualizerNode extends EventListenerList implements NodeListener, Tr
     /** Loads default icon if not loaded. */
     private static Icon getDefaultIcon() {
         if (defaultIcon == null) {
-            defaultIcon = ImageUtilities.image2Icon(ImageUtilities.loadImage(DEFAULT_ICON));
+            defaultIcon = ImageUtilities.loadImageIcon(DEFAULT_ICON, false);
         }
         return defaultIcon;
     }
