@@ -309,10 +309,14 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
     /** Access method for node delagate.
      * @return node delegate or null
      */
-    Node getNodeDelegateOrNull () {
+    final Node getNodeDelegateOrNull () {
         return nodeDelegate;
     }
 
+    final void setNodeDelegate(Node n) {
+        nodeDelegate = n;
+    }
+
     /** Provides node that should represent this data object.
     * <p>The default implementation creates an instance of {@link DataNode}.
     * Most subclasses will override this method to provide a <code>DataNode</code>
@@ -1171,6 +1175,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
 
     private static final class ModifiedRegistry extends HashSet<DataObject> {
         static final long serialVersionUID =-2861723614638919680L;
+        private static final Logger REGLOG = Logger.getLogger("org.openide.loaders.DataObject.Registry"); // NOI18N
         
         private final ChangeSupport cs = new ChangeSupport(this);
 
@@ -1195,6 +1200,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
         @Override
         public boolean add (DataObject o) {
             boolean result = super.add(o);
+            REGLOG.log(Level.FINER, "Data Object {0} modified, change {1}", new Object[] { o, result }); // NOI18N
             if (result) {
                 cs.fireChange();
             }
@@ -1204,6 +1210,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
         @Override
         public boolean remove (Object o) {
             boolean result = super.remove(o);
+            REGLOG.log(Level.FINER, "Data Object {0} unmodified, change {1}", new Object[] { o, result }); // NOI18N
             if (result) {
                 cs.fireChange();
             }
