@@ -48,9 +48,7 @@ import org.netbeans.junit.NbTestCase;
 import org.openide.cookies.OpenCookie;
 import org.openide.filesystems.FileLock;
 import org.openide.filesystems.FileObject;
-import org.openide.filesystems.FileSystem;
 import org.openide.filesystems.FileUtil;
-import org.openide.filesystems.Repository;
 import org.openide.loaders.DataLoader;
 import org.openide.loaders.DataObject;
 import org.openide.loaders.DataObjectExistsException;
@@ -64,7 +62,6 @@ import org.openide.util.test.MockLookup;
 
 public class DataEditorSupportSaveAsTest extends NbTestCase {
     
-    FileSystem fs;
     static {
         System.setProperty("org.openide.windows.DummyWindowManager.VISIBLE", "false");
     }
@@ -77,25 +74,23 @@ public class DataEditorSupportSaveAsTest extends NbTestCase {
     protected void setUp() throws Exception {
         clearWorkDir();
         MockLookup.setInstances(new Pool());
-        fs = org.openide.filesystems.FileUtil.createMemoryFileSystem ();
     }
     
     public void testUnmodifiedDocumentSaveAs() throws IOException {
-        FileSystem fs = Repository.getDefault().getDefaultFileSystem();
-        FileUtil.createData(fs.getRoot(), "someFolder/someFile.obj");
+        FileUtil.createData(FileUtil.getConfigRoot(), "someFolder/someFile.obj");
         
-        DataObject obj = DataObject.find(fs.findResource("someFolder/someFile.obj"));
+        DataObject obj = DataObject.find(FileUtil.getConfigFile("someFolder/someFile.obj"));
         assertEquals( MyDataObject.class, obj.getClass());
         assertTrue( "we need UniFileLoader", obj.getLoader() instanceof UniFileLoader );
         
         MyEnv env = new MyEnv( obj );
         MyDataEditorSupport des = new MyDataEditorSupport( obj, env );
         
-        FileObject newFolder = FileUtil.createFolder(fs.getRoot(), "otherFolder");
+        FileObject newFolder = FileUtil.createFolder(FileUtil.getConfigRoot(), "otherFolder");
         
         des.saveAs( newFolder, "newFile.newExt" );
         
-        DataObject newObj = DataObject.find(fs.findResource("otherFolder/newFile.newExt"));
+        DataObject newObj = DataObject.find(FileUtil.getConfigFile("otherFolder/newFile.newExt"));
         assertEquals( MyDataObject.class, newObj.getClass());
         MyDataObject myObj = (MyDataObject)newObj;
         
@@ -105,10 +100,9 @@ public class DataEditorSupportSaveAsTest extends NbTestCase {
     }
     
     public void testModifiedDocumentSaveAs() throws IOException {
-        FileSystem fs = Repository.getDefault().getDefaultFileSystem();
-        FileUtil.createData(fs.getRoot(), "someFolder/someFile.obj");
+        FileUtil.createData(FileUtil.getConfigRoot(), "someFolder/someFile.obj");
         
-        DataObject obj = DataObject.find(fs.findResource("someFolder/someFile.obj"));
+        DataObject obj = DataObject.find(FileUtil.getConfigFile("someFolder/someFile.obj"));
         assertEquals( MyDataObject.class, obj.getClass());
         assertTrue( "we need UniFileLoader", obj.getLoader() instanceof UniFileLoader );
         
@@ -117,11 +111,11 @@ public class DataEditorSupportSaveAsTest extends NbTestCase {
         MyEnv env = new MyEnv( obj );
         MyDataEditorSupport des = new MyDataEditorSupport( obj, env );
         
-        FileObject newFolder = FileUtil.createFolder(fs.getRoot(), "otherFolder");
+        FileObject newFolder = FileUtil.createFolder(FileUtil.getConfigRoot(), "otherFolder");
         
         des.saveAs( newFolder, "newFile.newExt" );
         
-        DataObject newObj = DataObject.find(fs.findResource("otherFolder/newFile.newExt"));
+        DataObject newObj = DataObject.find(FileUtil.getConfigFile("otherFolder/newFile.newExt"));
         assertEquals( MyDataObject.class, newObj.getClass());
         MyDataObject myObj = (MyDataObject)newObj;
         
@@ -133,10 +127,9 @@ public class DataEditorSupportSaveAsTest extends NbTestCase {
     }
     
 //    public void testEnvAddsSaveAsImpl() throws IOException {
-//        FileSystem fs = Repository.getDefault().getDefaultFileSystem();
-//        FileUtil.createData(fs.getRoot(), "someFolder/someFile.obj");
+//        FileUtil.createData(FileUtil.getConfigRoot(), "someFolder/someFile.obj");
 //        
-//        DataObject obj = DataObject.find(fs.findResource("someFolder/someFile.obj"));
+//        DataObject obj = DataObject.find(FileUtil.getConfigFile("someFolder/someFile.obj"));
 //        assertEquals( MyDataObject.class, obj.getClass());
 //        assertTrue( "we need UniFileLoader", obj.getLoader() instanceof UniFileLoader );
 //        
@@ -147,11 +140,10 @@ public class DataEditorSupportSaveAsTest extends NbTestCase {
 //    }
 //    
 //    public void testNoSaveAsImpl() throws IOException {
-//        FileSystem fs = Repository.getDefault().getDefaultFileSystem();
-//        FileUtil.createData(fs.getRoot(), "someFolder/x.prima");
-//        FileUtil.createData(fs.getRoot(), "someFolder/x.seconda");
+//        FileUtil.createData(FileUtil.getConfigRoot(), "someFolder/x.prima");
+//        FileUtil.createData(FileUtil.getConfigRoot(), "someFolder/x.seconda");
 //        
-//        DataObject obj = DataObject.find(fs.findResource("someFolder/x.prima"));
+//        DataObject obj = DataObject.find(FileUtil.getConfigFile("someFolder/x.prima"));
 //        assertEquals( MyMultiFileDataObject.class, obj.getClass());
 //        assertEquals( "we need an object with MultiFileLoader", MyMultiFileLoader.class, obj.getLoader().getClass());
 //        
