@@ -41,13 +41,13 @@
 
 package org.openide.text;
 
-
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.*;
 import java.util.concurrent.Callable;
 import java.util.logging.Level;
+import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 import javax.swing.*;
 import javax.swing.border.Border;
@@ -65,6 +65,11 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     private static final String HELP_ID = "editing.editorwindow"; // !!! NOI18N
     static final long serialVersionUID = -185739563792410059L;
 
+    private static final RequestProcessor RP = new RequestProcessor("org.openide.text Editor Initialization");
+    
+    /** Thread for postprocessing work in CloneableEditor.DoInitialize.initRest */
+    private static final RequestProcessor RPPostprocessing = new RequestProcessor("org.openide.text Document Postprocessing");
+
     /** editor pane  */
     protected JEditorPane pane;
 
@@ -78,6 +83,10 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * and initialization time. */
     private int cursorPosition = -1;
 
+    private StringBuffer sb = new StringBuffer(1024);
+
+    private final Object CLOSE_LAST_LOCK = new Object();
+
     // #20647. More important custom component.
 
     /** Custom editor component, which is used if specified by document
@@ -86,7 +95,9 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     private Component customComponent;
     private JToolBar customToolbar;
     private DoInitialize doInitialize;
-
+    
+    private static final Logger LOG = Logger.getLogger("org.openide.text.CloneableEditor"); // NOI18N
+    
     /** For externalization of subclasses only  */
     public CloneableEditor() {
         this(null);
@@ -102,6 +113,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
         updateName();
         _setCloseOperation();
+        setMinimumSize(new Dimension(10, 10));
     }
     @SuppressWarnings("deprecation")
     private void _setCloseOperation() {
@@ -119,6 +131,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
     /** Overriden to explicitely set persistence type of CloneableEditor
      * to PERSISTENCE_ONLY_OPENED */
+    @Override
     public int getPersistenceType() {
         return TopComponent.PERSISTENCE_ONLY_OPENED;
     }
@@ -129,6 +142,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * then that it used, else general help on the editor is provided.
      * @return context help
      */
+    @Override
     public HelpCtx getHelpCtx() {
         Object kit = support.cesKit();
         HelpCtx fromKit = kit == null ? null : HelpCtx.findHelp(kit);
@@ -145,6 +159,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * Adds scheduling of "emptying" editor pane and removing all sub components.
      * {@inheritDoc}
      */
+    @Override
     public boolean canClose() {
         boolean result = super.canClose();
         return result;
@@ -152,6 +167,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
     /** Overrides superclass method. In case it is called first time,
      * initializes this <code>CloneableEditor</code>. */
+    @Override
     protected void componentShowing() {
         super.componentShowing();
         initialize();
@@ -197,13 +213,20 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         private int phase;
         private EditorKit kit;
         private JComponent tmpComp;
+        /** Flag to avoid never ending wait in initDocument. It is to handle case
+         *  when someone closes document ie. sets doc to null between initNonVisual
+         *  and initDocument issue #136601.
+         */
+        private boolean initialized = false;
+        /** Flag to avoid recursive call of initVisual. */
+        private boolean isInInitVisual = false;
 
         public DoInitialize(QuietEditorPane tmp) {
             this.tmp = tmp;
             this.tmpComp = initLoading();
             new Timer(1000, this).start();
             if (newInitialize()) {
-                task = CloneableEditorSupport.RP.create(this);
+                task = RP.create(this);
                 task.setPriority(Thread.MIN_PRIORITY + 2);
                 task.schedule(0);
             } else {
@@ -236,13 +259,26 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             
             int phaseNow = phase;
             switch (phase++) {
-            case 0: 
-                initNonVisual();
+            case 0:
+                synchronized (CLOSE_LAST_LOCK) {
+                    initNonVisual();
+                }
                 if (newInitialize()) {
                     WindowManager.getDefault().invokeWhenUIReady(this);
                     break;
                 }
             case 1:
+                if (CloneableEditor.this.pane != this.tmp) {
+                    //#138686: Cancel initialization when TC was closed in the meantime
+                    //and pane is null or even different instance
+                    phase = Integer.MAX_VALUE;
+                    break;
+                }
+                if ((support.getDocument() == null) || (support.cesKit() == null)) {
+                    //#138686, #161902, #149771: Cancel initialization when document was closed in the meantime
+                    phase = Integer.MAX_VALUE;
+                    break;
+                }
                 initVisual();
                 if (newInitialize()) {
                     task.schedule(1000);
@@ -258,7 +294,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             long howLong = System.currentTimeMillis() - now;
             if (TIMER.isLoggable(Level.FINE)) {
                 String thread = SwingUtilities.isEventDispatchThread() ? "AWT" : "RP"; // NOI18N
-                Object who = doc.getProperty(Document.StreamDescriptionProperty);
+                Document d = doc;
+                Object who = d == null ? null : d.getProperty(Document.StreamDescriptionProperty);
                 if (who == null) {
                     who = support.messageName();
                 }
@@ -270,11 +307,32 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         }
             
         private void initNonVisual() {
+            if (LOG.isLoggable(Level.FINE)) {
+                LOG.log(Level.FINE,"DoInitialize.initNonVisual Enter"
+                + " Time:" + System.currentTimeMillis()
+                + " Thread:" + Thread.currentThread().getName()
+                + " ce:[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                + " Name:" + CloneableEditor.this.getName());
+            }
             Task prepareTask = support.prepareDocument();
-
-            // load the doc synchronously
+            assert prepareTask != null : "Failed to get prepareTask";
             prepareTask.waitFinished();
-
+            Throwable ex = support.getPrepareDocumentRuntimeException();
+            if (ex instanceof CloneableEditorSupport.DelegateIOExc) {
+                if ("org.openide.text.DataEditorSupport$Env$ME".equals(ex.getCause().getClass().getName())) {
+                    if (ex.getCause() instanceof UserQuestionException) {
+                        UserQuestionException e = (UserQuestionException) ex.getCause();
+                        try {
+                            e.confirmed();
+                        } catch (IOException ioe) {
+                        }
+                        prepareTask = support.prepareDocument();
+                        assert prepareTask != null : "Failed to get prepareTask";
+                        prepareTask.waitFinished();
+                    }
+                }
+            }
             // Init action map: cut,copy,delete,paste actions.
             javax.swing.ActionMap am = getActionMap();
 
@@ -289,8 +347,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             paneMap.put(DefaultEditorKit.copyAction, getAction(DefaultEditorKit.copyAction));
             paneMap.put("delete", getAction(DefaultEditorKit.deleteNextCharAction)); // NOI18N
             paneMap.put(DefaultEditorKit.pasteAction, getAction(DefaultEditorKit.pasteAction));
-            
-            
+
+
             EditorKit k = support.cesKit();
             if (newInitialize() && k instanceof Callable) {
                 try {
@@ -301,17 +359,70 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             }
             
             synchronized (this) {
-                doc = support.getDocument();
+                try {
+                    doc = support.openDocument();
+                } catch (IOException exc) {
+                    LOG.log(Level.INFO,"Failed to open document",exc);
+                }
                 kit = k;
+                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                + " -- CloneableEditor.DoInitialize.initNonVisual"
+                + " name:" + getName()
+                + " SET kit:" + kit
+                + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                + " Thread:[" + Thread.currentThread().getName() + "]");
+                initialized = true;
+                if (LOG.isLoggable(Level.FINE)) {
+                    LOG.log(Level.FINE,"DoInitialize.initNonVisual doc and kit are set"
+                    + " Time:" + System.currentTimeMillis()
+                    + " Thread:" + Thread.currentThread().getName()
+                    + " ce:[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                    + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
+                    + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                    + " Name:" + CloneableEditor.this.getName()
+                    + " doc:" + doc
+                    + " kit:" + kit);
+                    LOG.log(Level.FINE,"DoInitialize.initNonVisual Call notifyAll"
+                    + " Time:" + System.currentTimeMillis()
+                    + " Thread:" + Thread.currentThread().getName()
+                    + " [" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                    + " Name:" + CloneableEditor.this.getName());
+                }
                 notifyAll();
             }
         }
         
         private void initCustomEditor() {
+            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+            + " -- CloneableEditor.DoInitialize.initCustomEditor"
+            + " name:" + getName()
+            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+            + " Thread:[" + Thread.currentThread().getName() + "]" + " ENTER");
             if (doc instanceof NbDocument.CustomEditor) {
                 NbDocument.CustomEditor ce = (NbDocument.CustomEditor) doc;
-                customComponent = ce.createEditor(tmp);
-
+                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                + " -- CloneableEditor.DoInitialize.initCustomEditor"
+                + " tmp:" + tmp.getClass().getName() + "@" + Integer.toHexString(System.identityHashCode(tmp)));
+                sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                + " -- CloneableEditor.DoInitialize.initCustomEditor"
+                + " tmp.ui:" + tmp.getUI().getClass().getName());
+                try {
+                    customComponent = ce.createEditor(tmp);
+                } catch (NullPointerException ex) {
+                    Logger logger = Logger.getLogger("org.netbeans.ui.text");   // NOI18N
+                    LogRecord rec = new LogRecord(Level.INFO, "LOG_CLONEABLE_EDITOR");   // NOI18N
+                    rec.setLoggerName(logger.getName());
+                    String log;
+                    if (tmp instanceof QuietEditorPane) {
+                        log = ((QuietEditorPane) tmp).getLog();
+                    } else {
+                        log = "Pane is not QuietEditorPane";
+                    }
+                    rec.setParameters(new Object [] {"1.0", sb.toString(), log, support.getLog()});
+                    logger.log(rec);
+                    throw new IllegalStateException("Unexpected UI class:" + tmp.getUI().getClass().getName(), ex);
+                }
+                
                 if (customComponent == null) {
                     throw new IllegalStateException(
                         "Document:" + doc // NOI18N
@@ -344,11 +455,32 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                 for (;;) {
                     d = doc;
                     k = kit;
-                    if (d != null && k != null) {
+                    if (initialized) {
                         break;
                     }
                     try {
+                        if (LOG.isLoggable(Level.FINE)) {
+                            LOG.log(Level.FINE,"DoInitialize.initDocument Starting wait"
+                            + " Time:" + System.currentTimeMillis()
+                            + " Thread:" + Thread.currentThread().getName()
+                            + " ce:[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                            + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
+                            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                            + " Name:" + CloneableEditor.this.getName()
+                            + " doc:" + doc
+                            + " kit:" + kit);
+                        }
                         wait();
+                        if (LOG.isLoggable(Level.FINE)) {
+                            LOG.log(Level.FINE,"DoInitialize.initDocument Wait finished"
+                            + " Thread:" + Thread.currentThread().getName()
+                            + " ce:[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                            + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
+                            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                            + " Name:" + CloneableEditor.this.getName()
+                            + " doc:" + doc
+                            + " kit:" + kit);
+                        }
                     } catch (InterruptedException ex) {
                         Exceptions.printStackTrace(ex);
                     }
@@ -357,17 +489,56 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             if (tmp.getDocument() == doc) {
                 return false;
             }
+            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+            + " -- CloneableEditor.DoInitialize.initDocument INVOKE tmp.setEditorKit(kit)"
+            + " kit:" + kit
+            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+            + " Thread:[" + Thread.currentThread().getName() + "]");
             tmp.setEditorKit(kit);
+            // #132669, do not fire prior setting the kit, which by itself sets a bogus document, etc.
+            // if this is a problem please revert the change and initialize QuietEditorPane.working = FIRE
+            // and reopen #132669
+            tmp.setWorking(QuietEditorPane.FIRE);
             tmp.setDocument(doc);
+            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+            + " -- CloneableEditor.DoInitialize.initDocument"
+            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+            + " RETURN TRUE"
+            + " Thread:[" + Thread.currentThread().getName() + "]");
             return true;
         }
         
         final void initVisual() {
-            // wait for document and init it
-            if (!initDocument()) {
+            //Do not allow recursive call
+            if (isInInitVisual) {
                 return;
             }
-            
+            isInInitVisual = true;
+            // wait for document and init it
+            if (!initDocument()) {
+                isInInitVisual = false;
+                return;
+            }
+            sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+            + " -- CloneableEditor.DoInitialize.initVisual"
+            + " name:" + getName()
+            + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+            + " ENTER"
+            + " Thread:[" + Thread.currentThread().getName() + "]");
+            if (LOG.isLoggable(Level.FINE)) {
+                /*Exception ex = new Exception();
+                StringWriter sw = new StringWriter(500);
+                PrintWriter pw = new PrintWriter(sw);
+                ex.printStackTrace(pw);*/
+                LOG.log(Level.FINE,"DoInitialize.initVisual Enter"
+                + " Time:" + System.currentTimeMillis()
+                + " Thread:" + Thread.currentThread().getName()
+                + " ce:[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                + " this:[" + Integer.toHexString(System.identityHashCode(this)) + "]"
+                + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                + " Name:" + CloneableEditor.this.getName());
+                //+ " " + sw.toString());
+            }
             initCustomEditor();
             if (customComponent != null) {
                 add(support.wrapEditorComponent(customComponent), BorderLayout.CENTER);
@@ -397,14 +568,37 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
                     caret.setDot(cursorPosition);
                 }
             }
-
-            requestFocusInWindow();
+            ActionMap p = getActionMap().getParent();
+            getActionMap().setParent(null);
+            getActionMap().setParent(p);
+            //#134910: If editor TopComponent is already activated request focus
+            //to it again to get focus to correct subcomponent eg. QuietEditorPane which
+            //is added above.
+            if (CloneableEditor.this.equals(getRegistry().getActivated())) {
+                requestFocusInWindow();
+            }
+            //#162961: Force repaint of editor on Mac. Undocked editor stays empty otherwise sometimes.
+            if (org.openide.util.Utilities.isMac()) {
+                SwingUtilities.invokeLater(new Runnable () {
+                    public void run () {
+                        revalidate();
+                    }
+                });
+            }
+            isInInitVisual = false;
         }
         
         private void initRest() {
-            support.ensureAnnotationsLoaded();
+            //#132662 Post this task to another worker private thread
+            //to avoid deadlock.
+            RPPostprocessing.post(new Runnable() {
+                public void run() {
+                    support.ensureAnnotationsLoaded();
+                }
+            });
         }
     } // end of DoInitialize
+    @Override
     protected CloneableTopComponent createClonedObject() {
         return support.createCloneableTopComponent();
     }
@@ -414,6 +608,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * {@link org.openide.cookies.EditorCookie.Observable#PROP_OPENED_PANES}
      * property change on their own.
      */
+    @Override
     protected void componentOpened() {
         super.componentOpened();
 
@@ -429,17 +624,30 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * {@link org.openide.cookies.EditorCookie.Observable#PROP_OPENED_PANES}
      * property change on their own.
      */
+    @Override
     protected void componentClosed() {
         SwingUtilities.invokeLater(
             new Runnable() {
                 public void run() {
+                    sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                    + " -- CloneableEditor.componentClosed.run ENTER"
+                    + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                    + " Thread:[" + Thread.currentThread().getName() + "]");
                     // #23486: pane could not be initialized yet.
-                    // #114608 - commenting out setting of the empty document and null kit
-//                    if (pane != null) {
+                    if (pane != null) {
+                        // #114608 - commenting out setting of the empty document
 //                        Document doc = support.createStyledDocument(pane.getEditorKit());
 //                        pane.setDocument(doc);
-//                        pane.setEditorKit(null);
-//                    }
+                        
+                        // #138611 - this calls kit.deinstall, which is what our kits expect,
+                        // calling it with null does not impact performance, because the pane
+                        // will not create new document and typically nobody listens on "editorKit" prop change
+                        sb.append("\n[" + Integer.toHexString(System.identityHashCode(CloneableEditor.this)) + "]"
+                        + " -- CloneableEditor.componentClosed.run INVOKE pane.setEditorKit(null)"
+                        + " support:[" + Integer.toHexString(System.identityHashCode(support)) + "]"
+                        + " Thread:[" + Thread.currentThread().getName() + "]");
+                        pane.setEditorKit(null);
+                    }
 
                     removeAll();
                     customComponent = null;
@@ -459,30 +667,10 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         }
     }
 
-    /** Overrides superclass version. Opens top component only if
-     * it is in valid state.
-     * (Editor top component may become invalid after deserialization).<br>
-     * Also tries to open all other top components which are docked
-     * in editor mode on given workspace, but not visible.<br>
-     */
-    @SuppressWarnings("deprecation")
-    public void open(Workspace workspace) {
-        if (discard()) {
-            Logger.getAnonymousLogger().warning(
-                "Can not open " + this + " component," // NOI18N
-                 +" its support environment is not valid" // NOI18N
-                 +" [support=" + support + ", env=" // NOI18N
-                 +((support == null) ? null : support.cesEnv()) + "]"
-            ); // NOI18N
-        } else {
-            dockIfNeeded();
-            super.open(workspace);
-        }
-    }
-
     /** When closing last view, also close the document.
      * @return <code>true</code> if close succeeded
      */
+    @Override
     protected boolean closeLast() {
         if (!support.canClose()) {
             // if we cannot close the last window
@@ -490,7 +678,9 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         }
 
         // close everything and do not ask
-        support.notifyClosed();
+        synchronized (CLOSE_LAST_LOCK) {
+            support.notifyClosed();
+        }
 
         if (support.getLastSelected() == this) {
             support.setLastSelected(null);
@@ -502,6 +692,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     /** The undo/redo manager of the support.
      * @return the undo/redo manager shared by all editors for this support
      */
+    @Override
     public UndoRedo getUndoRedo() {
         return support.getUndoRedo();
     }
@@ -520,9 +711,14 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
             Class<? extends SystemAction> c = Class.forName("org.openide.actions.FileSystemAction", true, l).asSubclass(SystemAction.class); // NOI18N
             SystemAction ra = SystemAction.findObject(c, true);
 
-            Action[] a2 = new Action[a.length + 1];
+            Class<? extends SystemAction> c2 = Class.forName("org.netbeans.modules.diff.DiffAction", true, l).asSubclass(SystemAction.class); // NOI18N
+            SystemAction ra2 = SystemAction.findObject(c2, true);           
+            
+            Action[] a2 = new Action[a.length + 2];
             System.arraycopy(a, 0, a2, 0, a.length);
             a2[a.length] = ra;
+            a2[a.length + 1] = ra2;
+
             return a2;
         } catch (Exception ex) {
             // ok, we no action like this I guess
@@ -533,7 +729,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
     /** Transfer the focus to the editor pane.
      */
-    @SuppressWarnings("deprecation")
+    @Deprecated
+    @Override
     public void requestFocus() {
         super.requestFocus();
 
@@ -548,7 +745,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
     /** Transfer the focus to the editor pane.
      */
-    @SuppressWarnings("deprecation")
+    @Deprecated
+    @Override
     public boolean requestFocusInWindow() {
         super.requestFocusInWindow();
 
@@ -563,7 +761,8 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         return false;
     }
 
-    @SuppressWarnings("deprecation")
+    @Deprecated
+    @Override
     public boolean requestDefaultFocus() {
         if ((customComponent != null) && !SwingUtilities.isDescendingFrom(pane, customComponent)) {
             return customComponent.requestFocusInWindow();
@@ -576,6 +775,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
 
     // XXX is this method really needed?
     /** @return Preferred size of editor top component  */
+    @Override
     public Dimension getPreferredSize() {
         @SuppressWarnings("deprecation")
         Rectangle bounds = WindowManager.getDefault().getCurrentWorkspace().getBounds();
@@ -610,6 +810,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     /** Overrides superclass method. Remembers last selected component of
      * support belonging to this component.
      * @see #componentDeactivated */
+    @Override
     protected void componentActivated() {
         support.setLastSelected(this);
     }
@@ -642,6 +843,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     }
 
     // override for simple and consistent IDs
+    @Override
     protected String preferredID() {
         final CloneableEditorSupport ces = cloneableEditorSupport();
 
@@ -652,6 +854,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         return "";
     }
 
+    @Override
     public void writeExternal(ObjectOutput out) throws IOException {
         super.writeExternal(out);
 
@@ -706,6 +909,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
         out.writeObject(new Integer(pos));
     }
 
+    @Override
     public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
         super.readExternal(in);
 
@@ -738,6 +942,7 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
      * @throws ObjectStreamException When problem during serialization occures.
      * @throws NotSerializableException When this <code>CloneableEditor</code>
      *               is invalid and doesn't want to be serialized. */
+    @Override
     protected Object writeReplace() throws ObjectStreamException {
         if (discard()) {
             throw new NotSerializableException("Serializing component is invalid: " + this); // NOI18N
@@ -769,33 +974,6 @@ public class CloneableEditor extends CloneableTopComponent implements CloneableE
     private boolean discard() {
         return (support == null) || !support.cesEnv().isValid();
     }
-
-    /** Dock this top component to editor mode if it is not docked
-     * in some mode at this time  */
-    private void dockIfNeeded() {
-        // dock into editor mode if possible
-        Mode ourMode = WindowManager.getDefault().findMode(this);
-        if( null == ourMode ) {
-            //dock into 'editor' mode to avoid being tagged as a pre-version-4.0 
-            //TopComponent that is allowed to be drag and dropped outside the editor area
-            
-            //first check the active mode as it might be a floating editor window
-            TopComponent activeTc = TopComponent.getRegistry().getActivated();
-            if( null != activeTc ) {
-                ourMode = WindowManager.getDefault().findMode( activeTc );
-                if( !WindowManager.getDefault().isEditorMode( ourMode ) )
-                    ourMode = null;
-            }
-            if( null == ourMode )
-                ourMode = WindowManager.getDefault().findMode( "editor" );
-            if( null != ourMode ) {
-                 ourMode.dockInto( this );
-            } else {
-                //should not happen - editor mode is always defined
-                Logger.getAnonymousLogger().warning("The window system cannot find the default editor mode." );
-            }
-        }
-    }
     
     //
     // Implements the CloneableEditorSupport.Pane interface
