@@ -45,6 +45,9 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.lang.ref.Reference;
+import java.lang.ref.WeakReference;
+import java.lang.reflect.Method;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -53,32 +56,27 @@ import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 import java.util.WeakHashMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import org.openide.util.Lookup;
-import org.openide.util.SharedClassObject;
-import org.openide.util.WeakSet;
 
-/** A lookup that implements the JDK1.3 JAR services mechanism and delegates
- * to META-INF/services/name.of.class files.
- * <p>It is not dynamic - so if you need to change the classloader or JARs,
- * wrap it in a ProxyLookup and change the delegate when necessary.
- * Existing instances will be kept if the implementation classes are unchanged,
- * so there is "stability" in doing this provided some parent loaders are the same
- * as the previous ones.
- * <p>If this is to be made public, please move it to the org.openide.util.lookup
- * package; currently used by the core via reflection, until it is needed some
- * other way.
+/**
  * @author Jaroslav Tulach, Jesse Glick
+ * @see Lookups#metaInfServices(ClassLoader,String)
  * @see "#14722"
  */
 final class MetaInfServicesLookup extends AbstractLookup {
 
     private static final Logger LOGGER = Logger.getLogger(MetaInfServicesLookup.class.getName());
-
-    private static final Map<Class,Object> knownInstances = new WeakHashMap<Class,Object>();
+    private static int knownInstancesCount;
+    private static final List<Reference<Object>> knownInstances;
+    static {
+        knownInstances = new ArrayList<Reference<Object>>();
+        for (int i = 0; i < 512; i++) {
+            knownInstances.add(null);
+        }
+    }
 
     /** A set of all requested classes.
      * Note that classes that we actually succeeded on can never be removed
@@ -86,7 +84,7 @@ final class MetaInfServicesLookup extends AbstractLookup {
      * However we also hold classes which are definitely not loadable by
      * our loader.
      */
-    private final Set<Class> classes = new WeakSet<Class>(); // Set<Class>
+    private final Map<Class,Object> classes = new WeakHashMap<Class,Object>();
 
     /** class loader to use */
     private final ClassLoader loader;
@@ -102,19 +100,21 @@ final class MetaInfServicesLookup extends AbstractLookup {
         LOGGER.log(Level.FINE, "Created: {0}", this);
     }
 
+    @Override
     public String toString() {
         return "MetaInfServicesLookup[" + loader + "]"; // NOI18N
     }
 
     /* Tries to load appropriate resources from manifest files.
      */
+    @Override
     protected final void beforeLookup(Lookup.Template t) {
         Class c = t.getType();
 
         HashSet<AbstractLookup.R> listeners;
 
         synchronized (this) {
-            if (classes.add(c)) {
+            if (classes.put(c, "") == null) { // NOI18N
                 // Added new class, search for it.
                 LinkedHashSet<AbstractLookup.Pair<?>> arr = getPairsAsLHS();
                 search(c, arr);
@@ -128,7 +128,10 @@ final class MetaInfServicesLookup extends AbstractLookup {
             }
         }
 
-        notifyListeners(listeners);
+        NotifyListeners notify = new NotifyListeners(listeners);
+        if (notify.shallRun()) {
+            notify.run();
+        }
     }
 
     /** Finds all pairs and adds them to the collection.
@@ -188,14 +191,14 @@ final class MetaInfServicesLookup extends AbstractLookup {
                 if (realMcCoy != clazz) {
                     // Either the interface class is not available at all in our loader,
                     // or it is not the same version as we expected. Don't provide results.
-                    if (LOGGER.isLoggable(Level.FINER)) {
+                    if (LOGGER.isLoggable(Level.WARNING)) {
                         if (realMcCoy != null) {
-                            LOGGER.log(Level.FINER,
+                            LOGGER.log(Level.WARNING,
                                 clazz.getName() + " is not the real McCoy! Actually found it in " +
                                 realMcCoy.getClassLoader()
                             ); // NOI18N
                         } else {
-                            LOGGER.log(Level.FINER, clazz.getName() + " could not be found in " + loader); // NOI18N
+                            LOGGER.log(Level.WARNING, clazz.getName() + " could not be found in " + loader); // NOI18N
                         }
                     }
 
@@ -389,6 +392,7 @@ final class MetaInfServicesLookup extends AbstractLookup {
             }
         }
 
+        @Override
         public boolean equals(Object o) {
             if (o instanceof P) {
                 return ((P) o).clazz().equals(clazz());
@@ -397,6 +401,7 @@ final class MetaInfServicesLookup extends AbstractLookup {
             return false;
         }
 
+        @Override
         public int hashCode() {
             return clazz().hashCode();
         }
@@ -419,20 +424,57 @@ final class MetaInfServicesLookup extends AbstractLookup {
 
                     try {
                         Class<?> c = ((Class) o);
+                        o = null;
 
                         synchronized (knownInstances) { // guards only the static cache
-                            o = knownInstances.get(c);
+                            int size = knownInstances.size();
+                            int index = c.hashCode() % size;
+                            for (int i = 0; i < size; i++) {
+                                Reference<Object> ref = knownInstances.get(index);
+                                Object obj = ref == null ? null : ref.get();
+                                if (obj == null) {
+                                    break;
+                                }
+                                if (c == obj.getClass()) {
+                                    o = obj;
+                                    break;
+                                }
+                                if (++index == size) {
+                                    index = 0;
+                                }
+                            }
                         }
 
                         if (o == null) {
-                            if (SharedClassObject.class.isAssignableFrom(c)) {
-                                o = SharedClassObject.findObject(c.asSubclass(SharedClassObject.class), true);
-                            } else {
-                                o = c.newInstance();
-                            }
+                            o = createInstance(c);
 
                             synchronized (knownInstances) { // guards only the static cache
-                                knownInstances.put(c, o);
+                                hashPut(o);
+
+                                int size = knownInstances.size();
+                                if (knownInstancesCount > size * 2 / 3) {
+                                    LOGGER.log(Level.CONFIG, "Cache of size {0} is 2/3 full. Rehashing.", size);
+                                    HashSet<Reference<Object>> all = new HashSet<Reference<Object>>();
+                                    all.addAll(knownInstances);
+                                    for (int i = 0; i < size; i++) {
+                                        knownInstances.set(i, null);
+                                    }
+                                    for (int i = 0; i < size; i++) {
+                                        knownInstances.add(null);
+                                    }
+                                    knownInstancesCount = 0;
+                                    for (Reference<Object> r : all) {
+                                        if (r == null) {
+                                            continue;
+                                        }
+                                        Object instance = r.get();
+                                        if (instance == null) {
+                                            continue;
+                                        }
+                                        hashPut(instance);
+                                    }
+                                }
+
                             }
                         }
 
@@ -442,7 +484,7 @@ final class MetaInfServicesLookup extends AbstractLookup {
                         // could see and return immediately.
                         object = o;
                     } catch (Exception ex) {
-                        LOGGER.log(Level.WARNING, null, ex);
+                        LOGGER.log(Level.WARNING, "Cannot create " + object, ex);
                         object = null;
                     }
                 }
@@ -462,5 +504,52 @@ final class MetaInfServicesLookup extends AbstractLookup {
         protected boolean creatorOf(Object obj) {
             return obj == object;
         }
+
+        private static void hashPut(Object o) {
+            Class<?> c = o.getClass();
+            int size = knownInstances.size();
+            int index = c.hashCode() % size;
+            for (int i = 0; i < size; i++) {
+                Reference<Object> ref = knownInstances.get(index);
+                Object obj = ref == null ? null : ref.get();
+                if (obj == null) {
+                    knownInstances.set(index, new WeakReference<Object>(o));
+                    knownInstancesCount++;
+                    break;
+                }
+                if (++index == size) {
+                    index = 0;
+                }
+            }
+        }
+
+        private static boolean findSharedClassObjectSkip;
+        private static Method findSharedClassObject;
+        /** Basically does c.newInstance(), however the method is complicated
+         * with a special behaviour for old and almost obsoleted NetBeans
+         * class: SharedClassObject.
+         */
+        private static Object createInstance(Class<?> c) throws InstantiationException, IllegalAccessException {
+            if (!findSharedClassObjectSkip) {
+                try {
+                    if (findSharedClassObject == null) {
+                        Class<?> sco;
+                        try {
+                            sco = Class.forName("org.openide.util.SharedClassObject"); // NOI18N
+                        } catch (ClassNotFoundException ex) {
+                            findSharedClassObjectSkip = true;
+                            return c.newInstance();
+                        }
+                        findSharedClassObject = sco.getMethod("findObject", Class.class, boolean.class);
+                    }
+                    if (findSharedClassObject.getReturnType().isAssignableFrom(c)) {
+                        return findSharedClassObject.invoke(null, c, true);
+                    }
+                } catch (Exception problem) {
+                    throw (InstantiationException)new InstantiationException(problem.getMessage()).initCause(problem);
+                }
+            }
+            return c.newInstance();
+        }
     }
 }
